function [ AsP, bsP, csP, bsPnConstr, csPnCoef, BRLAsP, BRLbsP, LimBRL, NBRL ] = ...
           brl_parse( AsP, bsP, csP, ptype, bsPnConstr, csPnCoef, nPol, nCoefPol, KsPflqrs, isEQForm, nPolNU, isT, isR, nOld, nNonzOld, dmax, ...
                      k, isBRL, IsBRLK1, BRLniLines, BRLNLines, BRLK1, BRLK2 )

% BRL_PARSE Parse the BRL constraints
%
%   This function deletes de lines introduced by the user, describing
%   the filters from the BRLs, in 'AsP' and 'bsP', after they were
%   copied to 'BRLAsP' and 'BRLbsP', respectively. 'NBRL' is the number
%   of BRLs in the POS3POLY problem and 'LimBRL' denote the limits
%   for each BRL in 'BRLAsP' and 'BRLbsP'.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 18.02.10-17.08.10

% cumsum for 'BRLniLines'
BRLniLinesCS = cumsum( BRLniLines );

% lines for the BRL, for the description
% of the filter
BRLAsP    = zeros( BRLNLines, csPnCoef );
BRLbsP    = zeros( BRLNLines, 1        );
% lines in 'AsP' and 'bsP' for the BRL,
% to be deleted
BRLFLines = zeros( BRLNLines, 1        );

% number of BRLs
NBRL = nnz( BRLniLines );
% limit lines for the BRLs in 'BRLAsP' and 'BRLbsP'
LimBRL = zeros( NBRL, 2 );
LimBRL( 1, 1 ) = 1;
for i = 1 : NBRL - 1
    LimBRL( i    , 2 ) = BRLniLinesCS( i );
    LimBRL( i + 1, 1 ) = LimBRL( i, 2 ) + 1;
end
LimBRL( NBRL, 2 ) = BRLniLinesCS( NBRL ); 

% if ~isEQForm
    for i = 1 : nPol
        if isBRL( i )
            
            % copy BRL lines from 'AsP' and 'bsP'
            BRLAsP   ( LimBRL( i, 1 ) : LimBRL( i, 2 ), : ) = AsP( ptype{ i }.brl.hrows, : );
            BRLbsP   ( LimBRL( i, 1 ) : LimBRL( i, 2 ), : ) = bsP( ptype{ i }.brl.hrows, : );
            
            % lines to be deleted in 'AsP' and 'bsP'
            BRLFLines( LimBRL( i, 1 ) : LimBRL( i, 2 ), : ) = reshape( ptype{ i }.brl.hrows, BRLniLines( i ), 1 );
        end
    end
    
    % eliminate lines describing the filters
    % from the BRLs in 'AsP' and 'bsP'
    AsP( BRLFLines, : ) = [ ];
    bsP( BRLFLines, : ) = [ ];
% end

% for BRLK1(i)>1 replace the scalar constrains with matrix ones
%
if IsBRLK1
    % if there is at least one matrix BRL
    
    % cum. sum. for nCoefPol
    nCoefPolCS = [ 0; cumsum( nCoefPol ) ] + KsPflqrs;
    
    for i = 1 : nPol
        if isBRL( i )
            if BRLK1( i ) > 1

                % no sparse basis

                % REPLACE the columns
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                % number of rows in current AsP
                AsPNrRows   = size( AsP, 1 );

                nCoefPolNEW = ncoefpol( ptype{ i }, nOld{ i }, nNonzOld{ i }, BRLK1( i ), isT( i ), isR( i ), dmax );

                % update number of variables
                csPnCoefDif = nCoefPolNEW - nCoefPol( i );
                csPnCoef    = csPnCoef + csPnCoefDif;
                % update number of constraints
                bsPnConstr  = bsPnConstr + csPnCoefDif;

                % copy the block of columns
                AsPBlkCol = AsP( :, nCoefPolCS( i ) + 1 : nCoefPolCS( i + 1 ) );
                csPBlkCol = csP(    nCoefPolCS( i ) + 1 : nCoefPolCS( i + 1 ) );

                % delete block of columns
                AsP( :, nCoefPolCS( i ) + 1 : nCoefPolCS( i + 1 ) ) = [ ];
                csP(    nCoefPolCS( i ) + 1 : nCoefPolCS( i + 1 ) ) = [ ];

                % introduce new block of columns (of zeros) for the new variables
                AsP = [ AsP( :, 1 : nCoefPolCS( i ) ), sparse( AsPNrRows, nCoefPolNEW ), AsP( :, nCoefPolCS( i ) + 1 : end ) ];
                csP = [ csP( :, 1 : nCoefPolCS( i ) ), sparse(         1, nCoefPolNEW ), csP(    nCoefPolCS( i ) + 1 : end ) ];

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % introduce the columns for the scalar coefficients in the matrix ones;
                % we actually introduce the first element (1,1) in the matrix coefficients
                %
                % number of variables in half a matrix
                BRLK1Half = sumn( BRLK1( i ) );
                % number of variables in a whole matrix
                BRLK1All  = BRLK1( i ) ^ 2;
                % coefficient for (0,...,0)
                AsP( :, nCoefPolCS( i ) + 1 ) = AsPBlkCol( :, 1 );
                csP(    nCoefPolCS( i ) + 1 ) = csPBlkCol(    1 );

                % we continue with the rest of the coefficients;
                % we start from the second scalar coefficient; the first one is always symmetric
                Col = nCoefPolCS( i ) + 1 + BRLK1Half;
                % we jump accord to weather the coefficient is symmetric (real pol.)
                % or not (trigonometric or hybrid)
                Jump = cf( ~isT( i ), BRLK1Half, BRLK1All );
                for j = 2 : nCoefPol( i )
                    AsP( :, Col ) = AsPBlkCol( :, j );
                    csP( :, Col ) = csPBlkCol( :, j );
                    Col = Col + Jump;
                end

                % ADD EQUALITY CONSTRAINTS
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                % add the rows
                AsP = mextend( 'ar', 'zpd', AsP, csPnCoefDif, 'd' );
                bsP = mextend( 'ar', 'zpd', bsP, csPnCoefDif, 'd' );

                % introduce the coefficient for the degree (0,...,0) with the element (1,1);
                % the coefficient is always symmetric because it's the first one
                Line = AsPNrRows + 1;
                Col  = nCoefPolCS( i ) + 2;
                % the first column of the symmetric matrix
                for k1 = 2 : BRLK1( i )
                    AsP( Line, Col ) = 1;
                    Line = Line + 1;
                    Col = Col + 1;
                end
                % the rest of the columns
                for k2 = 2 : BRLK1( i )
                    for k1 = k2 : BRLK1( i )
                        AsP( Line, Col ) = 1;
                        if k1 == k2
                            AsP( Line, nCoefPolCS( i ) + 1 ) = -1;
                        end
                        Line = Line + 1;
                        Col = Col + 1;
                    end
                end
                Col = Col + 1;
                % the rest of the degrees; rest of the matrix coefficients
                %
                if ~isT( i )
                    % real polynomial: symmetric coefficients

                    for k = 1 : nCoefPol( i ) - 1
                        % the first column of the symmetric matrix
                        for k1 = 2 : BRLK1( i )
                            AsP( Line, Col ) = 1;
                            Line = Line + 1;
                            Col = Col + 1;
                        end
                        % the rest of the columns
                        for k2 = 2 : BRLK1( i )
                            for k1 = k2 : BRLK1( i )
                                AsP( Line, Col ) = 1;
                                if k1 == k2
                                    AsP( Line, nCoefPolCS( i ) + BRLK1Half + ( k - 1 ) * BRLK1Half + 1 ) = -1;
                                end
                                Line = Line + 1;
                                Col = Col + 1;
                            end
                        end

                        Col = Col + 1;
                    end
                else
                    % trig. or hybrid pol.: nonsymmetric coefficients

                    for k = 1 : nCoefPol( i ) - 1
                        % the first column of the nonsymmetric matrix
                        for k1 = 2 : BRLK1( i )
                            AsP( Line, Col ) = 1;
                            Line = Line + 1;
                            Col  = Col  + 1;
                        end
                        % the rest of the columns
                        for k2 = 2 : BRLK1( i )
                            for k1 = 1 : BRLK1( i )
                                AsP( Line, Col ) = 1;
                                if k1 == k2
                                    AsP( Line, nCoefPolCS( i ) + BRLK1Half + ( k - 1 ) * BRLK1All + 1 ) = -1;
                                end
                                Line = Line + 1;
                                Col  = Col  + 1;
                            end
                        end

                        Col = Col + 1;
                    end
                end

                nCoefPolCS( i + 1 : nPol + 1 ) = nCoefPolCS( i + 1 : nPol + 1 ) + csPnCoefDif;
            end
        end
    end
end