function [ AsP, bsP, csP, ptype, nConstr, bsPnConstr, nCoef, Ks, KsPflqrs, BRLniLines, BRLNLines, BRLlenpol, BRLlencpol ] = ...
           brl_parse_eq( AsP, bsP, csP, nPolNU, nOld, ptype, k, KsPflqrs, isBRL, BRLK1, BRLK2, ...
                         isT, isR, nConstr, bsPnConstr, nCoef, Ks, ...
                         BRLniLines, BRLNLines )

% BRL_PARSE_EQ BRL parse for EQForm
%
%   This function move the filter from the BRL in equality form
%   in the free variables and adds the POS3POLY constraints for the
%   filter.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 02.11.11-11.11.11

% length of the polynomials from BRL
BRLlenpol = zeros( nPolNU, 1 );
% length of the causal polynomials from BRL
BRLlencpol = zeros( nPolNU, 1 );

% put columns for the filters in the BRL
Col = KsPflqrs;
for i = 1 : nPolNU    
    if isBRL( i )
        
        % causal polynomial
        N = lencpol( nOld{ i }, [ BRLK1( i ) BRLK2( i ) ] );
        
        % blocks that will be moved
        BRLBlockMove = AsP( :, Col + 1 : Col + N );  % AsP
        BRLblockMove = csP( Col + 1 : Col + N );     % csP
        % delete blocks
        AsP( :, Col + 1 : Col + N ) = [ ];
        csP( Col + 1 : Col + N ) = [ ];
        % insert blocks after KsPflqrs
        AsP = [ AsP( :, 1 : Ks.f ), BRLBlockMove, AsP( :, Ks.f + 1 : end ) ];
        csP = [ csP(    1 : Ks.f ), BRLblockMove, csP(    Ks.f + 1 : end ) ];
        
        % add the constraints for the filter
        % extending 'AsP' and 'bsP'
        AsP = mextend( 'ar', 'zpd', AsP, N, 'd' );
        bsP = mextend( 'ar', 'zpd', bsP, N, 'd' );        
        AsP( nConstr + 1 : end, Ks.f + 1 : Ks.f + N ) = speye( N );        
        
        ptype{ i }.brl.hrows = nConstr + 1 : nConstr + N;
        
        BRLniLines( i ) = length( ptype{ i }.brl.hrows );
        
        BRLNLines = BRLNLines + BRLniLines( i );
        
        Col = Col + N;
        
        % increase the number of free variables
        % and number of constraints
        Ks.f       = Ks.f       + N;
        KsPflqrs   = KsPflqrs   + N;
        nConstr    = nConstr    + N;
        bsPnConstr = bsPnConstr + N;
        nCoef      = nCoef      + N;
        
        BRLlencpol( i ) = N;
    end
    
    if isT( i ) && isR( i )
        % hybrid polynomial
        BRLlenpol( i ) = lenpol( nOld{ i }, k( i ), ptype{ i }.trigonometric, ptype{ i }.real );
    elseif isT( i )
        % trigonometric polynomial
        BRLlenpol( i ) = lenpol( nOld{ i }, k( i ), ptype{ i }.trigonometric, 0 );
    else
        % real polynomial
        BRLlenpol( i ) = lenpol( nOld{ i }, k( i ), 0, ptype{ i }.real );
    end
    
    Col = Col + BRLlenpol( i );
end