function [ As, bsP, Ks ] = brl_set( As, bsP, k, bsPnConstr, LimD, KsPflqrs, nPol, QKsize, nNonzProd1, ...
                                    isBRL, BRLAsP, BRLbsP, BRLniLines, BRLNLines, LimBRL, NBRL, BRLK2, ...
                                    isC, Ks )
 
% BRL_SET Set BRL constraints 
%
%   This function sets the constraints describing the filters
%   in the BRLs.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 18.02.10-31.03.10

% number of lines in As before introducing
% the BRL constraints for the filter
AsSize = bsPnConstr - BRLNLines;

% copy LimBRL before adding 'AsSize'
LimBRLOld = LimBRL;
% update LimBRL to start from the last row of 'As'
% to reflect the matrix 'As'
LimBRL = LimBRL + AsSize;

% total number of lines introduced for the BRL;
% NBRL is for the values of 1 introduced in
% the extended Gram matrix:
% 1) scalar case
% [ Q0 h
%   h' 1 ]
% 2) matrix case
% [ Q0 H
%   H' I_K2 ]
% number of elems. in HALF
% the identity matrix
BRLK22Half   = BRLK2 .* ( BRLK2 + 1 ) ./ 2;
% cum. sum.
BRLK22HalfCS = cumsum( BRLK22Half );
% nr. of lines introduced
% for the all the BRLs
BRLNTotalLines = BRLNLines + sum( BRLK22Half );

% extending 'As' and 'bsP'
As  = mextend( 'ar', 'zpd', As , BRLNTotalLines, 'd' );
bsP = mextend( 'ar', 'zpd', bsP, BRLNTotalLines, 'd' );

% add number of lines for the values of one (scalar case),
% OR for the identity matrices (matrix. coef. case),
% for the past BRLs
for i = 2 : NBRL
    LimBRL( i, : ) = LimBRL( i, : ) + BRLK22HalfCS( i - 1 );
end

for i = 1 : nPol
    if isBRL( i )
        
        % nr of columns for the coefs. of the filter
        % in the Gram matrix
        nNonzProd1K1i = nNonzProd1( i ) * k( i );
        
        % vector 1 : BRLK2(i)
        BRLK2iConsec = 1 : BRLK2( i );
        
        % indexes for the Kronecker product;
        % we seek to create smth. like
        % [ x x x, y y y, z z z, ... ]
        KronOnes = ones( 1, BRLK2( i ) );
        
        % values to be substracted in order to compute the indexes
        % for the values inside the matrix filter;
        % we seek to create smth like
        % [ -3 -2 -1, -3 -2 -1, -3 -2 -1, ... ]
        SubtractValuesMatFilt  = repmat( -BRLK2( i ) : 1 : -1, 1, nNonzProd1K1i );
        
        SubtractValuesMatIdent = repmat( -BRLK2( i ) : 1 : -1, 1, BRLK2( i )    );
        
        % add KsPflqrs variables in the BRL
        As    ( LimBRL   ( i, 1 ) : LimBRL   ( i, 2 ), 1 : KsPflqrs ) = ...
        BRLAsP( LimBRLOld( i, 1 ) : LimBRLOld( i, 2 ), 1 : KsPflqrs );
        
        % indexes for the columns inside the Gram matrix; 
        %
        % for the matrices from the coefs. of the filter
        nColumnsIndexesMatFilt  = 1 : nNonzProd1K1i;
        % for the identity matrix
        nColumnsIndexesMatIdent = nNonzProd1K1i + BRLK2iConsec;
        
        % indexes for the scalar values from matrix coefs. of the filter;
        % we seek to obtain smth. like
        % [ x-3 x-2 x-1, y-3 y-2 y-1, z-3 z-2 z-1, ... ]
        MatFiltCoefsIndexes = kron( QKsize( i ) * nColumnsIndexesMatFilt , KronOnes ) + SubtractValuesMatFilt ; 
        IdentFiltIndexes    = kron( QKsize( i ) * nColumnsIndexesMatIdent, KronOnes ) + SubtractValuesMatIdent; 
        
        % insert values in 'As';
        %
        % for the filter with scalar/matrix coefs.
        As( LimBRL   ( i, 1 ) : LimBRL   ( i, 2 ), LimD( i, 1 ) + MatFiltCoefsIndexes ) = -speye( BRLniLines( i ) );
        % for the value of half the identity matrix (value of one in the scalar case);
        % from IdentFiltIndexes we must delete the indexes for the values
        % above the diagonal of the identity matrix; (we introduce constraints 
        % only for half of the identity matrix)
        for col = 1 : BRLK2( i ) - 1
            % put zeros for the elements that we eliminate
            IdentFiltIndexes( BRLK2( i ) * col + ( 1 : col ) ) = zeros( col, 1 );
        end
        % remove zeros
        IdentFiltIndexes( ~IdentFiltIndexes ) = [ ];
        % introduce constraints for half of the identity matrix
        As( LimBRL( i, 2 ) + ( 1 : BRLK22Half( i ) ), LimD( i, 1 ) + IdentFiltIndexes ) =  speye( BRLK22Half( i ) );
        
        % insert values in 'bsP';
        %
        % for the filter
        bsP( LimBRL( i, 1 ) : LimBRL( i, 2 ) ) = BRLbsP( LimBRLOld( i, 1 ) : LimBRLOld( i, 2 ) );
        % for the values of one the diag. of the identity matrix;
        % the elements are smth like Lim + [ 1, 5+1, 5+4+1, 5+4+3+1, 5+4+3+2+1 ]
        bsP( LimBRL( i, 2 ) + [ 0 cumsum( BRLK2( i ) : -1 : 2 ) ] + KronOnes ) = KronOnes;
        
        % for the complex case add lines for the BRL in 'ycomplex' field
        if isC( i )
            Ks.ycomplex = [ Ks.ycomplex LimBRL( i, 1 ) : LimBRL( i, 2 ) LimBRL( i, 2 ) + ( 1 : BRLK22Half( i ) ) ];
        end
    end
end