function [ D, Line ] = coef_var_mat_multi_h_pd_sb( D, Line, LimD, K, nzi, nt, TList, HList, TVecUnq, HVecUnq, ...
                                                   degDnz, coefD, ncD, ncCSi, nDomPol, nDomPolCS, ...
                                                   nzsi, nzosi, ...
                                                   dmpIdxAll, isBRLi, BRLK2i, ...
                                                   SBNrDegsi, SBDegsAddSubi, SBPosDegsi, ...
                                                   SBDegsAddSubDomPol, SBPosDegsDomPol )

% COEF_VAR_MAT_MULTI_H_PD_SB Create SeDuMi matrix for a multivariate hybrid polynomial with pos. dom.
%
%   This function returns the D matrix, that we use
%   to transform the user's problem into a SeDuMi
%   problem.
%
%   The problem
%
%   min  csP' * wtx
%   s.t. AsP  * wtx = bsP
%
%   becomes
%
%   min  csP' * D * x
%   s.t. AsP  * D * x = bsP

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 16.10.09-13.07.10

% degree + 1
nzi1 = nzi + 1;

% degrees for pos. dom. + 1
nzsi1 = nzsi + 1;

nNonzLen = length( nzi );

% for the 1st
% real variable
nt1 = nt + 1;

% find indexes in the "list"
% of indexes;
% S0
%
% trig. part
idx = sparse( 1 + nDomPol, nNonzLen );
for degi = 1 : nt
    idx( 1, degi ) = find( TVecUnq == nzi( degi ) );
end
% real part
for degi = nt1 : nNonzLen
    idx( 1, degi ) = find( HVecUnq == nzi( degi ) );
end
%
% Si
%
% trig. part
for DomPol = 1 : nDomPol
    for degi = 1 : nt
        idx( DomPol + 1, degi ) = find( TVecUnq == nzsi( DomPol, degi ) );
    end
end
% real part
for DomPol = 1 : nDomPol
    for degi = nt1 : nNonzLen
        idx( DomPol + 1, degi ) = find( HVecUnq == nzsi( DomPol, degi ) );
    end
end

if ~isBRLi
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % no BRL, no need to extend the first Gram vectorized matrix
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if K == 1
        % scalar coefficients

        % for every n-tuple
        for nLine = 1 : SBNrDegsi

            % degree from the set of n-tuples
            deg = SBDegsAddSubi( nLine, : );

            % S0;
            % init with the first degree, which is
            % certainly for a trig. variable (nt < nNonzLen)
            % the "trig. degrees"
            if deg( 1 ) >= 0
                T = TList{ idx( 1, 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : )  ;
            else
                T = TList{ idx( 1, 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )' ;
            end

            % the rest of the trig. degrees (if any)
            for degi = 2 : nt
                if deg( degi ) >= 0
                    T = kron( TList{ idx( 1, degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : )  , T );
                else
                    T = kron( TList{ idx( 1, degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )' , T );
                end
            end

            % the real degrees
            for degi = nt1 : nNonzLen
                T = kron( HList{ idx( 1, degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ),  T );
            end       

            % we add the vectorized
            % matrix from "trace"
            D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( cmatc( T, SBPosDegsi ) )';

            % Si;
            % for every polynomial
            % from the domain
            for DomPol = 1 : nDomPol
                
                % index for degree
                Idx = nDomPolCS + DomPol;

                % for every coefficient
                for coefi = 1 : ncD( DomPol )

                    % temporary degree for current coef.
                    degDi = degDnz( ncCSi( DomPol ) + coefi, : );

                    if ~iszero( degDi( 1 : nt ) )
                        % nonzero degree in the trig. part of the pol. from the pos. dom.

                        % z^n, z^{-n} - trig. part    
                        % deg = degDi - degSi
                        % deg = degDi + degSi
                        % t^n - real part
                        % deg = degDi - degSi
                        degTM = deg - degDi;
                        degTP(   1 : nt       ) = deg( 1 : nt ) + degDi( 1 : nt );
                        degTP( nt1 : nNonzLen ) = degTM( nt1 : end );

                        % t^n - deg. from pol. from pos. dom.
                        if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )

                            % z^{-n} - deg. from pol. from pos. dom.
                            if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )      
                                
                                % check if degree is among the possible ones
                                I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                
                                if ~isempty( I )

                                    % init with the first trig. degree
                                    if degTM( 1 ) >= 0
                                        T = TList{ idx( DomPol + 1, 1 ) } ...
                                        (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )  ;
                                    else
                                        T = TList{ idx( DomPol + 1, degi ) } ...
                                        ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )' ;
                                    end

                                    % the rest of the trig. degrees (if any)
                                    for degi = 2 : nt
                                        if degTM( degi ) >= 0
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                        else
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                        end
                                    end

                                    % the real degrees
                                    for degi = nt1 : nNonzLen
                                        T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                        ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                    end                           

                                    % we add the vectorized
                                    % matrix from "trace"
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                    coefD( ncCSi( DomPol ) + coefi ) * vec( cmatc( T, SBPosDegsDomPol{ Idx } ) )';     
                                end
                            end    

                            % z^n - deg. from pol. from pos. dom.
                            if leqm( abs( degTP( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                
                                % check if degree is among the possible ones
                                I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );

                                if ~isempty( I )
                                    
                                    % init with the first trig. degree
                                    if degTP( 1 ) >= 0
                                        T = TList{ idx( DomPol + 1, 1 ) } ...
                                        (  degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )  ;
                                    else
                                        T = TList{ idx( DomPol + 1, 1 ) } ...
                                        ( -degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )' ;
                                    end

                                    % the rest of the trig. degrees (if any)
                                    for degi = 2 : nt
                                        if degTP( degi ) >= 0
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            (  degTP( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                        else
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            ( -degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                        end
                                    end

                                    % the real degrees
                                    for degi = nt1 : nNonzLen
                                        T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                        ( degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                    end

                                    % we add the vectorized
                                    % matrix from "trace"
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                    coefD( ncCSi( DomPol ) + coefi )' * vec( cmatc( T, SBPosDegsDomPol{ Idx } ) )';
                                end
                            end
                        end
                    else
                        % zero degree in the trig. part of the pol. from the pos. dom.

                        degTM = deg - degDi;

                        % t^n - deg. from pol. from pos. dom.
                        if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )

                             % z^{-n} - deg. from pol. from pos. dom.
                             if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                 
                                % check if degree is among the possible ones
                                I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                
                                if ~isempty( I )

                                    % init with the first trig. degree
                                    if degTM( 1 ) >= 0
                                        T = TList{ idx( DomPol + 1, 1 ) } ...
                                        (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )  ;
                                    else
                                        T = TList{ idx( DomPol + 1, 1 ) } ...
                                        ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )' ;
                                    end

                                    % the rest of the trig. degrees (if any)
                                    for degi = 2 : nt
                                        if degTM( degi ) >= 0
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                        else
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                        end
                                    end

                                    % the real degrees
                                    for degi = nt1 : nNonzLen
                                        T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                        ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                    end

                                    % we add the vectorized
                                    % matrix from "trace"
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                    coefD( ncCSi( DomPol ) + coefi ) * vec( cmatc( T, SBPosDegsDomPol{ Idx } ) )';
                                end
                             end
                        end
                    end
                end
            end   

            Line = Line + 1;
        end
    else
        % matrix coefficients

        % number of elem.
        % in matrix
        K2 = K ^ 2;

        % "half" of the
        % matrix
        hK = sumn( K );
        
        % for all the n-tuples
        for nLine = 1 : SBNrDegsi
            
            % degree from the set of n-tuples
            deg = SBDegsAddSubi( nLine, : );
            
            if iszero( deg( 1 : nt ) )
                
                % deg( nt+1 : end ) = (0,0,...,0);
                % the real part for the degree is zero =>
                % the coefficient is a symmetric matrix
                for elem = 1 : hK
                    
                    [ row, col ] = sind2sub( K, elem );
                    
                    % selector matrix
                    E = sparse( col, row, 1, K, K, 1 );
                    
                    % S0;
                    % init with the first degree, which is
                    % certainly for a trig. variable (nt < nNonzLen)
                    % the "trig. degrees"
                    if deg( 1 ) >= 0
                        T = TList{ idx( 1, 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ) , : );
                    else
                        T = TList{ idx( 1, 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 )', : );
                    end
                    
                    % the rest of the trig. degrees (if any)
                    for degi = 2 : nt
                        if deg( degi ) >= 0
                            T = kron( TList{ idx( 1, degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ) , T );
                        else
                            T = kron( TList{ idx( 1, degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )', T );
                        end
                    end
                    
                    % the real degrees
                    for degi = nt1 : nNonzLen
                        T = kron( HList{ idx( 1, degi ) }( deg( degi ) * nzi1( degi ) + 1 : ( deg( degi ) + 1 ) * nzi1( degi ), : ), T );
                    end
                    
                    % we add the vectorized
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( kron( cmatc( T, SBPosDegsi ), E ) )';
                    
                    % Si;
                    % for every polynomial
                    % from the domain
                    for DomPol = 1 : nDomPol
                        
                        % index for degree
                        Idx = nDomPolCS + DomPol;
                        
                        % for every coefficient
                        for coefi = 1 : ncD( DomPol )
                            
                            % temporary degree for current coef.
                            degDi = degDnz( ncCSi( DomPol ) + coefi, : );
                            
                            if ~iszero( degDi( 1 : nt ) )
                                % nonzero degree in the trig. part of the pol. from the pos. dom.
                                
                                % z^n, z^{-n} - trig. part
                                % deg = degDi - degSi
                                % deg = degDi + degSi
                                % t^n - real part
                                % deg = degDi - degSi
                                degTM = deg - degDi;
                                degTP(   1 : nt       ) = degDi(   1 : nt  );
                                degTP( nt1 : nNonzLen ) = degTM( nt1 : end );
                                
                                % t^n - deg. from pol. from pos. dom.
                                if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )
                                    
                                    % z^{-n} - deg. from pol. from pos. dom.
                                    if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTM( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, degi ) } ...
                                                    ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTM( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi ) * vec( kron( cmatc( T, SBPosDegsDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                    
                                    % z^n - deg. from pol. from pos. dom.
                                    if leqm( abs( degTP( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTP( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    ( -degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTP( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTP( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi )' * vec( kron( cmatc( T, SBDegsAddSubDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                end
                            else
                                % zero degree in the trig. part of the pol. from the pos. dom.
                                
                                degTM = deg - degDi;
                                
                                % t^n - deg. from pol. from pos. dom.
                                if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )
                                    
                                    % z^{-n} - deg. from pol. from pos. dom.
                                    if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTM( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTM( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi ) * vec( kron( cmatc( T, SBDegsAddSubDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                end
                            end
                        end
                    end
                    
                    Line = Line + 1;
                end
            else
                % the non-symmetric matrices      
                
                for elem = 1 : K2
                    
                    [ row, col ] = ind2sub( K, elem );
                    
                    % selector matrix
                    E = sparse( col, row, 1, K, K, 1 );
                    
                    % S0;
                    % init with the first degree, which is
                    % certainly for a trig. variable (nt < nNonzLen)
                    % the "trig. degrees"
                    if deg( 1 ) >= 0
                        T = TList{ idx( 1, 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : ) ;
                    else
                        T = TList{ idx( 1, 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )';
                    end
                    
                    % the rest of the trig. degrees (if any)
                    for degi = 2 : nt
                        if deg( degi ) >= 0
                            T = kron( TList{ idx( 1, degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : )  , T );
                        else
                            T = kron( TList{ idx( 1, degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )' , T );
                        end
                    end
                    
                    % the real degrees
                    for degi = nt1 : nNonzLen
                        T = kron( HList{ idx( 1, degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ),  T );
                    end
                    
                    % we add the vectorized
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( kron( cmatc( T, SBPosDegsi ), E ) )';
                    
                    % Si;
                    % for every polynomial
                    % from the domain
                    for DomPol = 1 : nDomPol
                        
                        % index for degree
                        Idx = nDomPolCS + DomPol;
                        
                        % for every coefficient
                        for coefi = 1 : ncD( DomPol )
                            
                            % temporary degree for current coef.
                            degDi = degDnz( ncCSi( DomPol ) + coefi, : );
                            
                            if ~iszero( degDi( 1 : nt ) )
                                % nonzero degree in the trig. part of the pol. from the pos. dom.
                                
                                % z^n, z^{-n} - trig. part
                                % deg = degDi - degSi
                                % deg = degDi + degSi
                                % t^n - real part
                                % % deg = degDi - degSi
                                degTM = deg - degDi;
                                degTP(   1 : nt       ) = deg( 1 : nt ) + degDi( 1 : nt );
                                degTP( nt1 : nNonzLen ) = degTM( nt1 : end );
                                
                                % t^n - deg. from pol. from pos. dom.
                                if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )
                                    
                                    % z^{-n} - deg. from pol. from pos. dom.
                                    if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTM( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, degi ) } ...
                                                    ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTM( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi ) * vec( kron( cmatc( T, SBPosDegsDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                    
                                    % z^n - deg. from pol. from pos. dom.
                                    if leqm( abs( degTP( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTP( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    ( -degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTP( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTP( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi )' * vec( kron( cmatc( T, SBDegsAddSubDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                end
                            else
                                % zero degree in the trig. part of the pol. from the pos. dom.
                                
                                degTM = deg - degDi;
                                
                                % t^n - deg. from pol. from pos. dom.
                                if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )
                                    
                                    % z^{-n} - deg. from pol. from pos. dom.
                                    if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTM( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTM( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi ) * vec( kron( cmatc( T, SBDegsAddSubDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                end
                            end
                        end
                    end
                    
                    Line = Line + 1;
                end
            end
        end
    end
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % BRL, must extend the first Gram vectorized matrix
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if K == 1
        % scalar coefficients

        % for every n-tuple
        for nLine = 1 : SBNrDegsi

            % degree from the set of n-tuples
            deg = SBDegsAddSubi( nLine, : );

            % S0;
            % init with the first degree, which is
            % certainly for a trig. variable (nt < nNonzLen)
            % the "trig. degrees"
            if deg( 1 ) >= 0
                T = TList{ idx( 1, 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : )  ;
            else
                T = TList{ idx( 1, 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )' ;
            end

            % the rest of the trig. degrees (if any)
            for degi = 2 : nt
                if deg( degi ) >= 0
                    T = kron( TList{ idx( 1, degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : )  , T );
                else
                    T = kron( TList{ idx( 1, degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )' , T );
                end
            end

            % the real degrees
            for degi = nt1 : nNonzLen
                T = kron( HList{ idx( 1, degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ),  T );
            end       

            % we add the vectorized
            % matrix from "trace"
            D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', cmatc( T, SBPosDegsi ), BRLK2i, [ 'r' 'd' ] ) )';

            % Si;
            % for every polynomial
            % from the domain
            for DomPol = 1 : nDomPol
                
                % index for degree
                Idx = nDomPolCS + DomPol;

                % for every coefficient
                for coefi = 1 : ncD( DomPol )

                    % temporary degree for current coef.
                    degDi = degDnz( ncCSi( DomPol ) + coefi, : );

                    if ~iszero( degDi( 1 : nt ) )
                        % nonzero degree in the trig. part of the pol. from the pos. dom.

                        % z^n, z^{-n} - trig. part    
                        % deg = degDi - degSi
                        % deg = degDi + degSi
                        % t^n - real part
                        % deg = degDi - degSi
                        degTM = deg - degDi;
                        degTP(   1 : nt       ) = deg( 1 : nt ) + degDi( 1 : nt );
                        degTP( nt1 : nNonzLen ) = degTM( nt1 : end );

                        % t^n - deg. from pol. from pos. dom.
                        if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )

                            % z^{-n} - deg. from pol. from pos. dom.
                            if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )    
                                
                                % check if degree is among the possible ones
                                I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                
                                if ~isempty( I )

                                    % init with the first trig. degree
                                    if degTM( 1 ) >= 0
                                        T = TList{ idx( DomPol + 1, 1 ) } ...
                                        (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )  ;
                                    else
                                        T = TList{ idx( DomPol + 1, degi ) } ...
                                        ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )' ;
                                    end

                                    % the rest of the trig. degrees (if any)
                                    for degi = 2 : nt
                                        if degTM( degi ) >= 0
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                        else
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                        end
                                    end

                                    % the real degrees
                                    for degi = nt1 : nNonzLen
                                        T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                        ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                    end                           

                                    % we add the vectorized
                                    % matrix from "trace"
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                    coefD( ncCSi( DomPol ) + coefi ) * vec( cmatc( T, SBDegsAddSubDomPol{ Idx } ) )';     
                                end
                            end    

                            % z^n - deg. from pol. from pos. dom.
                            if leqm( abs( degTP( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                
                                % check if degree is among the possible ones
                                I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                
                                if ~isempty( I )

                                    % init with the first trig. degree
                                    if degTP( 1 ) >= 0
                                        T = TList{ idx( DomPol + 1, 1 ) } ...
                                        (  degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )  ;
                                    else
                                        T = TList{ idx( DomPol + 1, 1 ) } ...
                                        ( -degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )' ;
                                    end

                                    % the rest of the trig. degrees (if any)
                                    for degi = 2 : nt
                                        if degTP( degi ) >= 0
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            (  degTP( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                        else
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            ( -degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                        end
                                    end

                                    % the real degrees
                                    for degi = nt1 : nNonzLen
                                        T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                        ( degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                    end

                                    % we add the vectorized
                                    % matrix from "trace"
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                    coefD( ncCSi( DomPol ) + coefi )' * vec( cmatc( T, SBDegsAddSubDomPol{ Idx } ) )';
                                end
                            end
                        end
                    else
                        % zero degree in the trig. part of the pol. from the pos. dom.

                        degTM = deg - degDi;

                        % t^n - deg. from pol. from pos. dom.
                        if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )

                             % z^{-n} - deg. from pol. from pos. dom.
                             if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                 
                                % check if degree is among the possible ones
                                I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                
                                if ~isempty( I )

                                    % init with the first trig. degree
                                    if degTM( 1 ) >= 0
                                        T = TList{ idx( DomPol + 1, 1 ) } ...
                                        (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )  ;
                                    else
                                        T = TList{ idx( DomPol + 1, 1 ) } ...
                                        ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )' ;
                                    end

                                    % the rest of the trig. degrees (if any)
                                    for degi = 2 : nt
                                        if degTM( degi ) >= 0
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                        else
                                            T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                            ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                        end
                                    end

                                    % the real degrees
                                    for degi = nt1 : nNonzLen
                                        T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                        ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                    end

                                    % we add the vectorized
                                    % matrix from "trace"
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                    D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                    coefD( ncCSi( DomPol ) + coefi ) * vec( cmatc( T, SBDegsAddSubDomPol{ Idx } ) )';
                                end
                             end
                        end
                    end
                end
            end   

            Line = Line + 1;
        end
    else
        % matrix coefficients

        % number of elem.
        % in matrix
        K2 = K ^ 2;

        % "half" of the
        % matrix
        hK = sumn( K );        
        
        % for all the n-tuples
        for nLine = 1 : SBNrDegsi
            
            % degree from the set of n-tuples
            deg = SBDegsAddSubi( nLine, : );
            
            if iszero( deg( 1 : nt ) )
                
                % deg( nt+1 : end ) = (0,0,...,0);
                % the real part for the degree is zero =>
                % the coefficient is a symmetric matrix
                for elem = 1 : hK
                    
                    [ row, col ] = sind2sub( K, elem );
                    
                    % selector matrix
                    E = sparse( col, row, 1, K, K, 1 );
                    
                    % S0;
                    % init with the first degree, which is
                    % certainly for a trig. variable (nt < nNonzLen)
                    % the "trig. degrees"
                    if deg( 1 ) >= 0
                        T = TList{ idx( 1, 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ) , : );
                    else
                        T = TList{ idx( 1, 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 )', : );
                    end
                    
                    % the rest of the trig. degrees (if any)
                    for degi = 2 : nt
                        if deg( degi ) >= 0
                            T = kron( TList{ idx( 1, degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ) , T );
                        else
                            T = kron( TList{ idx( 1, degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )', T );
                        end
                    end
                    
                    % the real degrees
                    for degi = nt1 : nNonzLen
                        T = kron( HList{ idx( 1, degi ) }( deg( degi ) * nzi1( degi ) + 1 : ( deg( degi ) + 1 ) * nzi1( degi ), : ), T );
                    end
                    
                    % we add the vectorized
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', kron( cmatc( T, SBPosDegsi ), E ), BRLK2i, [ 'r' 'd' ] ) )';
                    
                    % Si;
                    % for every polynomial
                    % from the domain
                    for DomPol = 1 : nDomPol
                        
                        % index for degree
                        Idx = nDomPolCS + DomPol;
                        
                        % for every coefficient
                        for coefi = 1 : ncD( DomPol )
                            
                            % temporary degree for current coef.
                            degDi = degDnz( ncCSi( DomPol ) + coefi, : );
                            
                            if ~iszero( degDi( 1 : nt ) )
                                % nonzero degree in the trig. part of the pol. from the pos. dom.
                                
                                % z^n, z^{-n} - trig. part
                                % deg = degDi - degSi
                                % deg = degDi + degSi
                                % t^n - real part
                                % deg = degDi - degSi
                                degTM = deg - degDi;
                                degTP(   1 : nt       ) = degDi(   1 : nt  );
                                degTP( nt1 : nNonzLen ) = degTM( nt1 : end );
                                
                                % t^n - deg. from pol. from pos. dom.
                                if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )
                                    
                                    % z^{-n} - deg. from pol. from pos. dom.
                                    if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTM( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, degi ) } ...
                                                    ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTM( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi ) * vec( kron( cmatc( T, SBDegsAddSubDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                    
                                    % z^n - deg. from pol. from pos. dom.
                                    if leqm( abs( degTP( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTP( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    ( -degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTP( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTP( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi )' * vec( kron( cmatc( T, SBDegsAddSubDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                end
                            else
                                % zero degree in the trig. part of the pol. from the pos. dom.
                                
                                degTM = deg - degDi;
                                
                                % t^n - deg. from pol. from pos. dom.
                                if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )
                                    
                                    % z^{-n} - deg. from pol. from pos. dom.
                                    if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTM( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTM( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi ) * vec( kron( cmatc( T, SBDegsAddSubDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                end
                            end
                        end
                    end
                    
                    Line = Line + 1;
                end
            else
                % the non-symmetric matrices      
                
                for elem = 1 : K2
                    
                    [ row, col ] = ind2sub( K, elem );
                    
                    % selector matrix
                    E = sparse( col, row, 1, K, K, 1 );
                    
                    % S0;
                    % init with the first degree, which is
                    % certainly for a trig. variable (nt < nNonzLen)
                    % the "trig. degrees"
                    if deg( 1 ) >= 0
                        T = TList{ idx( 1, 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : ) ;
                    else
                        T = TList{ idx( 1, 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )';
                    end
                    
                    % the rest of the trig. degrees (if any)
                    for degi = 2 : nt
                        if deg( degi ) >= 0
                            T = kron( TList{ idx( 1, degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : )  , T );
                        else
                            T = kron( TList{ idx( 1, degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )' , T );
                        end
                    end
                    
                    % the real degrees
                    for degi = nt1 : nNonzLen
                        T = kron( HList{ idx( 1, degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ),  T );
                    end
                    
                    % we add the vectorized
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', kron( cmatc( T, SBPosDegsi ), E ), BRLK2i, [ 'r' 'd' ] ) )';
                    
                    % Si;
                    % for every polynomial
                    % from the domain
                    for DomPol = 1 : nDomPol
                        
                        % index for degree
                        Idx = nDomPolCS + DomPol;
                        
                        % for every coefficient
                        for coefi = 1 : ncD( DomPol )
                            
                            % temporary degree for current coef.
                            degDi = degDnz( ncCSi( DomPol ) + coefi, : );
                            
                            if ~iszero( degDi( 1 : nt ) )
                                % nonzero degree in the trig. part of the pol. from the pos. dom.
                                
                                % z^n, z^{-n} - trig. part
                                % deg = degDi - degSi
                                % deg = degDi + degSi
                                % t^n - real part
                                % % deg = degDi - degSi
                                degTM = deg - degDi;
                                degTP(   1 : nt       ) = deg( 1 : nt ) + degDi( 1 : nt );
                                degTP( nt1 : nNonzLen ) = degTM( nt1 : end );
                                
                                % t^n - deg. from pol. from pos. dom.
                                if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )
                                    
                                    % z^{-n} - deg. from pol. from pos. dom.
                                    if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTM( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, degi ) } ...
                                                    ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTM( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi ) * vec( kron( cmatc( T, SBDegsAddSubDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                    
                                    % z^n - deg. from pol. from pos. dom.
                                    if leqm( abs( degTP( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTP( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    ( -degTP( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTP( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTP( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTP( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTP( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTP( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi )' * vec( kron( cmatc( T, SBDegsAddSubDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                end
                            else
                                % zero degree in the trig. part of the pol. from the pos. dom.
                                
                                degTM = deg - degDi;
                                
                                % t^n - deg. from pol. from pos. dom.
                                if geqm( degTM( nt1 : end ), SBDegsAddSubi( 1, nt1 : end ) ) && leqm( degTM( nt1 : end ), nzosi( DomPol, nt1 : end ) )
                                    
                                    % z^{-n} - deg. from pol. from pos. dom.
                                    if leqm( abs( degTM( 1 : nt ) ), nzosi( DomPol, 1 : nt ) )
                                        
                                        % check if degree is among the possible ones
                                        I = findvm( SBDegsAddSubDomPol{ Idx }', degTM );
                                        
                                        if ~isempty( I )
                                            
                                            % init with the first trig. degree
                                            if degTM( 1 ) >= 0
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    (  degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : (  degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : ) ;
                                            else
                                                T = TList{ idx( DomPol + 1, 1 ) } ...
                                                    ( -degTM( 1 ) * nzsi1( DomPol, 1 ) + 1 : ( -degTM( 1 ) + 1 ) * nzsi1( DomPol, 1 ), : )';
                                            end
                                            
                                            % the rest of the trig. degrees (if any)
                                            for degi = 2 : nt
                                                if degTM( degi ) >= 0
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        (  degTM( degi ) * nzsi1( DomPol, degi ) + 1 : (  degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )  , T );
                                                else
                                                    T = kron( TList{ idx( DomPol + 1, degi ) } ...
                                                        ( -degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( -degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : )' , T );
                                                end
                                            end
                                            
                                            % the real degrees
                                            for degi = nt1 : nNonzLen
                                                T = kron( HList{ idx( DomPol + 1, degi ) } ...
                                                    ( degTM( degi ) * nzsi1( DomPol, degi ) + 1 : ( degTM( degi ) + 1 ) * nzsi1( DomPol, degi ), : ),  T );
                                            end
                                            
                                            % we add the vectorized
                                            % matrix from "trace"
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) = ...
                                            D( Line, LimD( dmpIdxAll( DomPol, 1 ) ) : LimD( dmpIdxAll( DomPol, 2 ) ) ) + ...
                                            coefD( ncCSi( DomPol ) + coefi ) * vec( kron( cmatc( T, SBDegsAddSubDomPol{ Idx } ), E ) )';
                                        end
                                    end
                                end
                            end
                        end
                    end
                    
                    Line = Line + 1;
                end
            end
        end
    end
end