function [ isBRL, BRLK1, BRLK2, IsBRLK1 ] = brl_check2( ptype, nPol )

% BRL_CHECK2 Check for BRLs for cvx style
%
%   This function is used to check for BRLs in the
%   POS3POLY problem.
%
%   OUTPUTS:
%   
%   isBRL   - 0 - BRL, 1 - not BRL
%   BRKK1   - K1 
%   BRKK2   - K2 
%   IsBRLK1 - if any matrix BRL (if any BRLK1>1)

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 08.02.10-17.08.10

% check for BRL
isBRL = zeros( nPol, 1 );
% 'K1' & 'K2' field for BRL;
% init with (default) scalar case
BRLK1 = zeros( nPol, 1 );
BRLK2 = zeros( nPol, 1 );
for i = 1 : nPol
    
    % check for field 'brl'
    if isfield( ptype{ i }, 'brl' )
        
        % BRL type
        isBRL( i ) = 1;
        
        brlSize  = isfield( ptype{ i }.brl, 'hsize' );
        
        % check that 'hsize' exists
        assert( brlSize, 'Field ''hsize'' from field ''brl'' is missing.' );
        
        % check if 'hsize' is empty
        assert( ~isempty( ptype{ i }.brl.hsize ), 'Field ''hsize'' from field ''brl'' is empty.' );
        
        % check if 'hsize' is scalar or vector
        assert( isvector( ptype{ i }.brl.hsize ), 'Field ''hsize'' from field ''brl'' must be scalar or vector.' );
        
        % number of elements in 'hsize'
        hsizeLen = length( ptype{ i }.brl.hsize );
        assert( hsizeLen == 1 || hsizeLen == 2, 'Field ''hsize'' from field ''brl'' must be of one or two elements.' );
        
        if hsizeLen == 1
            % set  fields 'K1' & 'K2' 
            BRLK1( i ) = ptype{ i }.brl.hsize;
            BRLK2( i ) = ptype{ i }.brl.hsize;
        else
            % copy fields 'K1' & 'K2' 
            BRLK1( i ) = ptype{ i }.brl.hsize( 1 );
            BRLK2( i ) = ptype{ i }.brl.hsize( 2 );
        end
    end
end

% check if any matrix BRL
IsBRLK1 = cf( isBRL, nnz( BRLK1 - 1 ), 0 );