function [ D ] = coef_var_mat2( AsPnCoef, LimD, ptype, nPol, n, k, nOld, nNonz, nNonzOld, KsPflqrs, nVar, dmax, IsT, IsR, isT, isR, isC,...
                                isIVec, TList, HList, TVecUnq, HVecUnq, Qsize, QOldsize, ...
                                IsD, isBRL, BRLK2, ...
                                isSB, SBNrDegs, SBDegsAddSub, SBPosDegs, SBDegsAddSubDomPol, SBPosDegsDomPol, ...
                                isD, degDNonz, nDomPol, ncCS, nDomPolCS, ...
                                nNonzSi, nNonzOldSi  )

% COEF_VAR_MAT2 Coefficients to variables matrix
%
%   This function returns the D matrix, that we use
%   to transform the user's problem into a SeDuMi
%   problem.
%
%   The POS3POLY problem
%
%   min  csP' * wtx
%   s.t. AsP  * wtx = bsP
%
%   becomes the SeDuMi problem
%
%   min  csP' * D * x
%   s.t. AsP  * D * x = bsP
%
%   Thus, the D matrix is a transformation matrix 
%   between the variables of the POS3POLY problem 
%   and the ones of the SeDuMi problem.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 22.11.08-14.07.10

Line = KsPflqrs + 1;

D = sparse( AsPnCoef, nVar ); 

% coefficients for f,l,q,r,s
D( 1 : KsPflqrs, 1 : KsPflqrs ) = speye( KsPflqrs );

if dmax == 1
    % if the greatest number of variables is one
    % then we are in the univariate case
    
    [ D ] = coef_var_mat_uni2( D, Line, LimD, ptype, nPol, n, k, nOld, IsT, IsR, isC, ...
                               isIVec, TList, HList, TVecUnq, HVecUnq, ...
                               isBRL, BRLK2, ...
                               isSB, SBNrDegs, SBDegsAddSub, SBPosDegs, SBDegsAddSubDomPol, SBPosDegsDomPol );   
else
    % the multivariate case
    
    if ~IsD
        % no pos. dom.
        
        [ D ] = coef_var_mat_multi2( D, Line, LimD, ptype, nPol, k, nOld, nNonz, nNonzOld, IsT, IsR, isT, isR, isC, ...
                                     TList, HList, TVecUnq, HVecUnq, Qsize, QOldsize, ...
                                     IsD, isBRL, BRLK2, ...
                                     isSB, SBNrDegs, SBDegsAddSub, SBPosDegs );
    else
        % with pos. dom.
        
        [ D ] = coef_var_mat_multi2( D, Line, LimD, ptype, nPol, k, nOld, nNonz, nNonzOld, IsT, IsR, isT, isR, isC, ...
                                     TList, HList, TVecUnq, HVecUnq, Qsize, QOldsize, ...
                                     IsD, isBRL, BRLK2, ...
                                     isSB, SBNrDegs, SBDegsAddSub, SBPosDegs, SBDegsAddSubDomPol, SBPosDegsDomPol, ...
                                     isD, degDNonz, nDomPol, ncCS, nDomPolCS, ...
                                     nNonzSi, nNonzOldSi );
    end
end