function [ D ] = coef_var_mat_uni2( D, Line, LimD, ptype, nPol, n, k, nOld, IsT, IsR, isC, ...
                                    isIVec, TList, HList, TVecUnq, HVecUnq, ...
                                    isBRL, BRLK2, ...
                                    isSB, SBNrDegs, SBDegsAddSub, SBPosDegs, SBDegsAddSubDomPolInt, SBPosDegsDomPolInt )

% COEF_VAR_MAT_UNI2 Create SeDuMi matrix for the univariate case
%
%   This function returns the D matrix, that we use
%   to transform the user's problem into a SeDuMi
%   problem.
%
%   The problem
%
%   min  csP' * wtx
%   s.t. AsP  * wtx = bsP
%
%   becomes
%
%   min  csP' * D * x
%   s.t. AsP  * D * x = bsP

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 22.11.08-13.05.10                   

global SDMP_TIME FID VER;

% list of matrices
MList   = cf( IsT, TList,   HList   );
% vector of unique values
MVecUnq = cf( IsT, TVecUnq, HVecUnq );

% for every polynomial
for i = 1 : nPol
    
    % print n, k
    p3p_fprintf( FID, [ 'POS3POLY ' VER ': n: %d, k: %d, ' ], nOld{ i }, k( i ) );
    % print nt, nr
    if IsT
        p3p_fprintf( FID, 'nt: %d, ', 1 );
        p3p_fprintf( FID, 'nt: %d, ', 0 );
    else
        p3p_fprintf( FID, 'nr: %d, ', 0 );
        p3p_fprintf( FID, 'nr: %d, ', 1 );
    end
    % print isC, isIVec, isBRL
    p3p_fprintf( FID, 'isC: %d, isIVec: %d, isBRL: %d, isSB: %d, ', isC( i ), isIVec( i ), isBRL( i ), isSB( i ) );        
    
    if ~isSB( i )
        [ D, Line ] = coef_var_mat_uni_pol( i, D, Line, LimD( i, : ), ptype{ i }, n{ i }, nOld{ i }, k( i ), IsR, ...
                                            isIVec( i ), MList, MVecUnq, n{ i } + 1, nOld{ i } + 1, ...
                                            isBRL( i ), BRLK2( i ) );
    else
        [ D, Line ] = coef_var_mat_uni_pol_sb( i, D, Line, LimD( i, : ), ptype{ i }, n{ i }, nOld{ i }, k( i ), IsR, ...
                                               isIVec( i ), MList, MVecUnq, n{ i } + 1, nOld{ i } + 1, ...
                                               isBRL( i ), BRLK2( i ), ...
                                               SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i }, ...
                                               SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } );
    end
    
    SDMP_TIME = cputime - SDMP_TIME;
    [ h, m, s ] = sec2hour( SDMP_TIME );
    p3p_fprintf( FID, 'time: %2d:%2d:%2d\n' , h, m, s );
end