function [ D, Ks, nCoef, k, isT, isC, isBRL, BRLK1, BRLK2, IsBRLK1, nNonzProd1, nCoefPol, isnunion, nULen1, pNU, ptypeNU ] = dmat( p, ptype )

% DMAT The D matrix
%
%   This function computes the D matrix which
%   describes the "trace" equalities for the
%   positive polynomial.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 26.01.09-07.10.10

% start time
global SDMP_TIME;
SDMP_TIME = cputime;

% fill d, n, k variables
%
% number of elements in p{i}
Npi = length( p );

% check nr. of elem. in p
switch Npi
    case 1            
        % p{ i } == [ n ];
        d = 1;
        n = p;
        k = 1;     
    otherwise
        % Npi > 1

        % p{ i } == [ n k ];
        d = Npi - 1;
        n = p( 1 : end - 1 );
        k = p( end ); 
end    

% check 'ptype' field
if nargin == 2
    % check if the 'ptype' fields
    % are nonempty

    if isempty( ptype ) || ( ~isfield( ptype, 'trigonometric' ) && ~isfield( ptype, 'real' ) )
        % introduce default values
        ptype.trigonometric = d;
    end
else
    % If field 'ptype' does not exist then we create one with
    % the default values for the environment.
    
    % introduce default values     
    ptype.trigonometric = d;
end

% variable for complex coefficients
isC = 0;
if isfield( ptype, 'complex_coef' )
    % if we have 'true' complex_coef
    if ptype.complex_coef
        
        isC = 1;
    end
end

ptypec = cell( 1 );
nc     = cell( 1 );
ptypec{ 1 } = ptype;
nc{ 1 } = n;

% check type of polynomials
[ isT, isR ] = istisr( ptypec, 1 );

% check positivity on domains
[ ptypec, isI, isD, nDomPol, isnunion nULen1, pNU, ptypeNU ] = pos_check2( ptype, ptypec, p, n, d, isT, isR, isC );

% check if any BRL
[ isBRL, BRLK1, BRLK2, IsBRLK1 ] = brl_check2( ptypec, 1 );
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% change size of matrix coefficient for matrix BRL
if IsBRLK1
    % matrix BRL
    
    k = BRLK1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% check if the polynomials must use
% sparse basis
[ isSB, ISSB ] = spbasis_check( ptypec, 1 );

% size of the Q matrices, isT, isR, nOld and nNonz
[ Qsize, QKsize, Q2size, QOldsize, nc, nOld, nNonz, nNonzOld, ZeroDeg, nNonzProd1, ...
  SBNrDegs, SBDegsAddSub, SBPosDegs ] = ...
  qsize( 1, ptypec, d, nc, k, isT, isR, isBRL, BRLK2, isSB );

% total number of coefficients
% from the descriptions of the polynomials
[ nCoef, nCoefPol ] = ncoef( 0, ptypec, nOld, nNonzOld, k, 1, isT, isR, d, isBRL, BRLK1, isSB );

Ks.s = [ ];
Ks.scomplex = [ ];

% check if we have pos. int.
IsI = cf( isI, 1, 0 );

% check if we have
% pos. dom.
if isD
    % yes
    
    IsD = 1;
    % compute degrees for the pol. from the pos. dom. of each pol. and
    % the size for the Gram matrices
    [ nSi, QDKsize, QD2size, nNonzSi, nNonzOldSi, degDNonz, nDomPolCS, ncCS, SBDegsAddSubDomPol, SBPosDegsDomPol ] = ...
           posdom( 1, ptypec, nc, nOld, k, d, isT, isR, isD, nDomPol, ZeroDeg, isSB, SBNrDegs, SBDegsAddSub );
else
    % no
    
    IsD = 0;
end

% 'nvar' function call
if ~IsD
    % no pos. dom
    [ nVar, Ks, isIVec, TVecUnq, HVecUnq, TList, HList, IsT, IsR, SBNrDegsMonoInt, SBDegsAddSubDomPolInt, SBPosDegsDomPolInt ] = ...
      nvar( 0, Ks, ptypec, 1, nc, k, nOld, QKsize, Q2size, isT, isR, isC, isI, ...
            isD, ...
            ISSB, isSB, SBNrDegs, SBDegsAddSub );
else
    % with pos. dom.
    [ nVar, Ks, isIVec, TVecUnq, HVecUnq, TList, HList, IsT, IsR, SBNrDegsMonoInt, SBDegsAddSubDomPolInt, SBPosDegsDomPolInt ] = ...
      nvar( 0, Ks, ptypec, 1, nc, k, nOld, QKsize, Q2size, isT, isR, isC, isI, ...
            isD, ...
            ISSB, isSB, SBNrDegs, SBDegsAddSub, ...
            d, nDomPol, nDomPolCS, nSi, QDKsize, QD2size );
end

% LimD is a matrix that holds the
% bounds for the limits of the 
% coefficients of the variables of the
% i-th polynomial in D;
% LimD(i,1) & LimD(i,2) S0
% LimD(i,3) & LimD(i,4) S1 (for pos. on int.)
% LimD(i,3) & LimD(i,4) & ... Si (for pos. on dom.)
if ~IsD
    % no pos. dom.
    
    [ LimD ] = lim_mat( 1, nc, k, isIVec, Q2size, 0, d, IsI, IsD, ...
                        ISSB, isSB, SBNrDegsMonoInt );
    
    [ D ]    = coef_var_mat2( nCoef, LimD, ptypec, 1, nc, k, nOld, nNonz, nNonzOld, 0, nVar, d, IsT, IsR, isT, isR, isC, ...
                              isIVec, TList, HList, TVecUnq, HVecUnq, Qsize, QOldsize, ...
                              IsD, isBRL, BRLK2, ...
                              isSB, SBNrDegs, SBDegsAddSub, SBPosDegs, SBDegsAddSubDomPolInt, SBPosDegsDomPolInt );
else
    % with pos. dom.
    
    [ LimD ] = lim_mat( 1, nc, k, isIVec, Q2size, 0, d, IsI, IsD, ...
                        ISSB, isSB, SBNrDegsMonoInt, ...
                        isD, nDomPol, nDomPolCS, QD2size );
    
    [ D ]    = coef_var_mat2( nCoef, LimD, ptypec, 1, nc, k, nOld, nNonz, nNonzOld, 0, nVar, d, IsT, IsR, isT, isR, isC, ...
                              isIVec, TList, HList, TVecUnq, HVecUnq, Qsize, QOldsize, ...
                              IsD, isBRL, BRLK2, ...
                              isSB, SBNrDegs, SBDegsAddSub, SBPosDegs, SBDegsAddSubDomPol, SBPosDegsDomPol, ...
                              isD, degDNonz, nDomPol, ncCS, nDomPolCS, ...
                              nNonzSi, nNonzOldSi );
end