function ptype = dom_full2sparse2( ptype, d, isT, isR )

% DOM_FULL2SPARSE2 Full to sparse for the positivity domain
%
%   This function changes the description of the positivity
%   domain, from full to sparse.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.08.2010

% number of degrees
[ npols  ] = size  ( ptype.dom.deg, 1 );
% number of coefficients
[ ncoefs ] = length( ptype.dom.coef );

% zero degree
o = zeros( 1, d );
% number of coefficients for 
% the pol. from pos. dom.
nCoefPolPD  = zeros( npols, 1 );
% ntuples for the pol. from pos. dom.
nTuplePolPD = cell ( npols, 1 );
if isT && isR
    % hybrid pol.

    for pp = 1 : npols
        % check if the degrees for the pol. from pos. dom. are nonnegative
        assert( logical( leqm( o, ptype.dom.deg( pp, : ) ) ), ...
                'The degrees for the pol. from the pos. dom. must be nonnegative.' );

         nCoefPolPD( pp ) = nchp   ( ptype.dom.deg( pp, : ), ptype.trigonometric );
        nTuplePolPD{ pp } = ntupleh( ptype.dom.deg( pp, : ), nCoefPolPD( pp ), ptype.trigonometric );
    end
elseif isT
    % trig. pol.

    for pp = 1 : npols
        % check if the degrees for the pol. from pos. dom. are nonnegative
        assert( logical( leqm( o, ptype.dom.deg( pp, : ) ) ), ...
                'The degrees for the pol. from the pos. dom. must be nonnegative.' );

         nCoefPolPD( pp ) = nch   ( ptype.dom.deg( pp, : ) );
        nTuplePolPD{ pp } = ntuple( ptype.dom.deg( pp, : ), nCoefPolPD( pp ), 1 );
    end
else
    % real pol.

    for pp = 1 : npols
        % check if the degrees for the pol. from pos. dom. are nonnegative
        assert( logical( leqm( o, ptype.dom.deg( pp, : ) ) ), ...
                'The degrees for the pol. from the pos. dom. must be nonnegative.' );

         nCoefPolPD( pp ) = prod  ( ptype.dom.deg( pp, : ) + 1 );
        nTuplePolPD{ pp } = ntuple( ptype.dom.deg( pp, : ), nCoefPolPD( pp ), 0 );
    end
end 

% check if the number of coefs. for the pols. from the pos. dom. are correct
assert( sum( nCoefPolPD ) == ncoefs, ...
        'The number of coefficients for all the polynomials describing the positivity domain is incorrect.' );  

% initialize 'nc'
ptype.dom.nc = zeros( 1, npols );
% cum. sum. for the number of coefficients
nCoefPolPDCS = [ 0; cumsum( nCoefPolPD ) ];
% initialize the degrees for all the pol. from pos. dom.
ptype.dom.deg = zeros( nCoefPolPDCS( end ), d );
for pp = 1 : npols

    % 'nc' for the p-th polynomial
    ptype.dom.nc( pp ) = nnz( ptype.dom.coef( nCoefPolPDCS( pp ) + 1 : nCoefPolPDCS( pp + 1 ) ) );
    % the degrees for the p-th polynomial
    ptype.dom.deg( nCoefPolPDCS( pp ) + 1 : nCoefPolPDCS( pp + 1 ), : ) = nTuplePolPD{ pp };
end
% remove the zero coefficients and their associated degrees
CoefsRem = ( ptype.dom.coef == 0 );
ptype.dom.coef( CoefsRem    ) = [ ];
ptype.dom.deg ( CoefsRem, : ) = [ ];