function [ ptypec, isI, isD, nDomPol, isnunion, nULen1, pNU, ptypeNU ] = pos_check2( ptype, ptypec, p, n, d, isT, isR, isC )

% POS_CHECK2 Check polynomials for positivity
%
%   This function computes the preliminary variables
%   for handling the positivity of polynomials.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.08.2010-16.10.2010

% variables for positivity on interval(s) or
% for positivity on domains;
% pos. dom. = positivity domain;
% isI, isD
isI = 0;
isD = 0;
%
% number of total coefficients for
% the positivy polynomials from the 
% pos. dom. for the polynomial;
% sumNC
%
% number of pol. for the pos. dom.
% of the polynomial;
% nDomPol
%
nDomPol = 0;
% init 'nunion'
nULen1  = [ ];
pNU     = [ ];
ptypeNU = [ ];
isnunion = 0;

if isfield( ptype, 'int' )
    % positivity on interval(s)

    % pos. int. and pos. dom. are 
    % mutually exclusive for a pol.;
    % if not - error
    assert( ~isfield( ptype, 'dom' ), 'Fields ''int'' and ''dom'' are mutually exclusive.' );

    % length of field 'int'
    nint = length( ptype.int );
    
    % check if 'nunion' exists and is non-empty
    if nint > 4
        isnunion = 1;
    elseif nint == 4
        % in order to have union of intervals we must eliminate three cases:
        % 1. trig pol complex coef: [ -pi, a ] U [ b, pi ]
        % 2. trig pol real coef   : [ 0, a ] U [ b, pi ]
        % 3. real pol even degree : ( -Inf, a ] U [ b, Inf )
        if ~( isT &&         isC && ptype.int( 1 ) ==  -pi && ptype.int( 4 ) ==  pi ) && ...
           ~( isT &&        ~isC && ptype.int( 1 ) ==    0 && ptype.int( 4 ) ==  pi ) && ...
           ~( isR && iseven( n ) && ptype.int( 1 ) == -Inf && ptype.int( 4 ) == Inf )
            
            isnunion = 1;
        end
    end
    
    if ~isnunion
        % no 'nunion'
        
        % check length of 'int';
        % must be two or four
        if  nint == 2 || nint == 4
            isI = 1;
        else
            error( 'Field ''int'' must be a vector of two or four elements.' );
        end
    else
        % 'nunion' style; we need to introduce new polynomials
        
        % check if 'nint' is even
        assert( logical( iseven( nint ) ), 'The number of elements of the vector ''int'' must be even.' );
        
        % activate interval positivity
        isI = 1;
        
        % total number of intervals
        nULen = nint / 2;
        % number of coefficients to add
        nULen1 = nULen - 1;
        % added polynomials
        pNU     = cell( nULen1, 1 );
        ptypeNU = cell( nULen1, 1 );
        
        for pp = 1 : nULen1
            
            % update the 'p'  and 'ptype' structures
            pNU    { pp } = p    ;
            ptypeNU{ pp } = ptype;
            
            % update field 'int'
            ptypeNU{ pp }.int = ptype.int( 2 + ( pp - 1 ) * 2 + 1 : 2 + pp * 2 );
        end
        
        % update 'int' for the first polynomial
        ptypec{ 1 }.int = ptype.int( 1 : 2 );
    end
elseif isfield( ptype, 'dom' )
    % positivity on domain

    % check if field 'deg' exists
    assert( isfield( ptype.dom, 'deg'  ), 'Field ''deg'' is missing from field ''dom''.'  );
    % check if field 'coef' exists
    assert( isfield( ptype.dom, 'coef' ), 'Field ''coef'' is missing from field ''dom''.' );
    % check if field 'deg' is empty
    assert( ~isempty( ptype.dom.deg ) , 'Field ''deg'' from field ''dom'' is empty.'  );
    % check if field 'coef' is empty
    assert( ~isempty( ptype.dom.coef ), 'Field ''coef'' from field ''dom'' is empty.' );
    
    % check if field 'nc' exists and is non-empty;
    % ncStyle = 1 uses deg, dom and nc; a sparse representation
    % of the polynomial from the pos. dom.
    % ncStyle = 2 used only deg and dom; a full description
    % of the polynomial from the pos. dom.
    isfieldNC = isfield( ptype.dom, 'nc' );
    if isfieldNC
        isemptyNC = isempty( ptype.dom.nc );
        if isemptyNC
            % empty 'nc', fullstyle
            ncStyle = 2;
        else
            % non-empty 'nc', sparse style
            ncStyle = 1;
        end
    else
        % no 'nc', full style
        ncStyle = 2;
    end
    % check if 'nunion' exists and is non-empty
    if isfield( ptype.dom, 'nunion' )
        if ~isempty( ptype.dom.nunion )
            % non-empty 'nunion' field;

            % number of domains in the union
            nULen = length( ptype.dom.nunion );
            if nULen > 1
                % more than one domain
                isnunion = 1;
            end
        end
    end
    
    if ncStyle == 1
        % sparse description of the pol. from pos. dom.
        
        if ~isnunion
            % no 'nunion'
            
            % number of degrees
            [ ndegs  ] = size  ( ptype.dom.deg, 1 );
            % number of coefficients
            [ ncoefs ] = length( ptype.dom.coef );

            % check the number of polynomials for 
            % 'deg' and 'coef' to be the same
            assert( ndegs == ncoefs, 'Nr. of deg. and coef. for the polynomial(s) defining the positivity domain must be equal.' );
            % check if nr. coefs. from 'coefs' equal
            % nr. of coefs. from 'nc' (sum(nc))
            sumNC = sum( ptype.dom.nc ); 
            assert( ndegs == sumNC, 'Number of coefficients from ''coef'' and ''nc'' must be equal.' );

            % check if the degrees from the pos. dom. are
            % not greater than the degree of the pos. pol
            for degi = 1 : ndegs
                assert( logical( leqm( ptype.dom.deg( degi, : ), n ) ), ...
                        'The degrees from the pos. dom. cannot be (elementwise) greater than the degree of the pos. pol.' );
            end
        else
            % 'nunion' style; we need to introduce new polynomials
            
            % check if sum of 'nunion' is equal to the length of 'nc'
            assert( sum( ptype.dom.nunion ) == length( ptype.dom.nc ), ...
                    'The sum of the elements in ''nunion'' must be equal to the number of elements in ''nc''.' );

            % number of coefficients to add
            nULen1 = nULen - 1;
            % added polynomials
            pNU     = cell( nULen1, 1 );
            ptypeNU = cell( nULen1, 1 );

            % number of coefficients for every pol.
            ncNU = zeros( nULen, 1 );
            % number of coefs for the 1st pol., which is the original one
            ncNU( 1 ) = sum( ptype.dom.nc( 1 : ptype.dom.nunion( 1 ) ) );
            % cum. sum. for 'nunion'
            nunionCS = [ 0; cumsum( reshape( ptype.dom.nunion, nULen, 1 ) ) ];
            % add the structure for the added polynomials
            for pp = 1 : nULen1

                % update the 'p' structure
                pNU    { pp } = p    ;
                ptypeNU{ pp } = ptype;

                % 'nc'
                ptypeNU{ pp }.dom.nc = ptype.dom.nc( nunionCS( pp + 1 ) + 1 : nunionCS( pp + 2 ) );
                % number of coefficients for the pols. to be added
                ncNU( pp + 1 ) = sum( ptypeNU{ pp }.dom.nc );
            end
            % cum. sum.
            ncNUCS = [ 0; cumsum( ncNU ) ];
            % 'deg' and 'coef'
            for pp = 1 : nULen1

                ptypeNU{ pp }.dom.deg  = ptype.dom.deg ( ncNUCS( pp + 1 ) + 1 : ncNUCS( pp + 2 ), : );
                ptypeNU{ pp }.dom.coef = ptype.dom.coef( ncNUCS( pp + 1 ) + 1 : ncNUCS( pp + 2 )    );
            end
            % update the first polynomial: 'nc', 'deg' and 'coef'
            ptypec{ 1 }.dom.nc   = ptype.dom.nc  ( nunionCS( 1 ) + 1 : nunionCS( 2 )    );
            ptypec{ 1 }.dom.deg  = ptype.dom.deg ( ncNUCS  ( 1 ) + 1 : ncNUCS  ( 2 ), : );
            ptypec{ 1 }.dom.coef = ptype.dom.coef( ncNUCS  ( 1 ) + 1 : ncNUCS  ( 2 )    );      

            % remove 'nunion' field
            for pp = 1 : nULen1
                ptypeNU{ pp }.dom = rmfield( ptypeNU{ pp }.dom, 'nunion' );
            end
        end
    else
        % full description of the pol. from pos. dom.
        
        if ~isnunion
            % no 'nunion'
            
            % full to sparse for 'dom'
            ptypec{ 1 } = dom_full2sparse2( ptype, d, isT, isR );
        else
            % 'nunion' style; we need to introduce new polynomials
            
            % number of polynomials in the reunion of domains
            [ npols  ] = size  ( ptype.dom.deg, 1 );
            % number of coefficients
            [ ncoefs ] = length( ptype.dom.coef );

            % check if the number of coefs. for the pols. from the pos. dom. are correct
            assert( sum( ptype.dom.nunion ) == npols, ...
                    'Inconsistency between the number of degrees of polynomials and the number of polynomials in ''nunion''.' );  

            % number of coefficients to add
            nULen1 = nULen - 1;
            % added polynomials
            pNU     = cell( nULen1, 1 );
            ptypeNU = cell( nULen1, 1 );

            % zero degree
            o = zeros( 1, d );
            % number of coefficients for 
            % the pol. from pos. dom.
            nCoefPolPD  = zeros( npols, 1 );
            if isT && isR
                % hybrid pol.

                for pp = 1 : npols

                    % check if the degrees for the pol. from pos. dom. are nonnegative
                    assert( logical( leqm( o, ptype.dom.deg( pp, : ) ) ), ...
                            'The degrees for the pol. from the pos. dom. must be nonnegative.' );
                    nCoefPolPD( pp ) = nchp   ( ptype.dom.deg( pp, : ), ptype.trigonometric );
                end
            elseif isT
                % trig. pol.

                for pp = 1 : npols

                    % check if the degrees for the pol. from pos. dom. are nonnegative
                    assert( logical( leqm( o, ptype.dom.deg( pp, : ) ) ), ...
                            'The degrees for the pol. from the pos. dom. must be nonnegative.' );
                    nCoefPolPD( pp ) = nch   ( ptype.dom.deg( pp, : ) );
                end
            else
                % real pol.

                for pp = 1 : npols

                    % check if the degrees for the pol. from pos. dom. are nonnegative
                    assert( logical( leqm( o, ptype.dom.deg( pp, : ) ) ), ...
                            'The degrees for the pol. from the pos. dom. must be nonnegative.' );
                    nCoefPolPD( pp ) = prod  ( ptype.dom.deg( pp, : ) + 1 );
                end
            end 

            % check if the number of coefs. for the pols. from the pos. dom. are correct
            assert( sum( nCoefPolPD ) == ncoefs, ...
                    'The number of coefficients for all the polynomials describing the positivity domain is incorrect.' );  

            % cum. sum. for 'nunion'
            nunionCS = [ 0; cumsum( reshape( ptype.dom.nunion, nULen, 1 ) ) ];
            % cum. sum. for 'nunion'
            nCoefPolPDCS = [ 0; cumsum( nCoefPolPD ) ];
            % add the structure for the added polynomials
            for pp = 1 : nULen1

                % update the 'p' structure
                pNU    { pp } = p    ;
                ptypeNU{ pp } = ptype;
            end
            % 'deg' and 'coef'
            for pp = 1 : nULen1

                % degrees from the list of degrees in 'deg'
                ptypeNU{ pp }.dom.deg  = ptype.dom.deg  ...
                                           (               nunionCS( pp + 1 ) + 1       :               nunionCS( pp + 2 ), :    );
                % extract the coefficients
                ptypeNU{ pp }.dom.coef = ptype.dom.coef ...
                                           ( nCoefPolPDCS( nunionCS( pp + 1 ) + 1 ) + 1 : nCoefPolPDCS( nunionCS( pp + 2 ) + 1 ) );
            end
            % update the first (original) polynomial: 'deg' and 'coef'
            ptypec{ 1 }.dom.deg  = ptype.dom.deg (               nunionCS( 1 ) + 1       :               nunionCS( 2 ), :    );
            ptypec{ 1 }.dom.coef = ptype.dom.coef( nCoefPolPDCS( nunionCS( 1 ) + 1 ) + 1 : nCoefPolPDCS( nunionCS( 2 ) + 1 ) );

            % full to sparse for dom for the original polynomial
            ptypec{ 1 } = dom_full2sparse2( ptypec{ 1 }, d, isT, isR );
            for pp = 1 : nULen1

                % full to sparse for dom for the added polynomials
                ptypeNU{ pp } = dom_full2sparse2( ptypeNU{ pp }, d, isT, isR );
            end
            
            % remove 'nunion' field
            for pp = 1 : nULen1
                ptypeNU{ pp }.dom = rmfield( ptypeNU{ pp }.dom, 'nunion' );
            end
        end
    end

    % nr of pol. in the pos. dom.;
    % (numbers of number of coefs.)
    nDomPol = length( ptypec{ 1 }.dom.nc );

    % polynomial positive
    % on domain
    isD = 1;
end