function [ SBNrDegsMonoi, SBDegsAddSubDomPoli, SBPosDegsDomPoli ] = ...
           getspbasisint( ni, nOldi, nNonzSi, nNonzOldSi, isT, isR, ncD, degD, SBNrDegsi, SBDegsAddSubi )

% POSDOM Positivity domain
%
%   This function computes the variables needed to describe
%   the positivity domains of the multivariable polynomials.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 04.06.09-01.07.10

% eliminate nNonzSi warning
if nNonzSi
    
end

% copy the degrees for the first pol. (S0) to the pol. from the pos. dom.
SBDegsAddSubDomPoli = SBDegsAddSubi;

% index for the degrees to eliminate
DegsRem = zeros( SBNrDegsi, 1 );
if isT
    % trigonometric polynomial
    
    % degree (0,0..,0);
    
    % for every coefficient
    for coefi = 1 : ncD
        
        % temporary degree for current coef.
        degDi = degD( coefi, : );
        
        if ~iszero( degDi )
            
            % z^n; deg = degDi - degSi
            degTP =  degDi;
            % z^{-n}; deg = degDi + degSi
            degTM = -degDi;
            
            % z^{-n} - deg. from pol. from pos. dom.
            if ~leqm( abs( degTM ), nNonzOldSi ) && ~leqm( abs( degTP ), nNonzOldSi )
                DegsRem( 1 ) = 1;
            end
        end
    end
    
    % the rest of the degrees
    for j = 2 : SBNrDegsi
        
        % for every coefficient
        for coefi = 1 : ncD
            
            % temporary degree for current coef.
            degDi = degD( coefi, : );
            
            if ~iszero( degDi )
                
                % z^n; deg = degDi - degSi
                degTP = SBDegsAddSubDomPoli( j, : ) + degDi;
                % z^{-n}; deg = degDi + degSi
                degTM = SBDegsAddSubDomPoli( j, : ) - degDi;
                
                % z^{-n} - deg. from pol. from pos. dom.
                if ~leqm( abs( degTM ), nNonzOldSi ) && ~leqm( abs( degTP ), nNonzOldSi )
                    DegsRem( j ) = j;
                end
            else
                if ~leqm( abs( SBDegsAddSubDomPoli( j, : ) ), nNonzOldSi )
                    DegsRem( j ) = j;
                end
            end
        end
    end
else
    % real polynomial
    
    for j = 1 : SBNrDegsi
        
        % for every coefficient
        for coefi = 1 : ncD
            
            Deg = 0;
            % temporary degree for current coef.
            degDi = degD( coefi, : );
            
            % t^n; deg = degDi + degSi
            degTM = SBDegsAddSubDomPoli( j, : ) - degDi;
            
            % t^n - deg. from pol. from pos. dom.
            if ( geqm( degTM, 0 ) && leqm( degTM, nNonzOldSi ) )
                Deg = 1;
                break;            
            end
        end
        
        if ~Deg 
            DegsRem( j ) = j;
        end
    end
end

% eliminate degrees that cannot be made by the pol. from the pos. dom.
SBDegsAddSubDomPoli( DegsRem( DegsRem ~= 0 ), : ) = [ ];

if ~leqm( max( SBDegsAddSubDomPoli ), nOldi )
    error( [ 'The available degrees for which we write the equalities are not consistent with the degree of the '...
        'SOS polynomial multiplied with the polynomial from the positivity domain. Increase the degree of the main polynomial.' ] );
end

% degrees to generate the degrees for the Si polynomials
if isT
    % trigonometric polynomial
    spdegsDomPol = sparsedegs( SBDegsAddSubDomPoli, ni, nOldi, 1, 0 );
else
    % real polynomial
    spdegsDomPol = sparsedegs( SBDegsAddSubDomPoli, ni, nOldi, 0, 1 );
end

if isT
    % trigonometric
    
    % eliminate negative degrees
    spdegsDomPol = spdegs2spbasis( spdegsDomPol );
    
    [ SBDegsAddSubDomPoli, SBNrDegs, SBPosDegsDomPoli, SBNrDegsMonoi ] = ...
    genspbasis( ni, nOldi, 1, isT, isR, prod( ni + 1 ), spdegsDomPol, 2 );
else
    % real polynomial
    
    [ SBDegsAddSubDomPoli, SBNrDegs, SBPosDegsDomPoli, SBNrDegsMonoi ] = ...
    genspbasis( ni, nOldi, 1, isT, isR, prod( ni + 1 ), spdegsDomPol, 2 );
    
end