function [ AsP, bsP, csP, Ks, ptype, nCoef, nConstr, KsPflqrs, AsPnCoef, bsPnConstr, csPnCoef, nConstrNU, BRLniLines, BRLNLines ] = ...
           ineq_data( AsP, bsP, csP, Ks, ptype, n, nOld, nNonzOld, nPol, k, dmax, nCoef, nConstr, bsPnConstr, KsPflqrs, ...
                      nCoefPol, IsBRL, isBRL, BRLniLines, isT, isR, isNU, BRLK1, BRLK2, isSB, isnunion, ...
                      nPolNU, nULen1, nConstrNU, nConstrBeforeNU, nCoefBeforeNU )

% INEQ_DATA Data of the inequality form
%
%   This function transforms the INEQuality (dual) form into the
%   EQuality (primal) form.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 16.11.11

% check if we have BRLs
if ~IsBRL
    % no BRL
    
    if ~isNU
        % no nunion
        
        % Eye is the identity matrix formed with
        % identity matrices for each polynomial
        Eye = speye( nCoef );
        
        % check if we can concatenate matrix 'AsP' with 'Eye'
        assert( nConstr == nCoef, 'Inconsistency between the size of the matrix ''AsP'' and the types of the polynomials.' );
        
        % create backup matrices
        bsPOld = bsP;
        % modify matrices
        AsP = [ AsP Eye ];
        bsP = csP;
        csP = [ -bsPOld sparse( 1, nCoef ) ];
        % add Ks.f for the old dual variable 'y'
        if isfield( Ks, 'f' )
            Ks.f = Ks.f + bsPnConstr;
        else
            Ks.f = bsPnConstr;
        end
        % update 'nCoef'
        nCoef = nCoef + bsPnConstr;
        % update KsPflqrs
        KsPflqrs = KsPflqrs + bsPnConstr;
        
        % update sizes for data
        bsPnConstr = length( bsP );
        csPnCoef   = length( csP );
        [ nConstr, AsPnCoef ] = size( AsP );
    else
        % at least one nunion
        
        % number of coefficients for a polynomial
        M = zeros( nPolNU );
        
        % add columns for all the polynomials Si
        AsP = mextend( 'ac', 'zpd', AsP, nCoef, 'r' );
        
        % append identity matrix
        EyeMod = speye( nCoefBeforeNU );
        nCoefPolCS = [ 0; cumsum( nCoefPol( 1 : nPolNU) ) ];
        
        % for each polynomial
        for i = 1 : nPolNU
            
            % compute length for the polynomial
            if isT( i ) && isR( i )
                % hybrid polynomial
                M( i ) = lenpol( nOld{ i }, k( i ), ptype{ i }.trigonometric, ptype{ i }.real );
            elseif isT( i )
                % trigonometric polynomial
                M( i ) = lenpol( nOld{ i }, k( i ), ptype{ i }.trigonometric, 0 );
            else
                % real polynomial
                M( i ) = lenpol( nOld{ i }, k( i ), 0, ptype{ i }.real );
            end
                      
            if isnunion( i )
                ConstrVarAdd = nULen1( i ) * M( i );
                
                EyeMod = mextend( '2D', 'zpd', EyeMod, [ ConstrVarAdd ConstrVarAdd ], [ 'd' 'r' ] );
                
                EyeMod( nConstrBeforeNU + 1 : end, nCoefBeforeNU + 1 : end ) = speye( ConstrVarAdd );
                EyeMod( nConstrBeforeNU + 1 : end, nCoefPolCS( i ) + 1 : nCoefPolCS( i + 1 ) ) = -repmat( speye( M( i ) ), nULen1( i ), 1 );
                
                nCoefBeforeNU = nCoefBeforeNU + ConstrVarAdd;
                nConstrBeforeNU  = nConstrBeforeNU  + ConstrVarAdd;
            end
            
        end
        
        % create backup matrices
        bsPOld = bsP;
        % modify matrices
        AsP( :, bsPnConstr + 1 : end ) = EyeMod;
        bsP = csP;
        csP = [ -bsPOld sparse( 1, nCoef ) ];
        % add Ks.f for the old dual variable 'y'
        if isfield( Ks, 'f' )
            Ks.f = Ks.f + bsPnConstr;
        else
            Ks.f = bsPnConstr;
        end
        % update 'nCoef'
        nCoef = nCoef + bsPnConstr;
        % update KsPflqrs
        KsPflqrs = KsPflqrs + bsPnConstr;
        
        % update sizes for data
        bsPnConstr = length( bsP );
        csPnCoef   = length( csP );
        [ nConstr, AsPnCoef ] = size( AsP );
        nConstrNU = nConstr;
    end
    
    BRLNLines = 0;
else
    % dual BRL form
    
    % number of coefficients for a polynomial
    M = zeros( nPolNU );
    % number of coefficients for a filter
    N = zeros( nPolNU );
    
    % nCoef for BRL
    nCoefBRL = ncoef( KsPflqrs, ptype, nOld, nNonzOld, ones( nPol, 1), nPol, isT, isR, dmax, isBRL, BRLK1, isSB );
    
    % add columns for all the polynomials Si
    AsP = mextend( 'ac', 'zpd', AsP, nCoefBRL, 'r' );
    
    Line = 0;
    Col  = 0;
    EyeMod = [ ];
    
    % for each polynomial
    for i = 1 : nPolNU
        
        if isBRL( i )
            % yes, we have BRL in dual form
            
            % compute length for the R polynomial
            if isT( i ) && isR( i )
                % hybrid polynomial
                M( i ) = lenpol( nOld{ i }, 1, ptype{ i }.trigonometric, ptype{ i }.real );
            elseif isT( i )
                % trigonometric polynomial
                M( i ) = lenpol( nOld{ i }, 1, ptype{ i }.trigonometric, 0 );
            else
                % real polynomial
                M( i ) = lenpol( nOld{ i }, 1, 0, ptype{ i }.real );
            end
            
            Im = speye( M( i ) );
            
            % length of the causal polynomial
            N( i ) = lencpol( n{ i }, [ BRLK1( i ) BRLK2( i ) ] );
            
            % the lines that need to be cut and added afterwards
            if i <= nPolNU
                ptype{ i }.brl.hrows = ( Line + 1 ) : ( Line + N );
                BRLniLines( i ) = length( ptype{ i }.brl.hrows );
            end
            
            EyeMod = mextend( '2D', 'zpd', EyeMod, [ ( N( i ) + M( i ) ) M( i ) ], [ 'd' 'r' ] );
            
            EyeMod( Line + N( i ) + 1 : Line + N( i ) + M( i ), Col + 1 : Col + M( i ) ) = Im;
            
            % advance to the next polynomial
            Line = Line + M( i ) + N( i );
            ColOld = Col;
            Col = Col + M( i );
        else
            % not BRL
            
            % compute length for the R polynomial
            if isT( i ) && isR( i )
                % hybrid polynomial
                M( i ) = lenpol( nOld{ i }, k( i ), ptype{ i }.trigonometric, ptype{ i }.real );
            elseif isT( i )
                % trigonometric polynomial
                M( i ) = lenpol( nOld{ i }, k( i ), ptype{ i }.trigonometric, 0 );
            else
                % real polynomial
                M( i ) = lenpol( nOld{ i }, k( i ), 0, ptype{ i }.real );
            end
            
            Im = speye( M( i ) );
            
            EyeMod = mextend( '2D', 'zpd', EyeMod, [ M( i ) M( i ) ], [ 'd' 'r' ] );
            
            EyeMod( Line + 1 : Line + M( i ), Col + 1 : Col + M( i ) ) = Im;
            
            % advance to the next polynomial
            Line = Line + M( i );
            ColOld = Col;
            Col = Col + M( i );
        end
        
        if i <= nPolNU
            if isnunion( i )
                ConstrVarAdd = nULen1( i ) * M( i );
                
                EyeMod = mextend( '2D', 'zpd', EyeMod, [ ConstrVarAdd ConstrVarAdd ], [ 'd' 'r' ] );
                
                EyeMod( Line + 1 : end, Col + 1 : Col + ConstrVarAdd ) = speye( ConstrVarAdd );
                EyeMod( Line + 1 : end, ColOld + 1 : ColOld + M( i ) ) = -repmat( speye( M( i ) ), nULen1( i ), 1 );
                
                if nULen1( i )
                    for j = 1 : nULen1( i )
                        ptype{ nPolNU + j }.brl.hrows = ptype{ i }.brl.hrows;
                        BRLniLines( nPolNU + j ) = BRLniLines( i );
                    end
                end
            end
        end
    end
    BRLNLines = sum( BRLniLines );
    
    % create backup matrices
    bsPOld = bsP;
    % modify matrices
    AsP( :, bsPnConstr + 1 : end ) = EyeMod;
    bsP = csP;
    csP = [ -bsPOld sparse( 1, nCoefBRL ) ];
    % add Ks.f for the old dual variable 'y'
    if isfield( Ks, 'f' )
        Ks.f = Ks.f + bsPnConstr;
    else
        Ks.f = bsPnConstr;
    end
    % update 'nCoef'
    nCoef = nCoef + bsPnConstr;
    % update KsPflqrs
    KsPflqrs = KsPflqrs + bsPnConstr;
    
    % update sizes for data
    bsPnConstr = length( bsP );
    csPnCoef   = length( csP );
    [ nConstr, AsPnCoef ] = size( AsP );
    nConstrNU = nConstr;
    for i = 1 : nPolNU
        if nULen1( i )
            nConstr    = nConstr    + nULen1( i ) * N( i );
            bsPnConstr = bsPnConstr + nULen1( i ) * N( i );
        end
    end
end