function [ Ks, KsPflqrs ] = init_ks( KsP )

% INIT_KS Initialize structure Ks
%
%   [ KS, KSPFLQRS ] = INIT_KS( KSP ) initializes the KS
%   structure using information from KSP. The KSPFLQRS
%   variable will retain the total number of variables in
%   the 'f', 'l', 'q', 'r', 's' fields of KS.   

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 12.02.09-30.03.10

% check 'f' field
if isfield( KsP, 'f' )
    KsPf = KsP.f;
    Ks.f = KsPf;
else
    KsPf = 0;
end

% check 'l' field
if isfield( KsP, 'l' )
    KsPl = KsP.l;
    Ks.l = KsPl;
else
    KsPl = 0;
end

% check 'q' field
if isfield( KsP, 'q' )
    KsPq = sum ( KsP.q );
    Ks.q = KsP.q;
else
    KsPq = 0;
end

% check 'r' field
if isfield( KsP, 'r' )
    KsPr = sum( KsP.r );
    Ks.r = KsP.r;
else
    KsPr = 0;
end

% check 's' field
if isfield( KsP,'s' )      
    KsPs = sum( KsP.s .^ 2 );
    Ks.s = KsP.s;
else
    KsPs = 0;
    Ks.s = [ ];
end

% check 'scomplex' field
if isfield( KsP, 'scomplex' )
    Ks.scomplex = KsP.scomplex;
else
    Ks.scomplex = [ ];
end

% setting complex equality constraints
if isfield( KsP, 'ycomplex' )
    Ks.ycomplex = KsP.ycomplex;
end

% setting complex variables in f, l, q, r
if isfield( KsP, 'xcomplex' )
    Ks.xcomplex = KsP.xcomplex;
end

% the number of elements in f, l, q, r, s
KsPflqrs = KsPf + KsPl + KsPq + KsPr + KsPs;