function [ nCoef, nCoefPol ] = ncoef( KsPflqrs, ptype, nOld, nNonzOld, k, nPol, isT, isR, dmax, isBRL, BRLK1, isSB )

% NCOEF Number of coefficients
%
%   This function will compute the number of
%   coefficients of the problem posed by the 
%   user, using the KsP structure (KsPflqrs and 
%   the descriptions of the polynomials).
%
%   The problem seen by the user looks like:
%
%   min  csP' * wtx
%   s.t. AsP  * wtx = bsP
%
%   The result of the function must be equal with
%   the length of csP.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 13.12.08-19.08.2010

% nCoef will hold all the coefficients
% (including KsPflqrs) from wtx
nCoef = KsPflqrs;

% number of coefficients for every pol.
nCoefPol = zeros( nPol, 1 );

for i = 1 : nPol
    
    if ~isSB( i )
        % no sparse basis
        
        % the number of coefs. for the current pol.
        nCoefPolNew           = ncoefpol( ptype{ i }, nOld{ i }, nNonzOld{ i }, k( i ), isT( i ), isR( i ), dmax );

        if ~isBRL( i )
            nCoefPol( i )     = nCoefPolNew;
        else
            % if we have BRL then the polynomial is always scalar
            if BRLK1( i ) > 1
                % matrix BRL
                nCoefPol( i ) = ncoefpol( ptype{ i }, nOld{ i }, nNonzOld{ i },      1, isT( i ), isR( i ), dmax );
            else
                % scalar BRL
                nCoefPol( i ) = nCoefPolNew;
            end
        end
    else
        % sparse basis
        
        % the number of coefs. for the current pol.
        nCoefPolNew           = getnrcoefsb( nOld{ i }, ptype{ i }, k( i ), isT( i ), isR( i ), isSB( i ) );

        if ~isBRL( i )
            nCoefPol( i )     = nCoefPolNew;
        else
            % if we have BRL then the polynomial is always scalar
            if BRLK1( i ) > 1
                % matrix BRL
                nCoefPol( i ) = getnrcoefsb( nOld{ i }, ptype{ i }, 1, isT( i ), isR( i ), isSB( i ) );
            else
                % scalar BRL
                nCoefPol( i ) = nCoefPolNew;
            end
        end
    end
    
    % add to the total number of coefficients
    nCoef = nCoef + nCoefPolNew;
end