function nCoefPol = ncoefpol( ptypei, nOldi, nNonzOldi, K, isTi, isRi, dmax )

% NCOEF Number of coefficients
%
%   This function will compute the number of
%   coefficients of the problem posed by the 
%   user, using the KsP structure (KsPflqrs and 
%   the descriptions of the polynomials).
%
%   The problem seen by the user looks like:
%
%   min  csP' * wtx
%   s.t. AsP  * wtx = bsP
%
%   The result of the function must be equal with
%   the length of csP.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 13.12.08-05.09.2011

if dmax == 1
    % univariate case
    
    % number of multipliers
    M = nOldi + 1;        

    if K > 1
        % matrix coefficients

        if  ~isTi && isRi
            % real polynomial

            nCoefPol = M * sumn( K );
        else
            % trigonometric polynomial

            nCoefPol = ( M - 1 ) * K ^ 2 + sumn( K );
        end            
    else
        % K == 1
        % scalar coefficients

        % M
        nCoefPol = M;    
    end

else
    % multivariate case           

    if K > 1
        % matrix coefficients

        if  ~isTi && isRi
            % real polynomial

            nCoefPol = prod( nNonzOldi + 1 ) * sumn( K );
        elseif isTi && ~isRi
            % trigonometric polynomial

            nCoefPol = ( -1 + prod( 2 * nNonzOldi + 1 ) ) / 2 * K ^ 2 + sumn( K );
        else                
            % hybrid polynomial

            % number of coef. for the trig. part
            Nt = ( 1 + prod( 2 * nonzeros( nOldi( 1 : ptypei.trigonometric ) ) + 1 ) ) / 2;
            % number of coef. for the real part
            Nr = prod( nonzeros( nOldi( ptypei.trigonometric + 1 : end ) ) + 1 );
            
            nCoefPol = ( Nt - 1 ) * Nr * K ^ 2 + Nr * sumn( K );
        end 
    else
        % K == 1
        % scalar coefficients

        if  ~isTi && isRi
            % real polynomial

            nCoefPol = prod( nNonzOldi + 1 );
        elseif isTi && ~isRi
            % trigonometric polynomial

            nCoefPol = ( 1 + prod( 2 * nNonzOldi + 1 ) ) / 2;
        else                
            % hybrid polynomial

            nCoefPol = ( 1 + prod( 2 * nonzeros( nOldi( 1 : ptypei.trigonometric ) ) + 1 ) ) / 2 * ...
                         prod( nonzeros( nOldi( ptypei.trigonometric + 1 : end ) ) + 1 );
        end   
    end
end