function [ nPol, nPolNU, p, ptype, n, nOld, k, d, isT, isR, isC, isI, isD, nDomPol, isNU, isnunion, nULen1, ...
                      AsP, bsP, csP, bsPnConstr, csPnCoef, nConstr, AsPnCoef] = ...
           pos_check( nPol, p, ptype, n, nOld, k, d, dmax, isT, isR, isC, KsPflqrs, nCoefPol, ...
                      AsP, bsP, csP, bsPnConstr, csPnCoef, nConstr, AsPnCoef, ...
                      isBRL, BRLniLines, isEQForm )

% POS_CHECK Check polynomials for positivity
%
%   This function computes the preliminary variables
%   for handling the positivity of polynomials.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 23.08.2010-15.10.2010

% before update with NU
nPolNU = nPol;

% cum. sum. for nCoefPol (to be used for 'nunion')
nCoefPolCS = [ 0; cumsum( nCoefPol ) ] + KsPflqrs;

% vectors for positivity on interval(s) or
% for positivity on domains;
% pos. dom. = positivity domain
isI = zeros( nPol, 1 );
isD = zeros( nPol, 1 );
% number of total coefficients for
% the positivy polynomials from the 
% pos. dom. for each polynomial
sumNC    = zeros( nPol, 1 );
% number of pol. for the pos. dom.
% of each polynomial
nDomPol  = zeros( nPol, 1 );
% if we have 'nunion' in the pos. dom.
isnunion = zeros( nPol, 1 );
% number of polynomials to add, if 'nunion'
% for every polynomial
nULen1 = zeros( nPol, 1 );
for i = 1 : nPol
    if isfield( ptype{ i }, 'int' )
        % positivity on interval(s)
        
        % pos. int. and pos. dom. are 
        % mutually exclusive for a pol.;
        % if not - error
        assert( ~isfield( ptype{ i }, 'dom' ), ...
                'Fields ''int'' and ''dom'' are mutually exclusive for a certain element of ''ptype''; ppol: %d.', i );
            
        % length of field 'int'
        nint = length( ptype{ i }.int );
        
        % check if 'nunion' exists and is non-empty
        isnunion( i ) = 0;
        if nint > 4
            isnunion( i ) = 1;
        elseif nint == 4
            % in order to have union of intervals we must eliminate three cases:
            % 1. trig pol complex coef: [ -pi, a ] U [ b, pi ]
            % 2. trig pol real coef   : [ 0, a ] U [ b, pi ]
            % 3. real pol even degree : ( -Inf, a ] U [ b, Inf )
            if ~( isT( i ) &&         isC( i ) && ptype{ i }.int( 1 ) ==  -pi && ptype{ i }.int( 4 ) ==  pi ) && ...
               ~( isT( i ) &&        ~isC( i ) && ptype{ i }.int( 1 ) ==    0 && ptype{ i }.int( 4 ) ==  pi ) && ...
               ~( isR( i ) && iseven( n{ i } ) && ptype{ i }.int( 1 ) == -Inf && ptype{ i }.int( 4 ) == Inf )
           
                isnunion( i ) = 1;
            end
        end        
        
        if ~isnunion( i )
            % no 'nunion'
            
            % check length of 'int';
            % must be two or four
            if  nint == 2 || nint == 4
                isI( i ) = 1;
            else
                error( 'Field ''int'' must be a vector of two or four elements; ppol: %d.', i );
            end
        else
            % 'nunion' style; we need to introduce new polynomials
            
            % check if 'nint' is even
            assert( logical( iseven( nint ) ), 'The number of elements of the vector ''int'' must be even; ppol: %d.', i );
            
            % activate interval positivity
            isI( i ) = 1;
            
            % number of lines/columns to extend the restrictions
            % for one polynomial
            N = n{ i } + 1;
            
            % total number of intervals
            nULen = nint / 2;
            % number of coefficients to add
            nULen1( i ) = nULen - 1;
            % total number to extend
            ConstrVarAdd = nULen1( i ) * N;
            % extend
            if isEQForm
                % EQForm
                AsP = mextend( '2D', 'zpd', AsP, [ ConstrVarAdd ConstrVarAdd ], [ 'd' 'r' ] );
                bsP = mextend( 'ar', 'zpd', bsP,   ConstrVarAdd,                  'd'       );
                csP = mextend( 'ac', 'zpd', csP,   ConstrVarAdd,                      'r'   );
            else
                % INEQForm
                AsP = mextend( 'ar', 'zpd', AsP, ConstrVarAdd, 'd' );
                csP = mextend( 'ar', 'zpd', csP, ConstrVarAdd, 'd' );
            end
            
            % insert values in 'AsP' for the new polynomial variables
            if isEQForm
                % EQForm
                AsP( nConstr + 1 : end, AsPnCoef + 1 : end ) = speye( ConstrVarAdd );
                AsP( nConstr + 1 : end, nCoefPolCS( i ) + 1 : nCoefPolCS( i + 1 ) ) = -repmat( speye( N ), nULen1( i ), 1 ); 
            end
            
            % add the structure for the added polynomials;
            % number of polynomials added before parsing polynomial 'i'
            nPolULen1 = nPol + sum( nULen1( 1 : i - 1 ) );
            for pp = 1 : nULen1( i )
                
                nPolpp = nPolULen1 + pp;
                
                % update the 'p'  and 'ptype' structures
                p    { nPolpp } = p    { i };
                ptype{ nPolpp } = ptype{ i };
                
                % update field 'int'
                ptype{ nPolpp }.int = ptype{ i }.int( 2 + ( pp - 1 ) * 2 + 1 : 2 + pp * 2 );
                
                % introduce 'isT' and 'isR'
                isT( nPolpp ) = isT( i );
                isR( nPolpp ) = isR( i );
                % introduce 'd'
                d( nPolpp ) = d( i );
                % introduce 'n'
                n{ nPolpp } = n{ i };
                % introduce 'nOld'
                nOld{ nPolpp } = nOld{ i };
                % introduce 'k'
                k( nPolpp ) = k( i );
            end

            % update 'int' for the first polynomial
            ptype{ i }.int = ptype{ i }.int( 1 : 2 );
            
            % update variables
            if isEQForm
                bsPnConstr = bsPnConstr + ConstrVarAdd;
                csPnCoef   = csPnCoef   + ConstrVarAdd;
                nConstr    = nConstr    + ConstrVarAdd;
                AsPnCoef   = AsPnCoef   + ConstrVarAdd;
                
                if isBRL( i )
                    % if we have BRL, update the constraints
                    
                    bsPnConstr = bsPnConstr + nULen1( i ) * BRLniLines( i );
                    nConstr    = nConstr    + nULen1( i ) * BRLniLines( i );
                end
            else
                csPnCoef   = csPnCoef   + ConstrVarAdd;
                nConstr    = nConstr    + ConstrVarAdd;
                
                if isBRL( i )
                    % if we have BRL, update the constraints
                    
                    csPnCoef   = csPnCoef   + nULen1( i ) * BRLniLines( i );
                    nConstr    = nConstr    + nULen1( i ) * BRLniLines( i );
                end
            end
        end
    elseif isfield( ptype{ i }, 'dom' )
        % positivity on domain
        
        % check if field 'deg' exists
        assert( isfield( ptype{ i }.dom, 'deg' ),  'Field ''deg'' is missing from field ''dom''; ppol: %d.' , i );

        % check if field 'coef' exists
        assert( isfield( ptype{ i }.dom, 'coef' ), 'Field ''coef'' is missing from field ''dom''; ppol: %d.', i );
        
        % check if field 'deg' is empty
        assert( ~isempty( ptype{ i }.dom.deg ) , 'Field ''deg'' from field ''dom'' is empty; ppol: %d.' , i );

        % check if field 'coef' is empty
        assert( ~isempty( ptype{ i }.dom.coef ), 'Field ''coef'' from field ''dom'' is empty; ppol: %d.', i );
        
        % check if field 'nc' exists and is non-empty;
        % ncStyle = 1 uses deg, dom and nc; a sparse representation
        % of the polynomial from the pos. dom.
        % ncStyle = 2 used only deg and dom; a full description
        % of the polynomial from the pos. dom.
        isfieldNC = isfield( ptype{ i }.dom, 'nc' );        
        if isfieldNC
            isemptyNC = isempty( ptype{ i }.dom.nc );
            if isemptyNC
                % empty 'nc', fullstyle
                ncStyle = 2;
            else
                % non-empty 'nc', sparse style
                ncStyle = 1;
            end
        else
            % no 'nc', full style
            ncStyle = 2;
        end
        % check if 'nunion' exists and is non-empty
        isnunion( i ) = 0;
        if isfield( ptype{ i }.dom, 'nunion' )
            if ~isempty( ptype{ i }.dom.nunion )
                % non-empty 'nunion' field;
                
                % number of domains in the union
                nULen = length( ptype{ i }.dom.nunion );
                if nULen > 1
                    % more than one domain
                    isnunion( i ) = 1;
                end
            end
        end
           
        if ncStyle == 1
            % sparse description of the pol. from pos. dom.
            
            if ~isnunion( i )
                % no 'nunion'
                
                % number of degrees
                [ ndegs  ] = size  ( ptype{ i }.dom.deg, 1 );
                % number of coefficients
                [ ncoefs ] = length( ptype{ i }.dom.coef );

                % check the number of polynomials for 
                % 'deg' and 'coef' to be the same
                assert( ndegs == ncoefs, 'Nr. of deg. and coef. for the polynomial(s) defining the positivity domain must be equal; ppol: %d.', i );

                % check if nr. coefs. from 'coefs' equal
                % nr. of coefs. from 'nc' (sum(nc))
                sumNC( i ) = sum( ptype{ i }.dom.nc ); 
                assert( ndegs == sumNC( i ), 'Number of coefficients from ''coef'' and ''nc'' must be equal; ppol: %d.', i );

                % check if the degrees from the pos. dom. are
                % not greater than the degree of the pos. pol
                for degi = 1 : ndegs
                    assert( logical( leqm( ptype{ i }.dom.deg( degi, : ), n{ i } ) ), ...
                            'The degrees from the pos. dom. cannot be (elementwise) greater than the degree of the pos. pol.; ppol: %d.', i );
                end
            else
                % 'nunion' style; we need to introduce new polynomials
                
                % check if sum of 'nunion' is equal to the length of 'nc'
                assert( sum( ptype{ i }.dom.nunion ) == length( ptype{ i }.dom.nc ), ...
                        'The sum of the elements in ''nunion'' must be equal to the number of elements in ''nc''; ppol: %d.', i );

                % number of lines/columns to extend the restrictions
                % for one polynomial
                if isT( i ) && isR( i )
                    % hybrid pol.
                    N = nchp( n{ i }, ptype{ i }.trigonometric );
                elseif isT( i )
                    % trig. pol.
                    N = nch( n{ i } );
                else
                    % real pol.
                    N = prod( n{ i } + 1 );
                end 
                % number of coefficients to add
                nULen1( i ) = nULen - 1;
                % total number to extend
                ConstrVarAdd = nULen1( i ) * N;
                % extend
                if isEQForm
                    % EQForm
                    AsP = mextend( '2D', 'zpd', AsP, [ ConstrVarAdd ConstrVarAdd ], [ 'd' 'r' ] );
                    bsP = mextend( 'ar', 'zpd', bsP,   ConstrVarAdd,                  'd'       );
                    csP = mextend( 'ac', 'zpd', csP,   ConstrVarAdd,                      'r'   );
                else
                    % INEQForm
                    AsP = mextend( 'ar', 'zpd', AsP, ConstrVarAdd, 'd' );
                    csP = mextend( 'ar', 'zpd', csP, ConstrVarAdd, 'd' );
                end
                
                % insert values in 'AsP' for the new polynomial variables
                if isEQForm
                    % EQForm
                    AsP( nConstr + 1 : end, AsPnCoef + 1 : end ) = speye( ConstrVarAdd );
                    AsP( nConstr + 1 : end, nCoefPolCS( i ) + 1 : nCoefPolCS( i + 1 ) ) = -repmat( speye( N ), nULen1( i ), 1 );
                end
                
                % number of coefficients for every pol.
                ncNU = zeros( nULen, 1 );
                % number of coefs for the 1st pol., which is the original one
                ncNU( 1 ) = sum( ptype{ i }.dom.nc( 1 : ptype{ i }.dom.nunion( 1 ) ) );
                % cum. sum. for 'nunion'
                nunionCS = [ 0; cumsum( reshape( ptype{ i }.dom.nunion, nULen, 1 ) ) ];
                % add the structure for the added polynomials;
                % number of polynomials added before parsing polynomial 'i'
                nPolULen1 = nPol + sum( nULen1( 1 : i - 1 ) );
                for pp = 1 : nULen1( i )
                    
                    nPolpp = nPolULen1 + pp;
                    
                    % update the 'p' and 'ptype' structures
                    p    { nPolpp } = p    { i };
                    ptype{ nPolpp } = ptype{ i };
                    
                    % 'nc'
                    ptype{ nPolpp }.dom.nc = ptype{ i }.dom.nc( nunionCS( pp + 1 ) + 1 : nunionCS( pp + 2 ) );
                    % number of coefficients for the pols. to be added
                    ncNU( pp + 1 ) = sum( ptype{ nPolpp }.dom.nc );
                end
                % cum. sum.
                ncNUCS = [ 0; cumsum( ncNU ) ];
                % 'deg' and 'coef'
                for pp = 1 : nULen1( i )
                    
                    nPolpp = nPolULen1 + pp;
                    
                    ptype{ nPolpp }.dom.deg  = ptype{ i }.dom.deg ( ncNUCS( pp + 1 ) + 1 : ncNUCS( pp + 2 ), : );
                    ptype{ nPolpp }.dom.coef = ptype{ i }.dom.coef( ncNUCS( pp + 1 ) + 1 : ncNUCS( pp + 2 )    );
                    
                    % introduce 'isT' and 'isR'
                    isT( nPolpp ) = isT( i );
                    isR( nPolpp ) = isR( i );
                    % introduce 'd'
                    d( nPolpp ) = d( i );
                    % introduce 'n'
                    n{ nPolpp } = n{ i };
                    % introduce 'nOld'
                    nOld{ nPolpp } = nOld{ i };
                    % introduce 'k'
                    k( nPolpp ) = k( i );
                end
                % update the first polynomial: 'nc', 'deg' and 'coef'
                ptype{ i }.dom.nc   = ptype{ i }.dom.nc  ( nunionCS( 1 ) + 1 : nunionCS( 2 )    );
                ptype{ i }.dom.deg  = ptype{ i }.dom.deg ( ncNUCS  ( 1 ) + 1 : ncNUCS  ( 2 ), : );
                ptype{ i }.dom.coef = ptype{ i }.dom.coef( ncNUCS  ( 1 ) + 1 : ncNUCS  ( 2 )    );
                
                % update variables
                if isEQForm
                    % EQForm
                    
                    bsPnConstr = bsPnConstr + ConstrVarAdd;
                    csPnCoef   = csPnCoef   + ConstrVarAdd;
                    nConstr    = nConstr    + ConstrVarAdd;
                    AsPnCoef   = AsPnCoef   + ConstrVarAdd;
                    
                    if isBRL( i )
                        % if we have BRL, update the constraints
                        
                        bsPnConstr = bsPnConstr + nULen1( i ) * BRLniLines( i );
                        nConstr    = nConstr    + nULen1( i ) * BRLniLines( i );
                    end
                else
                    % INEQForm
                    
                    csPnCoef   = csPnCoef   + ConstrVarAdd;
                    nConstr    = nConstr    + ConstrVarAdd;
                    
                    if isBRL( i )
                        % if we have BRL, update the constraints
                        
                        csPnCoef   = csPnCoef   + nULen1( i ) * BRLniLines( i );
                        nConstr    = nConstr    + nULen1( i ) * BRLniLines( i );
                    end
                end
            end
        else
            % full description of the pol. from pos. dom.
            
            if ~isnunion( i )
                % no 'nunion'
            
                % full to sparse for 'dom'
                ptype{ i } = dom_full2sparse( ptype{ i }, dmax, isT( i ), isR( i ), i );
            else
                % 'nunion' style; we need to introduce new polynomials
                
                % number of polynomials in the reunion of domains
                [ npols  ] = size  ( ptype{ i }.dom.deg, 1 );
                % number of coefficients
                [ ncoefs ] = length( ptype{ i }.dom.coef );
                
                % check if the number of coefs. for the pols. from the pos. dom. are correct
                assert( sum( ptype{ i }.dom.nunion ) == npols, ...
                        'Inconsistency between the number of degrees of polynomials and the number of polynomials in ''nunion''; ppol: %d.', i );  
                
                % number of lines/columns to extend the restrictions
                % for one polynomial
                if isT( i ) && isR( i )
                    % hybrid pol.
                    N = nchp( n{ i }, ptype{ i }.trigonometric );
                elseif isT( i )
                    % trig. pol.
                    N = nch( n{ i } );
                else
                    % real pol.
                    N = prod( n{ i } + 1 );
                end 
                % number of coefficients to add
                nULen1( i ) = nULen - 1;
                % total number to extend
                ConstrVarAdd = nULen1( i ) * N;
                % extend
                if isEQForm
                    % EQForm
                    AsP = mextend( '2D', 'zpd', AsP, [ ConstrVarAdd ConstrVarAdd ], [ 'd' 'r' ] );
                    bsP = mextend( 'ar', 'zpd', bsP,   ConstrVarAdd,                  'd'       );
                    csP = mextend( 'ac', 'zpd', csP,   ConstrVarAdd,                      'r'   );
                else
                    % INEQForm
                    AsP = mextend( 'ar', 'zpd', AsP, ConstrVarAdd, 'd' );
                    csP = mextend( 'ar', 'zpd', csP, ConstrVarAdd, 'd' );
                end
                
                % insert values in 'AsP' for the new polynomial variables
                if isEQForm
                    % EQForm
                    AsP( nConstr + 1 : end, AsPnCoef + 1 : end ) = speye( ConstrVarAdd );
                    AsP( nConstr + 1 : end, nCoefPolCS( i ) + 1 : nCoefPolCS( i + 1 ) ) = -repmat( speye( N ), nULen1( i ), 1 );
                end
                
                % zero degree
                o = zeros( 1, dmax );
                % number of coefficients for 
                % the pol. from pos. dom.
                nCoefPolPD  = zeros( npols, 1 );
                if isT( i ) && isR( i )
                    % hybrid pol.

                    for pp = 1 : npols
                        
                        % check if the degrees for the pol. from pos. dom. are nonnegative
                        assert( logical( leqm( o, ptype{ i }.dom.deg( pp, : ) ) ), ...
                                'The degrees for the pol. from the pos. dom. must be nonnegative; ppol: %d.', i );
                        nCoefPolPD( pp ) = nchp   ( ptype{ i }.dom.deg( pp, : ), ptype{ i }.trigonometric );
                    end
                elseif isT( i )
                    % trig. pol.

                    for pp = 1 : npols
                        
                        % check if the degrees for the pol. from pos. dom. are nonnegative
                        assert( logical( leqm( o, ptype{ i }.dom.deg( pp, : ) ) ), ...
                                'The degrees for the pol. from the pos. dom. must be nonnegative; ppol: %d.', i );
                        nCoefPolPD( pp ) = nch   ( ptype{ i }.dom.deg( pp, : ) );
                    end
                else
                    % real pol.

                    for pp = 1 : npols
                        
                        % check if the degrees for the pol. from pos. dom. are nonnegative
                        assert( logical( leqm( o, ptype{ i }.dom.deg( pp, : ) ) ), ...
                                'The degrees for the pol. from the pos. dom. must be nonnegative; ppol: %d.', i );
                        nCoefPolPD( pp ) = prod  ( ptype{ i }.dom.deg( pp, : ) + 1 );
                    end
                end 
                
                % check if the number of coefs. for the pols. from the pos. dom. are correct
                assert( sum( nCoefPolPD ) == ncoefs, ...
                        'The number of coefficients for all the polynomials describing the positivity domain is incorrect; ppol: %d.', i );  

                % cum. sum. for 'nunion'
                nunionCS = [ 0; cumsum( reshape( ptype{ i }.dom.nunion, nULen, 1 ) ) ];
                % cum. sum. for 'nunion'
                nCoefPolPDCS = [ 0; cumsum( nCoefPolPD ) ];
                % add the structure for the added polynomials;
                % number of polynomials added before parsing polynomial 'i'                
                nPolULen1 = nPol + sum( nULen1( 1 : i - 1 ) );
                for pp = 1 : nULen1( i )
                    
                    nPolpp = nPolULen1 + pp;
                    
                    % update the 'p' and 'ptype' structures
                    p    { nPolpp } = p    { i };
                    ptype{ nPolpp } = ptype{ i };
                end
                % 'deg' and 'coef'
                for pp = 1 : nULen1( i )
                    
                    nPolpp = nPolULen1 + pp;
                    
                    % degrees from the list of degrees in 'deg'
                    ptype{ nPolpp }.dom.deg  = ptype{ i }.dom.deg  ...
                                               (               nunionCS( pp + 1 ) + 1       :               nunionCS( pp + 2 ), :    );
                    % extract the coefficients
                    ptype{ nPolpp }.dom.coef = ptype{ i }.dom.coef ...
                                               ( nCoefPolPDCS( nunionCS( pp + 1 ) + 1 ) + 1 : nCoefPolPDCS( nunionCS( pp + 2 ) + 1 ) );
                    
                    % introduce 'isT' and 'isR'
                    isT( nPolpp ) = isT( i );
                    isR( nPolpp ) = isR( i );
                    % introduce 'd'
                    d( nPolpp ) = d( i );
                    % introduce 'n'
                    n{ nPolpp } = n{ i };
                    % introduce 'nOld'
                    nOld{ nPolpp } = nOld{ i };
                    % introduce 'k'
                    k( nPolpp ) = k( i );
                end
                % update the first (original) polynomial: 'deg' and 'coef'
                ptype{ i }.dom.deg  = ptype{ i }.dom.deg (               nunionCS( 1 ) + 1       :               nunionCS( 2 ), :    );
                ptype{ i }.dom.coef = ptype{ i }.dom.coef( nCoefPolPDCS( nunionCS( 1 ) + 1 ) + 1 : nCoefPolPDCS( nunionCS( 2 ) + 1 ) );
                
                % full to sparse for dom for the original polynomial
                ptype{ i } = dom_full2sparse( ptype{ i }, dmax, isT( i ), isR( i ), i );
                for pp = 1 : nULen1( i )
                    
                    nPolpp = nPolULen1 + pp;
                    
                    % full to sparse for dom for the added polynomials
                    ptype{ nPolpp } = dom_full2sparse( ptype{ nPolpp }, dmax, isT( nPolpp ), isR( nPolpp ), nPolpp );
                end
                    
                % update variables
                if isEQForm
                    % EQForm
                    
                    bsPnConstr = bsPnConstr + ConstrVarAdd;
                    csPnCoef   = csPnCoef   + ConstrVarAdd;
                    nConstr    = nConstr    + ConstrVarAdd;
                    AsPnCoef   = AsPnCoef   + ConstrVarAdd;
                    
                    if isBRL( i )
                        % if we have BRL, update the constraints
                        
                        bsPnConstr = bsPnConstr + nULen1( i ) * BRLniLines( i );
                        nConstr    = nConstr    + nULen1( i ) * BRLniLines( i );
                    end
                else
                    % INEQForm
                    
                    csPnCoef   = csPnCoef   + ConstrVarAdd;
                    nConstr    = nConstr    + ConstrVarAdd;
                    
                    if isBRL( i )
                        % if we have BRL, update the constraints
                        
                        csPnCoef   = csPnCoef   + nULen1( i ) * BRLniLines( i );
                        nConstr    = nConstr    + nULen1( i ) * BRLniLines( i );
                    end
                end
            end
        end
        
        % nr of pol. in the pos. dom.;
        % (numbers of number of coefs.)
        nDomPol( i ) = length( ptype{ i }.dom.nc );
        
        % polynomial positive
        % on domain
        isD( i ) = 1;        
    end
    
    % introduce nDomPol, isI, isD for the new pols.
    if isnunion( i )
        for pp = 1 : nULen1( i )
            
            nPolpp = nPolULen1 + pp;
            
            % introduce
            nDomPol( nPolpp ) = nDomPol( i );
            isC    ( nPolpp ) = isC    ( i );
            isI    ( nPolpp ) = isI    ( i );
            isD    ( nPolpp ) = isD    ( i );            
        end
    end
end
% check if any 'nunion'
isNU = nnz( isnunion );
% update number of polynomials
if isNU
    nPol = nPol + sum( nULen1 );    
end