function [ nSi, QDKsize, QD2size, nNonzSi, nNonzOldSi, degDNonz, nDomPolCS, ncCS, SBDegsAddSubDomPol, SBPosDegsDomPol ] = ...
           posdom( nPol, ptype, n, nOld, k, dmax, isT, isR, isD, nDomPol, ZeroDeg, isSB, SBNrDegs, SBDegsAddSub )
       
% POSDOM Positivity domain
%
%   This function computes the variables needed to describe
%   the positivity domains of the multivariable polynomials.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 04.06.09-27.07.10

% total number of pol. 
% for the pos. doms.
NDomPol = sum( nDomPol );
% degree for all pol. from all
% the positivity domains;
% "old" & "new"
nSi    = zeros( NDomPol, dmax );
nOldSi = zeros( NDomPol, dmax );

% nonzeros
nNonzSi    = cell( nPol, 1 );
nNonzOldSi = cell( nPol, 1 );

% degrees for pol. from pos. dom.
degDNonz   = cell( nPol, 1 );

% Qsize for pol. from pos. dom.
QDsize    = zeros( NDomPol, 1 );
QDKsize   = zeros( NDomPol, 1 );
QD2size   = zeros( NDomPol, 1 );
QDOldsize = zeros( NDomPol, 1 );

% cum. sum for nDomPol
nDomPolCS = cumsum( [ 0; nDomPol( 1 : end - 1 ) ] );

% cum. sum. for nc for every pol.
ncCS = cell( nPol, 1 );

% degrees for the SOS pol. from the pos. dom.
% for sparse basis
SBDegsAddSubDomPol = cell ( NDomPol, 1 );

% position for the sparse basis for
% the pol. from the pos. dom.
SBPosDegsDomPol    = cell ( NDomPol, 1 );

for i = 1 : nPol
    if isD( i )
        % nr. of coefs. for each pol.
        nc   = ptype{ i }.dom.nc;
        % degrees for the pos. dom.
        degs = ptype{ i }.dom.deg;
        
        % cum. sum for 'nc'
        ncCS{ i } = cumsum( [ 0; reshape( nc( 1 : end - 1 ), nDomPol( i ) - 1 , 1 ) ] );
        
        % for every polynomial from
        % the positivity domain
        for DomPol = 1 : nDomPol( i )
            % maximum degree
            maxDeg = max( abs( degs( ncCS{ i }( DomPol ) + 1 : ncCS{ i }( DomPol ) + nc( DomPol ), : ) ), [ ], 1 );
            % degree for Si (i=1:nDomPol(i))
            nOldSi( nDomPolCS( i ) + DomPol, : ) = nOld{ i } - maxDeg;
        end
        
        % adjust to even degree for the degrees of the 
        % real variables; subtract 1 if necessary
        if isR( i )
            if isT( i )
                % hybrid polynomial
                
                % subtract 1, if needed
                for degsi = nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i )
                    for degi = ptype{ i }.trigonometric + 1 : dmax
                        if ~iseven( nOldSi( degsi, degi ) )
                            nOldSi( degsi, degi ) = nOldSi( degsi, degi ) - 1;
                        end
                    end
                end
            else
                % real polynomial
                
                % subtract 1, if needed
                for degsi = nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i )
                    for degi = 1 : dmax
                        if ~iseven( nOldSi( degsi, degi ) )
                            nOldSi( degsi, degi ) = nOldSi( degsi, degi ) - 1;
                        end
                    end
                end
            end            
        end
        
        % divide by 2 for the size 
        % of the Gram matrices
        if isR( i )
            if isT( i )
                % hybrid polynomial
                
                        % copy all
                        nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : ) = ...
                     nOldSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : );
                
                        % divide by 2 for the real part
                        nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), ptype{ i }.trigonometric + 1 : end ) = ...
                        nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), ptype{ i }.trigonometric + 1 : end ) / 2;
            else
                % real polynomial
                
                        nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : ) = ...
                     nOldSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : ) / 2;
            end            
        else
            % trigonometric polynomial
            
               nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : ) = ...
            nOldSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : );    
        end
        
        % nonzeros
        if ~isempty( ZeroDeg{ i } )
            % delete zero degree
            Indexes = logical( ~onesc( dmax, 1, ZeroDeg{ i } ) ); 
            
            degDNonz{ i }   = degs( :, Indexes );
            nNonzSi{ i }    = nSi    ( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), Indexes );
            nNonzOldSi{ i } = nOldSi ( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), Indexes );
        else
            % no zero degree
            degDNonz{ i }   = degs;
            nNonzSi{ i }    = nSi    ( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : );
            nNonzOldSi{ i } = nOldSi ( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : );
        end
        
        % SPARSE BASIS CHECK
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % copy the degrees from the leading polynomial
        % to all the polynomials from the pos. dom.
        if isSB( i )
            
            for DomPol = 1 : nDomPol( i )
                
                % index for degree
                idx = nDomPolCS( i ) + DomPol;
                
                % copy the degrees for the first pol. (S0) to the pol. from the pos. dom.
                SBDegsAddSubDomPol{ idx } = SBDegsAddSub{ i };
                
                % index for the degrees to eliminate
                DegsRem = zeros( SBNrDegs( i ), 1 );
                if isT( i ) && isR( i )
                    % hybrid polynomial
                    
                    nt = ptype{ i }.trigonometric;
                    nt1 = nt + 1;
                    
                    % degree (0,0..,0);
                    
                    % for every coefficient
                    for coefi = 1 : ptype{ i }.dom.nc( DomPol )
                        
                        % temporary degree for current coef.
                        degDi = degDNonz{ i }( ncCS{ i }( DomPol ) + coefi, : );
                        
                        if ~iszero( degDi )
                            
                            % z^n, z^{-n} - trig. part    
                            % deg = degDi - degSi
                            % deg = degDi + degSi
                            % t^n - real part
                            % deg = degDi - degSi
                            degTM = -degDi;
                            degTP(   1 : nt   ) = degDi( 1   : nt  );
                            degTP( nt1 : dmax ) = degTM( nt1 : end );
                            
                            % t^n - deg. from pol. from pos. dom.
                            if ~geqm( degTM( nt1 : end ), SBDegsAddSub{ i }( 1, nt1 : end ) ) && ~leqm( degTM( nt1 : end ), nNonzOldSi{ i }( DomPol, nt1 : end ) )
                                DegsRem( 1 ) = 1;
                            else
                                if ~leqm( abs( degTM( 1 : nt ) ), nNonzOldSi{ i }( DomPol, 1 : nt ) ) && ~leqm( abs( degTP( 1 : nt ) ), nNonzOldSi{ i }( DomPol, 1 : nt ) )
                                    DegsRem( 1 ) = 1;
                                end
                            end
                        end
                    end
                    
                    % the rest of the degrees
                    for j = 2 : SBNrDegs( i )
                        
                        % for every coefficient
                        for coefi = 1 : ptype{ i }.dom.nc( DomPol )
                            
                            % temporary degree for current coef.
                            degDi = degDNonz{ i }( ncCS{ i }( DomPol ) + coefi, : );
                            
                            if ~iszero( degDi( 1 : nt ) )
                                
                                % z^n, z^{-n} - trig. part    
                                % deg = degDi - degSi
                                % deg = degDi + degSi
                                % t^n - real part
                                % deg = degDi - degSi
                                degTM = SBDegsAddSubDomPol{ idx }( j, : ) - degDi;
                                degTP(   1 : nt       ) = SBDegsAddSubDomPol{ idx }( j, 1 : nt ) + degDi( 1 : nt );
                                degTP( nt1 : nNonzLen ) = degTM( nt1 : end );
                                
                                % t^n - deg. from pol. from pos. dom.
                                if ~geqm( degTM( nt1 : end ), SBDegsAddSub{ i }( 1, nt1 : end ) ) && ~leqm( degTM( nt1 : end ), nNonzOldSi{ i }( DomPol, nt1 : end ) )
                                    DegsRem( j ) = j;
                                else
                                    if ~leqm( abs( degTM( 1 : nt ) ), nNonzOldSi{ i }( DomPol, 1 : nt ) ) && ~leqm( abs( degTP( 1 : nt ) ), nNonzOldSi{ i }( DomPol, 1 : nt ) )
                                        DegsRem( j ) = j;
                                    end
                                end
                            else
                                degTM = SBDegsAddSubDomPol{ idx }( j, : ) - degDi;
                                
                                % t^n - deg. from pol. from pos. dom.
                                if ~geqm( degTM( nt1 : end ), SBDegsAddSub{ i }( 1, nt1 : end ) ) && ~leqm( degTM( nt1 : end ), nNonzOldSi{ i }( DomPol, nt1 : end ) )
                                    DegsRem( j ) = j;
                                else
                                    if ~leqm( abs( degTM( 1 : nt ) ), nNonzOldSi{ i }( DomPol, 1 : nt ) )
                                        DegsRem( j ) = j;
                                    end
                                end
                            end
                        end
                    end
                elseif isT( i )
                    % trigonometric polynomial
                    
                    % degree (0,0..,0);
                    
                    % for every coefficient
                    for coefi = 1 : ptype{ i }.dom.nc( DomPol )
                        
                        % temporary degree for current coef.
                        degDi = degDNonz{ i }( ncCS{ i }( DomPol ) + coefi, : );
                        
                        if ~iszero( degDi )
                            
                            % z^n; deg = degDi - degSi
                            degTP =  degDi;
                            % z^{-n}; deg = degDi + degSi
                            degTM = -degDi;
                            
                            % z^{-n} - deg. from pol. from pos. dom.
                            if ~leqm( abs( degTM ), nNonzOldSi{ i }( DomPol, : ) ) && ~leqm( abs( degTP ), nNonzOldSi{ i }( DomPol, : ) )
                                DegsRem( 1 ) = 1;
                            end
                        end
                    end
                    
                    % the rest of the degrees
                    for j = 2 : SBNrDegs( i )
                        
                        % for every coefficient
                        for coefi = 1 : ptype{ i }.dom.nc( DomPol )
                            
                            % temporary degree for current coef.
                            degDi = degDNonz{ i }( ncCS{ i }( DomPol ) + coefi, : );
                            
                            if ~iszero( degDi )
                                
                                % z^n; deg = degDi - degSi
                                degTP = SBDegsAddSubDomPol{ idx }( j, : ) + degDi;
                                % z^{-n}; deg = degDi + degSi
                                degTM = SBDegsAddSubDomPol{ idx }( j, : ) - degDi;
                                
                                % z^{-n} - deg. from pol. from pos. dom.
                                if ~leqm( abs( degTM ), nNonzOldSi{ i }( DomPol, : ) ) && ~leqm( abs( degTP ), nNonzOldSi{ i }( DomPol, : ) )
                                    DegsRem( j ) = j;
                                end
                            else
                                if ~leqm( abs( SBDegsAddSubDomPol{ idx }( j, : ) ), nNonzOldSi{ i }( DomPol, : ) )
                                    DegsRem( j ) = j;
                                end
                            end
                        end
                    end
                else
                    % real polynomial
                    
                    for j = 1 : SBNrDegs( i )
                        
                        % for every coefficient
                        for coefi = 1 : ptype{ i }.dom.nc( DomPol )
                            
                            % temporary degree for current coef.
                            degDi = degDNonz{ i }( ncCS{ i }( DomPol ) + coefi, : );
                            
                            % t^n; deg = degDi + degSi
                            degTM = SBDegsAddSubDomPol{ idx }( j, : ) - degDi;
                            
                            % t^n - deg. from pol. from pos. dom.
                            if ~( geqm( degTM, SBDegsAddSub{ i }( 1, : ) ) && leqm( degTM, nNonzOldSi{ i }( DomPol, : ) ) )
                                DegsRem( j ) = j;
                            end
                        end
                    end
                end
                
                % eliminate degrees that cannot be made by the pol. from the pos. dom.
                SBDegsAddSubDomPol{ idx }( DegsRem( DegsRem ~= 0 ), : ) = [ ];
                
                if ~leqm( max( SBDegsAddSubDomPol{ idx } ), nOld{ i } )
                    error( [ 'The available degrees for which we write the equalities are not consistent with the degree of the '...
                             'SOS polynomial multiplied with the polynomial from the positivity domain. Increase the degree of the main polynomial.' ] );
                end
                
                % degrees to generate the degrees for the Si polynomials
                if isT( i ) && isR( i )
                    % hybrid polynomial
                    spdegsDomPol = sparsedegs( SBDegsAddSubDomPol{ idx }, n{ i }, nOld{ i }, ptype{ i }.trigonometric, ptype{ i }.real );
                elseif isT( i )
                    % trigonometric polynomial
                    spdegsDomPol = sparsedegs( SBDegsAddSubDomPol{ idx }, n{ i }, nOld{ i }, ptype{ i }.trigonometric, 0 );
                else
                    % real polynomial
                    spdegsDomPol = sparsedegs( SBDegsAddSubDomPol{ idx }, n{ i }, nOld{ i }, 0, ptype{ i }.real );
                end
                
                if isT( i ) && isR( i )
                    % hybrid polynomial
                    
                    % eliminate negative degrees
                    spdegsDomPol = spdegs2spbasis( spdegsDomPol );
                    
                    [ SBDegsAddSubDomPol{ idx }, SBNrDegs, SBPosDegsDomPol{ idx }, SBNrDegsMono ] = ...
                      genspbasis( n{ i }, nOld{ i }, dmax, isT, isR, prod( n{ i } + 1 ), spdegsDomPol, 2, ptype{ i }.trigonometric );
                elseif isT( i )
                    % trigonometric
                    
                    % eliminate negative degrees
                    spdegsDomPol = spdegs2spbasis( spdegsDomPol );
                    
                    [ SBDegsAddSubDomPol{ idx }, SBNrDegs, SBPosDegsDomPol{ idx }, SBNrDegsMono ] = ...
                      genspbasis( n{ i }, nOld{ i }, dmax, isT, isR, prod( n{ i } + 1 ), spdegsDomPol, 2 );
                else
                    % real polynomial
                    
                    [ SBDegsAddSubDomPol{ idx }, SBNrDegs, SBPosDegsDomPol{ idx }, SBNrDegsMono ] = ...
                      genspbasis( n{ i }, nOld{ i }, dmax, isT, isR, prod( n{ i } + 1 ), spdegsDomPol, 2 );

                end             
                
                % replace nNonzSi and nNonzOldSi
                nNonzSi   { i } = repmat( n   { i }, nDomPol( i ), 1 );
                nNonzOldSi{ i } = repmat( nOld{ i }, nDomPol( i ), 1 );
                
                % QDsize
                QDsize ( idx ) = SBNrDegsMono;
            end
        else
            
            for DomPol = 1 : nDomPol( i )
                
                % index for degree
                idx = nDomPolCS( i ) + DomPol;
                
                % QDsize && QDKsize && QD2size
                QDsize ( idx )  = prod( nSi( idx, : ) + 1 );
            end
        end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        % size for Gram matrices
        for DomPol = 1 : nDomPol( i )
            
            % index for degree
            idx = nDomPolCS( i ) + DomPol;
            
            % QDKsize && QD2size
            QDKsize( idx )   = QDsize( idx ) * k( i );
            
            QD2size( idx )   = ( QDKsize( idx ) ) ^ 2;
            
            % QDOldsize
            QDOldsize( idx ) = prod( nOldSi( idx, : ) + 1 );
        end   
    end
end