function [ isSB, ISSB ] = spbasis_check( ptype, nPol )

% SPBASIS_CHECK Sparse basis check
%
%   This function check if the polynomials must use
%   sparse basis. The function returns a variable 
%   which is a vector telling if the polynomials 
%   use sparse basis.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 11.06.10-06.08.10

% if we have sparse basis
isSB = zeros( nPol, 1 );

for i = 1 : nPol
    
    % if we have sparse basis
    if isfield( ptype{ i }, 'spbasis' )
        
        % if we have field 'ind'
        if isfield( ptype{ i }.spbasis, 'ind' )
            % fields 'ind' and 'deg' are mutually exclusive
            if isfield( ptype{ i }.spbasis, 'deg' )
                error( 'Fields ''ind'' and ''deg'' are mutually exclusive.' );
            end
            
            % activate sparse basis;
            % activate with 1 for 'ind'
            if ~isempty( ptype{ i }.spbasis.ind )
                isSB( i ) = 1;
            end
        else
            % check if field 'deg'
            if isfield( ptype{ i }.spbasis, 'deg' )
                
                % activate sparse basis;
                % activate with 2 for 'deg'
                if ~isempty( ptype{ i }.spbasis.deg )
                    isSB( i ) = 2;
                end
            else
                % no field 'deg' and no field 'ind'
                error( 'No fields for spase basis. Add field ''ind'' or ''deg''.' );
            end
        end
    end
end

% if any sparse basis
ISSB = nnz( isSB );