function spdegs = spdegs2spbasis( spdegs )

% SPDEGS2SPBASIS Sparse degs to sparse basis
%
%   This function modifies the matrix so that it constains only
%   positive degrees. For a negative degree, e.g. [ -1 2 ], we replace
%   this one with [ 1 0 ] and [ 0 2 ].

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 17.06.10

% number of degrees
ndegs = size( spdegs, 1 );

% index for negative degrees
DegsIdRem = zeros( ndegs, 1 );

% search for negative degrees
for i = 1 : ndegs
    if nnz( spdegs( i, : ) < 0 )
        DegsIdRem( i ) = i;
    end
end

% number of negative degrees
nNeg = nnz( DegsIdRem );

% if there are negative degrees
if nNeg
    
    % negative degrees
    DegsRem = spdegs( DegsIdRem ~= 0, : );
    % remove negative degrees
    spdegs = spdegs( DegsIdRem == 0, : );
    
    for i = 1 : nNeg
        % the negative degrees
        Neg = DegsRem( i, : );
        % the positive degrees
        Pos = DegsRem( i, : );
        
        Neg( Neg < 0 ) = 0;
        Pos( Pos >= 0 ) = 0;
        Pos = -Pos;
        
        spdegs = mextend( 'ar', 'zpd', spdegs, 2, 'd' );
        % add the degrees
        spdegs( end - 1, : ) = Pos;
        spdegs( end    , : ) = Neg;
    end
end