function [ m ] = min_poly_value_multi_general_trig_cvx( n, r )

% MIN_POLY_VALUE_MULTI_GENERAL_TRIG_CVX Minimum value of a multivariate trig. polynomial
%
%   This function solves the problem of finding the minimum of a 
%   multivariate trigonometric polynomial, given in Section 3.5, 
%   page 79, in [1], using POS3POLY.
%
%   [ M ] = MIN_POLY_VALUE_MULTI_GENERAL_TRIG_CVX( N, R )
%
%   INPUT:
%     
%   N - degree of the multivariate polynomial
%   R - coefficients of the trigonometric polynomial
%
%   OUTPUTS:
%
%   MU - minimum value
%   
%   The original problem is
%
%   mu* = max  mu
%           
%         s.t. R(z) - mu >= 0, z in T^d.
%
%   Example:
%
%   m = min_poly_value_multi_general_trig_cvx( [ 2 1 ], [ 38 18 4 1 2 1 -8 -5 ]' );
%      
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 12.03.09-28.03.09

nC = lenpol( n );
nr = length( r );
if nr ~= nC
    error( 'Inconsistency between ''r'' and ''n''.' );
end

cvx_begin
    variable m;
    
    maximize m;

    subject to
    
    % SOS polynomial
    r - m * unitpol( n ) == sos_pol( [ n 1 ] );
cvx_end