function [ mu ] = min_poly_value_multi_general_hybrid_dual( n, h, nt )

% MIN_POLY_VALUE_MULTI_GENERAL_HYBRID_DUAL Minimum value of a multivariate hybrid polynomial
%
%   This function solves the problem of finding the minimum of a
%   multivariate real polynomial using POS3POLY.
%
%   [ MU ] = MIN_POLY_VALUE_MULTI_GENERAL_HYBRID_DUAL( N, BSP, NT )
%
%   INPUTS:
%   
%   N    - degree of the multivariate polynomial
%   h - coefficients of the polynomial
%   NT   - number of trigonometric variables of the pol.
%
%   OUTPUT:
%
%   MU - minimum value
%   
%   The problem is
%
%   mu* = max  mu
%           
%         s.t. H(z,t) - mu >= 0.
%      
%      
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 5.04.09-17.11.11

% number of coefficients
nr = length( n ) - nt;

% AsP matrix
AsP = unitpol( n, 1, nt, nr );

% the first polynomial is dependent
% on nt trig. variables and 
% length(n)-nt real variables
KsP.ptype{ 1 }.trigonometric = nt;
KsP.ptype{ 1 }.real = nr;

% the degree of the polynomial
% is n1-1, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];     

% use POS3POLY
mu = pos3poly( AsP, 1, h, KsP );