function test_p3p_example( nr )

% TEST_P3P_EXAMPLE Test POS3POLY examples
%
%   This function gives an example of usage for each function
%   in the 'examples' directory.
%
%   TEST_P3P_EXAMPLE( NR ) tests the function with the number NR.
%
%   Associated numbers to the functions:
%
%    (1) min_poly_value.m          
%    (2) min_poly_value_general_trig.m    
%    (3) min_poly_value_general_trig_v2.m  
%    (4) min_poly_value_general_real_2_7.m          
%    (5) nearest_autocorrelation.m                              
%    (6) min_matrix_poly_eigen_general_trig.m                      
%    (7) min_matrix_poly_eigen_general_real.m
%    (8) lin_phase_fir.m  
%    (9) lin_phase_fir_v2.m 
%   (10) min_poly_value_multi_general_trig_3_5.m
%   (11) min_poly_value_multi_general_real.m
%   (12) min_matrix_poly_eigen_multi_general_trig.m
%   (13) min_matrix_poly_eigen_multi_general_real.m
%   (14) min_poly_value_multi_general_hybrid.m
%   (15) min_matrix_poly_eigen_multi_general_hybrid.m
%   (16) lin_phase_fir2d.m
%   (17) fir3d_lp.m
%   (18) dist2unstab_r.m
%   (19) dist2unstab_t.m
%   (20) adj_fir.m
%   (21) neutral_stab.m
%   (22) hinf_fir.m
%   (23) adj_fir_approx_lp.m
%   (24) approx_lin_phase_fir2d_5_2_3.m
%   (25) mimo_fir2d.m
%   (26) approx_lin_phase_fir.m
%   (27) mat_filt.m
%   (28) min_poly_value_multi_general_trig_cvx.m
%   (29) lin_phase_fir_5_1_1_cvx.m
%   (30) dist2unstab_r_cvx.m
%   (31) hinf_fir_cvx.m
%   (32) approx_lin_phase_fir_5_1_3_cvx.m
%   (33) mimo_fir2d_cvx.m
%   (34) mat_filt_cvx.m
%   (35) adj_fir_cvx.m
%   (36) adj_fir_v2.m
%   (37) lin_phase_fir2d_v2.m
%   (38) mimo_fir2d_cvx_v2.m
%   (39) lin_phase_fir2d_v3.m
%   (40) lin_phase_fir2d_v4.m
%   (41) lin_phase_fir2d_5_2_2_cvx.m
%   (42) lin_phase_fir2d_5_2_2_cvx_v2.m
%   (43) lin_phase_fir2d_5_2_2_cvx_v3.m
%   (44) approx_lin_phase_fir2d_5_2_3_cvx.m
%   (45) mimo_fir2d_cvx_v3.m
%   (46) min_poly_value_dual.m
%   (47) min_eig_trig.m
%   (48) lp_fir2d.m
%   (49) app_lp_fir2d.m
%   (50) min_eig_trig_cvx.m
%   (51) deconv2d.m
%   (52) min_poly_value_general_trig_dual.m
%   (53) min_poly_value_multi_general_hybrid_dual.m
%   (54) min_matrix_poly_eigen_general_real_dual.m
%   (55) min_matrix_poly_eigen_multi_general_trig_dual.m
%   (56) lin_phase_fir2d_dual.m
%   (57) lin_phase_fir2d_v2_dual.m
%   (58) lin_phase_fir2d_v3_dual.m
%   (59) dist2unstab_r_dual.m
%   (60) lin_phase_fir_dual.m
%
%   Example:
%
%   If one wants to test function "nearest_autocorrelation"
%   will type the following command:
%
%   test_sdmp_example( 5 );

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 23.12.08-16.11.11

switch nr
    case 1
        % minimu value of a particular trig. pol.
        [ mu ] = min_poly_value;
        fprintf( '\nMinimum value:\n' );
        disp( mu );        
    case 2
        % minimum value of a trig. pol.
        r = [ 6 -3 2 ];
        [ mu ] = min_poly_value_general_trig( r );
        fprintf( '\nMinimum value:\n' );
        disp( mu );
    case 3
        % second variant for the minimum of a trig. pol.
        r = [ 6 -3 2 ];
        [ mu ] = min_poly_value_general_trig_v2( r );
        fprintf( '\nMinimum value:\n' );
        disp( mu );
    case 4
        % minimum value of a real pol.
        p = [ 2 2 7 -2 1 ];
        [ mu ] = min_poly_value_general_real_2_7( p );
        fprintf( '\nMinimum value:\n' );
        disp( mu );
    case 5
        % find nearest autocorrelation
        
        % causal part of the 
        % given polynomial
        whr = [ 3 2 - 1i 1 + 1i ];
        r = nearest_autocorrelation( whr );
        fprintf( '\nCausal part of the nearest polynomial:\n' );
        disp( r );
    case 6
        % minimal value of the smallest eigenvalue
        % for a trig. pol. with matrix coefficients
        R( :, :, 1 ) = [ 1 0; 0 1 ];
        R( :, :, 2 ) = [ 0 2; 1 0 ];
        [ mu ] = min_matrix_poly_eigen_general_trig( R );
        fprintf( '\nMinimum eigenvalue:\n' );
        disp( mu );
    case 7
        % minimal value of the smallest eigenvalue
        % for a real pol. with matrix coefficients
        P( :, :, 1 ) = [ 5 0; 0 5 ];
        P( :, :, 2 ) = [ 0 1; 1 0 ];
        P( :, :, 3 ) = [ 2 0; 0 2 ];
        [ mu ] = min_matrix_poly_eigen_general_real( P );
        fprintf( '\nMinimum eigenvalue:\n' );
        disp( mu );
    case 8
        % design of linear phase filters,
        % optimizing the magnitude
        [ wth ] = lin_phase_fir( 25, 0.2 * pi, 0.25 * pi, 0.1, 0.0158 );
        fvtool( [ flipud( wth( 2 : end ) ); wth ] );
    case 9
        % design of linear phase filters,
        % optimizing the magnitude;
        % second variant
        [ wth ] = lin_phase_fir_v2( 25, 0.2 * pi, 0.25 * pi, 0.1, 0.0158 );
        fvtool( [ flipud( wth( 2 : end ) ); wth ] );
    case 10
        % minimum value of a multivariate trig. pol.;
        % R(z1,z2) = sym^{-1} + 38 + 18z1 + 4z1^2 + z1^{-2}z2 + 2z1^{-1}z2 + z2 - 8z1z2 - 5z1^2z2,
        % where sym^{-1} is the symmetric part of the polynomial
        r = [ 38 18 4 1 2 1 -8 -5 ];
        n = [ 2 1 ];
        [ mu ] = min_poly_value_multi_general_trig_3_5( n, r );
        fprintf( '\nMinimum value:\n' );
        disp( mu );
    case 11
        % minimum value of a multivariate real pol.;
        % P(t1,t2,t3) = 4t1^4 + 2t1^2 + 10t2^2 + 2t3^2 + 4t1^2t2 - 4t1^2t3 - 6t1t2 - 2t2t3 -
        %               -2t1t3 + 7

        % degree of the polynomial
        n = [ 4 2 2 ];
        % degrees of nonzero coefficients
        deg(   1, : ) = [ 4 0 0 ];
        deg(   2, : ) = [ 2 0 0 ];
        deg(   3, : ) = [ 0 2 0 ];
        deg(   4, : ) = [ 0 0 2 ];
        deg(   5, : ) = [ 2 1 0 ];
        deg(   6, : ) = [ 2 0 1 ];
        deg(   7, : ) = [ 1 1 0 ];
        deg(   8, : ) = [ 0 1 1 ];
        deg(   9, : ) = [ 1 0 1 ];
        deg(  10, : ) = [ 0 0 0 ];
        
        % nonzero coefficients of
        % the polynomial
        coef{  1 } =  4;
        coef{  2 } =  2;
        coef{  3 } = 10;
        coef{  4 } =  2;
        coef{  5 } =  4;
        coef{  6 } = -4;
        coef{  7 } = -6;
        coef{  8 } = -2;
        coef{  9 } = -2;
        coef{ 10 } =  7;     
        
        % build the 'bsP' term for the 'pos3poly' function
        % from the coefficients of the polynomial
        bsP = sppol2fvec( n, deg, coef, 0, 3 );
        
        [ mu ] = min_poly_value_multi_general_real( n, bsP );
        fprintf( '\nMinimum value:\n' );
        disp( mu );
    case 12
        % minimal value of the smallest eigenvalue for a
        % multivariate trig. pol. with matrix coefficients
        
        n = [ 1 1 ];
        deg( 1, : ) = [  0 0 ];
        deg( 2, : ) = [  1 0 ];
        deg( 3, : ) = [ -1 1 ];
        deg( 4, : ) = [  0 1 ];
        deg( 5, : ) = [  1 1 ];
        
        coef{ 1 } = [ 4 0; ...
                      0 4 ];
        coef{ 2 } = [ 1 0; ...
                      0 0 ];
        coef{ 3 } = [ 0 0;...
                      0 0 ];
        coef{ 4 } = [ 0 0;...
                      0 1 ];
        coef{ 5 } = [  0 0;...
                      -1 0 ];

        K   = length( coef{ 1 } );
        bsP = sppol2fvec( n, deg, coef, 2 ); 

        [ mu ] = min_matrix_poly_eigen_multi_general_trig( n, bsP, K );
        
        fprintf( '\nMinimum eigenvalue:\n' );
        disp( mu );
    case 13
        % minimal value of the smallest eigenvalue for a
        % multivariate real pol. with matrix coefficients
        
        n = [ 4 2 4 ];
        deg(  1, : ) = [ 0 0 0 ];
        deg(  2, : ) = [ 1 0 0 ];
        deg(  3, : ) = [ 2 0 0 ];
        deg(  4, : ) = [ 2 2 0 ];
        deg(  5, : ) = [ 0 2 2 ];
        deg(  6, : ) = [ 2 2 2 ];
        deg(  7, : ) = [ 4 0 0 ];
        deg(  8, : ) = [ 0 0 4 ];
        deg(  9, : ) = [ 0 1 0 ];
        deg( 10, : ) = [ 0 0 1 ];
        deg( 11, : ) = [ 0 2 0 ];
        deg( 12, : ) = [ 0 0 2 ];
        
        coef{  1 } = [ 3 0 0;...
                       0 3 0;...
                       0 0 3 ];
        coef{  2 } = [ 2 0 0;...
                       0 1 0;...
                       0 0 2 ];
        coef{  3 } = [ 3 0 1;...
                       0 5 0;...
                       1 0 2 ];
        coef{  4 } = [ 1 0 0;...
                       0 2 1;...
                       0 1 8 ];
        coef{  5 } = [ 1 0 1;...
                       0 5 0;...
                       1 0 7 ];
        coef{  6 } = [ 2 1 0;...
                       1 1 0;...
                       0 0 1 ];
        coef{  7 } = [ 1 0 0;...
                       0 1 0;...
                       0 0 1 ];
        coef{  8 } = [ 5 0 0;...
                       0 5 0;...
                       0 0 5 ];
        coef{  9 } = [ 1 0 0;...
                       0 0 0;...
                       0 0 2 ];
        coef{ 10 } = [ 1 0 0;...
                       0 1 0;...
                       0 0 1 ];
        coef{ 11 } = [ 1 0 0;...
                       0 1 0;...
                       0 0 1 ];
        coef{ 12 } = [ 4 0 0;...
                       0 2 0;...
                       0 0 4 ];
                
        K   = length( coef{ 1 } );
        bsP = sppol2fvec( n, deg, coef, 0, 3 ); 
        
        [ mu ] = min_matrix_poly_eigen_multi_general_real( n, bsP, K );  
        
        fprintf( '\nMinimum eigenvalue:\n' );
        disp( mu );
    case 14
        % minimum value of a multivariate hybrid pol.;
        % H(z1,z2,t3,t4) = sym^{-1} + 4*z1^2*z2^4*t1^2*t2^2 + 8*t1^2*t2^2 + z1^2*z2^2*t2^2 + 
        %                  4*z2^2*t2^2 + 10*t2^2 + z1 + z1*z2 - z2 + 8;
        % sym^{-1} is the symmetric part

        % degree of the polynomial
        n = [ 2 4 2 2 ];
        % degrees of nonzero coefficients
        deg( 1, : ) = [ 2 4 2 2 ];
        deg( 2, : ) = [ 0 0 2 2 ];
        deg( 3, : ) = [ 2 2 0 2 ];
        deg( 4, : ) = [ 0 2 0 2 ];
        deg( 5, : ) = [ 0 0 0 2 ];
        deg( 6, : ) = [ 1 0 0 0 ];
        deg( 7, : ) = [ 1 1 0 0 ];
        deg( 8, : ) = [ 0 1 0 0 ];
        deg( 9, : ) = [ 0 0 0 0 ];
        
        % nonzero coefficients of
        % the polynomial
        coef{ 1 } =  4;
        coef{ 2 } =  8;
        coef{ 3 } =  1;
        coef{ 4 } =  4;
        coef{ 5 } = 10;
        coef{ 6 } =  1;
        coef{ 7 } =  1;
        coef{ 8 } = -1;
        coef{ 9 } =  8;
        
        % build the 'bsP' term for the 'pos3poly' function
        % from the coefficients of the polynomial
        bsP = sppol2fvec( n, deg, coef, 2, 2 );
        
        [ mu ] = min_poly_value_multi_general_hybrid( n, bsP, 2 );
        fprintf( '\nMinimum value:\n' );
        disp( mu );
    case 15
        % minimal value of the smallest eigenvalue for a
        % multivariate hybrid pol. with matrix coefficients
        
        n = [ 2 2 2 2 ];
        deg( 1, : ) = [ 2 2 2 2 ];
        deg( 2, : ) = [ 0 0 2 2 ];
        deg( 3, : ) = [ 2 0 0 2 ];
        deg( 4, : ) = [ 0 0 0 2 ];
        deg( 5, : ) = [ 1 0 0 0 ];
        deg( 6, : ) = [ 0 0 0 0 ];
        
        coef{ 1 } = [ 17 12; ...
                      12 20 ];
        coef{ 2 } = [ 34 24; ...
                      24 40 ];
        coef{ 3 } = [ 1 0 ;...
                      0 25 ];
        coef{ 4 } = [ 2 0 ;...
                      0 50 ];
        coef{ 5 } = [ 2 2;...
                      2 5 ];
        coef{ 6 } = [ 3 2;...
                      2 3 ];

        K   = length( coef{ 1 } );
        bsP = sppol2fvec( n, deg, coef, 2, 2 ); 

        [ mu ] = min_matrix_poly_eigen_multi_general_hybrid( n, bsP, K, 2 );
        
        fprintf( '\nMinimum eigenvalue:\n' );
        disp( mu );
    case 16
        % design of linear phase 2-D filters
        
        n = [ 7 7 ];
        [ h ] = lin_phase_fir2d( n, 0.1 );
        freqz2d( half2all2d( h, n ) );
    case 17
        % design of linear 3-D FIR fan filters
        
        % "half degree" of the polynomial;
        % ( degree of the trig. pol. )
        n  = [ 4 4 4 ];
        % passband error bound
        gp = 0.1;
        % passband fan shape
        c  = 0.7; 

        % filter design
        [ h ] = fir3d_lp( n, gp, c );
        % half coefficients to all coefficients
        H = half2all3d( h, n );
        % frequency response
        Hw = freqz3d( H);
        % frequency grid for w1, w2
        f1 = -1 : 1e-2 : 1;
        f2 = -1 : 1e-2 : 1;
        % plot on w3
        plotf3d2d( Hw, 0 : 0.1 : 1, f1, f2, 1e-2, 1, 0, '', 0 );
    case 18
        % distance to unstabilizability using real polynomials
        % (with pos. dom.)
        
        % matrices
        A = [ 1   1  1; ...
              0.1 3  5; ...
              0  -1 -1  ];
        B = [ 1; 0.1; 0 ];
        
        % degree
        n = [ 4 4 ];
        
        % distance
        sigma = dist2unstab_r( A, B, n );
        
        fprintf( '\nDistance to unstabilizability:\n' );
        disp( sigma );
    case 19
        % distance to unstabilizability using trigonometric polynomials
        % (with pos. dom.)
        
        % matrices
        A = [ 1   1  1; ...
              0.1 3  5; ...
              0  -1 -1  ];
        B = [ 1; 0.1; 0 ];
        
        % degree
        n = [ 1 2 ];
        
        % distance
        sigma = dist2unstab_t( A, B, n );
        
        fprintf( '\nDistance to unstabilizability:\n' );
        disp( sigma );
    case 20
        % design of adjustible FIR filters
        % (hybrid pol. with pos. dom. scalar coefs.)
        
        % degree of trigonometric variable
        n2 = 13;
        % degree of real variable
        K = 6;        
        % "hybrid" degree
        n = [ n2 K ];
        % passband bound
        gp = 0.01;
        % delta
        D = 0.25;
        
        tl = 0;
        tu = 0.56;
        t0 = ( tl + tu ) / 2;
        
        % adjustable filters
        G = adj_fir( n, gp, D, tl, tu, t0 );
        
        K = 6;
        H = cell( K + 1, 1 );
        
        l = 1;
        u = n2 + 1;
        Shift = 0;
        for k = 0 : K
            H{ k + 1 } = [ G( u + Shift : -1 : l + 1 + Shift );  G( l + Shift : u + Shift ) ];
            Shift = Shift + n2 + 1;
        end        
        
        % draw filters;
        % we draw filters t = tl:Step:tu
        Step = 0.04;
        t = tl;
        G = zeros( length( H{ 1 } ), 1 );        
        tau = t - t0;
        Ht = H;
        for k = 0 : K
            Ht{ k + 1 } = Ht{ k + 1 } * tau ^ k;
        end        
        for k = 0 : K
            G = G + Ht{ k + 1 };
        end
        % first filter
        freqz( G );
        hold on
        % the rest of the filters
        for t = tl + Step : Step : tu
            G = zeros( length( H{ 1 } ), 1 );
            tau = t - t0;
            Ht = H;
            for k = 0 : K
                Ht{ k + 1 } = Ht{ k + 1 } * tau ^ k;
            end
            
            for k = 0 : K
                G = G + Ht{ k + 1 };
            end
            freqz( G );
        end
        hold off
    case 21
        % delay-dependent stability analysis of
        % neutral systems 
        
        % matrices defining the systems
        A0 = [ -0.9  0.2  ; ...
                0.1 -0.9 ];
        A1 = [ -1.1 -0.2  ; ...
               -0.1 -1.1 ];
        B  = [ -0.2  0    ; ...
                0.2 -0.1 ];

        ovTau = 3;
        % orders of the Pade approximation
        m1 = 3;
        m2 = 4;
        m3 = 5;
        
        Step = 0.1;
        tau = Step : Step : ovTau;
        len = length( tau );
        lambda1 = zeros( len, 1 );
        lambda2 = zeros( len, 1 );
        lambda3 = zeros( len, 1 );
        lambda4 = zeros( len, 1 );
         
        for i = 1 : len
            lambda1( i ) = neutral_stab( A0, A1, B, tau( i ), m1 );
            lambda2( i ) = neutral_stab( A0, A1, B, tau( i ), m2 );
            lambda3( i ) = neutral_stab( A0, A1, B, tau( i ), m3 );
        end
        
        % test on a grid
        lambda4( 1 ) = check_datko_grid( A0, A1, B, tau( 1 ) );
        for i = 2 : len
            lambda4( i ) = min( check_datko_grid( A0, A1, B, tau( i ) ), lambda4( i - 1 ) );
        end

        semilogy( tau, lambda1, ':'  ); 
        hold on;
        semilogy( tau, lambda2, '-.' ); 
        semilogy( tau, lambda3, '--' ); 
        semilogy( tau, lambda4, '-'  ); 
        hold off;  
        
        set( get( gca, 'XLabel' ), 'String', '$\widetilde{\tau}$', ...
                                   'Interpreter', 'LaTeX',         ...
                                   'FontSize', 12 );                      
        set( get( gca, 'YLabel' ), 'String', '$\log{\lambda_{max}}$',...
                                   'Interpreter', 'LaTeX',           ...
                                   'FontSize', 12 );
        
        legend( '\lambda_1, m = 3',  '\lambda_1, m = 4', '\lambda_1, m = 5', '\lambda_m' );     
    case 22
        % H_inf norm for 2-D FIR filter;
        % (BRL example, scalar case, no pos. dom.)
        
        n = [ 3 3 ];
        
        deg = [ 3 0; ...
                0 3; ...
                2 1; ...
                1 2; ...
                0 2; ...
                0 1 ];
        
        coef = { 1, 1, 3, 3, 1, 1 };        
        
        h = sppol2fvec( [ 3 3 ], deg, coef, 0, 2 );        
        
        hin = hinf_fir( n, h );
        
        fprintf( '\nH_inf norm for the FIR filter:\n' );
        disp( hin );
    case 23
        % design of approximately linear-phase adjustible FIR filters
        % (BRL, hybrid pol. with pos. dom. scalar coefs.)
        
        % degree of trigonometric variable
        n2 = 26;
        % degree of real variable
        K = 8;        
        % "hybrid" degree
        n = [ n2 K ];
        % passband bound
        gp = 0.01;
        % delta
        D = 0.29;
        
        tl = 0.019;
        tu = 0.518;
        t0 = ( tl + tu ) / 2;
        
        % desired group delay
        tau = 10;
        
        % approx. linear-phase adjustable filters
        G = adj_fir_approx_lp( n, gp, D, tl, tu, t0, tau );
        
        K = K / 2;
        H = cell( K + 1, 1 );
        
        l = 1;
        u = n2 + 1;
        Shift = 0;
        for k = 0 : K
            H{ k + 1 } = G( l + Shift : u + Shift );
            Shift = Shift + n2 + 1;
        end        
        
        % draw filters;
        % we draw filters t = tl:Step:tu
        Step = 0.04;
        t = tl;
        G = zeros( length( H{ 1 } ), 1 );        
        tau = t - t0;
        Ht = H;
        for k = 0 : K
            Ht{ k + 1 } = Ht{ k + 1 } * tau ^ k;
        end        
        for k = 0 : K
            G = G + Ht{ k + 1 };
        end
        % first filter
        freqz( G );
        hold on
        % the rest of the filters
        for t = tl + Step : Step : tu
            G = zeros( length( H{ 1 } ), 1 );
            tau = t - t0;
            Ht = H;
            for k = 0 : K
                Ht{ k + 1 } = Ht{ k + 1 } * tau ^ k;
            end
            
            for k = 0 : K
                G = G + Ht{ k + 1 };
            end
            freqz( G );
        end
        hold off
        
        figure;
        
        % draw group delay;
        t = tl;
        G = zeros( length( H{ 1 } ), 1 );        
        tau = t - t0;
        Ht = H;
        for k = 0 : K
            Ht{ k + 1 } = Ht{ k + 1 } * tau ^ k;
        end        
        for k = 0 : K
            G = G + Ht{ k + 1 };
        end
        % first filter
        w = 0 : acos( t + D ) / 100 : acos( t + D );
        grpdelay( G, 1, w );
        hold on
        % the rest of the filters
        for t = tl + Step : Step : tu
            G = zeros( length( H{ 1 } ), 1 );
            tau = t - t0;
            Ht = H;
            for k = 0 : K
                Ht{ k + 1 } = Ht{ k + 1 } * tau ^ k;
            end
            
            for k = 0 : K
                G = G + Ht{ k + 1 };
            end
            w = 0 : acos( t + D ) / 100 : acos( t + D );
            grpdelay( G, 1, w );
        end
        hold off
    case 24
        % design of approximately linear phase 2-D filters
        % (BRL multi. trig. pol. with pos. dom.)
        
        n = [ 10 10 ];
        gp = 0.1;
        tau = [ 4 4 ];
        % the filter will be diamond low pass
        [ h ] = approx_lin_phase_fir2d_5_2_3( n, gp, tau );
        freqz2d( mat( h ) );
    case 25
        % design of 2-D FIR MIMO filters       
        % (BRL multi. trig. pol. with matrix coef.)
        
        % degree of trig. pol.
        n = [ 4 4 ];
        % passband error bound
        gp = 0.1;
        % K1 K2
        K = [ 2 2 ];
        % D(z)
        degD = [ 0 0 ];
        coefD{ 1 } = eye( 2 ); 
        % passband edge
        wp = 0.4 * pi;
        % stopband edge
        ws = 0.9 * pi;        
        
        % filter design
        H = mimo_fir2d( n, gp, K, degD, coefD, wp, ws );
        
        % plot filter 1-1 (input-output)
        freqz2d( mat( H( 1 : 4 : prod( n + 1 ) * prod( K ) ) ) );
    case 26
        % design of 1-D approximately linear-phase filters
        % (BRL uni. trig. pol. with pos. int.)
        
        % degree of trig. pol.
        n = 50;
        % passband edge
        wp = 0.2  * pi;
        % stopband edge
        ws = 0.25 * pi;
        % passband error bound
        gp = 0.1;
        % stopband error bound
        gs = 0.0158;
        % desired group delay
        tau = 22;
        % design filter
        h = approx_lin_phase_fir( n, wp, ws, gp, gs, tau );
        figure;
        % plot filter
        freqz( h );
        figure;
        % group delay
        grpdelay( h, 1, 0 : 0.01 : wp );
    case 27
        % matrix filter design
        % (BRL matrix coef. uni trig. pol. with pos. int.)
        
        % size
        N = 15;
        % passband edge
        wp = 0.2 * pi;
        % stopband edges
        ws1 =  0.3 * pi;
        ws2 = -0.1 * pi;
        
        H = mat_filt( N, wp, ws1, ws2 );        
        A = mat( H );
        
        w = -pi : 0.01 : pi;
        len = length( w );
        
        Hw = zeros( N, len );
        for i = 0 : N - 1
            Hw( i + 1, : ) = freqz( A( i + 1, : ) / sqrt( N ), 1, w );
        end
        Hwn = zeros( len,1);
        for i = 1 : len
          Hwn( i ) = norm( Hw( :, i ) );
        end
        plot( w / pi, mag2db( Hwn ) );
        xlabel( 'Normalized frequency ($\omega/\pi$)', 'Interpreter', 'Latex' );
        ylabel( '$\| A \psi(e^{-j\omega})\|/\sqrt{N}$ (dB)', 'Interpreter', 'Latex' );
        title( 'Power response of the matrix filter' );
        grid on;
    case 28
        % minimum value of a multivariate trig. pol.;
        % R(z1,z2) = sym^{-1} + 38 + 18z1 + 4z1^2 + z1^{-2}z2 + 2z1^{-1}z2 + z2 - 8z1z2 - 5z1^2z2,
        % where sym^{-1} is the symmetric part of the polynomial
        %
        % cvx style
        
        r = [ 38 18 4 1 2 1 -8 -5 ]';
        n = [ 2 1 ];
        [ m ] = min_poly_value_multi_general_trig_cvx( n, r );
        fprintf( '\nMinimum value:\n' );
        disp( m );
    case 29
        % design of linear phase filters,
        % optimizing the magnitude
        %
        % cvx style
        [ wth ] = lin_phase_fir_5_1_1_cvx( 25, 0.2 * pi, 0.25 * pi, 0.1, 0.0158 );
        fvtool( [ flipud( wth( 2 : end ) ); wth ] );
    case 30
        % distance to unstabilizability using real polynomials
        % (with pos. dom.)
        %
        % cvx style
        
        % matrices
        A = [ 1   1  1; ...
              0.1 3  5; ...
              0  -1 -1  ];
        B = [ 1; 0.1; 0 ];
        
        % degree
        n = [ 4 4 ];
        
        % distance
        sigma = dist2unstab_r_cvx( A, B, n );
        
        fprintf( '\nDistance to unstabilizability:\n' );
        disp( sigma );
    case 31
        % H_inf norm for 2-D FIR filter;
        % (BRL example, scalar case, no pos. dom.)
        %
        % cvx style
        
        n = [ 3 3 ];
        
        deg = [ 3 0; ...
                0 3; ...
                2 1; ...
                1 2; ...
                0 2; ...
                0 1 ];
        
        coef = { 1, 1, 3, 3, 1, 1 };        
        
        h = sppol2fvec( [ 3 3 ], deg, coef, 0, 2 );        
        
        hin = hinf_fir_cvx( n, h );
        
        fprintf( '\nH_inf norm for the FIR filter:\n' );
        disp( hin );
    case 32
        % design of 1-D approximately linear-phase filters
        % (BRL uni. trig. pol. with pos. int.)
        %
        % cvx style
        
        % degree of trig. pol.
        n = 50;
        % passband edge
        wp = 0.2  * pi;
        % stopband edge
        ws = 0.25 * pi;
        % passband error bound
        gp = 0.1;
        % stopband error bound
        gs = 0.0158;
        % desired group delay
        tau = 22;
        % design filter
        h = approx_lin_phase_fir_5_1_3_cvx( n, wp, ws, gp, gs, tau );
        figure;
        % plot filter
        freqz( h );
        figure;
        % group delay
        grpdelay( h, 1, 0 : 0.01 : wp );
    case 33
        % design of 2-D FIR MIMO filters       
        % (BRL multi. trig. pol. with matrix coef.)
        %
        % cvx style
        
        % degree of trig. pol.
        n = [ 4 4 ];
        % passband error bound
        gp = 0.1;
        % K1 K2
        K = [ 2 2 ];
        % D(z)
        degD = [ 2 2 ];
        coefD{ 1 } = eye( 2 ); 
        % passband edge
        wp = 0.4 * pi;
        % stopband edge
        ws = 0.9 * pi;        
        
        % filter design
        H = mimo_fir2d_cvx( n, gp, K, degD, coefD, wp, ws );
        
        % plot filter 1-1 (input-output)
        freqz2d( mat( H( 1 : 4 : prod( n + 1 ) * prod( K ) ) ) );   
    case 34
        % matrix filter design
        % (BRL matrix coef. uni trig. pol. with pos. int.)
        %
        % cvx style
        
        % size
        N = 15;
        % passband edge
        wp = 0.2 * pi;
        % stopband edges
        ws1 =  0.3 * pi;
        ws2 = -0.1 * pi;
        
        H = mat_filt_cvx( N, wp, ws1, ws2 );        
        A = mat( H );
        
        w = -pi : 0.01 : pi;
        len = length( w );
        
        Hw = zeros( N, len );
        for i = 0 : N - 1
            Hw( i + 1, : ) = freqz( A( i + 1, : ) / sqrt( N ), 1, w );
        end
        Hwn = zeros( len,1);
        for i = 1 : len
          Hwn( i ) = norm( Hw( :, i ) );
        end
        plot( w / pi, mag2db( Hwn ) );
        xlabel( 'Normalized frequency ($\omega/\pi$)', 'Interpreter', 'Latex' );
        ylabel( '$\| A \psi(e^{-j\omega})\|/\sqrt{N}$ (dB)', 'Interpreter', 'Latex' );
        title( 'Power response of the matrix filter' );
        grid on;
    case 35
        % design of adjustible FIR filters
        % (hybrid pol. with pos. dom. scalar coefs.)
        %
        % cvx style
        
        % degree of trigonometric variable
        n2 = 13;
        % degree of real variable
        K = 6;        
        % "hybrid" degree
        n = [ n2 K ];
        % passband bound
        gp = 0.01;
        % delta
        D = 0.25;
        
        tl = 0;
        tu = 0.56;
        t0 = ( tl + tu ) / 2;
        
        % adjustable filters
        G = adj_fir_cvx( n, gp, D, tl, tu, t0 );
        
        K = 6;
        H = cell( K + 1, 1 );
        
        l = 1;
        u = n2 + 1;
        Shift = 0;
        for k = 0 : K
            H{ k + 1 } = [ G( u + Shift : -1 : l + 1 + Shift );  G( l + Shift : u + Shift ) ];
            Shift = Shift + n2 + 1;
        end        
        
        % draw filters;
        % we draw filters t = tl:Step:tu
        Step = 0.04;
        t = tl;
        G = zeros( length( H{ 1 } ), 1 );        
        tau = t - t0;
        Ht = H;
        for k = 0 : K
            Ht{ k + 1 } = Ht{ k + 1 } * tau ^ k;
        end        
        for k = 0 : K
            G = G + Ht{ k + 1 };
        end
        % first filter
        [ HH, W ] = freqz( G );
        plot( W, 20 * log10( abs( HH ) ) );
        hold on
        % the rest of the filters
        for t = tl + Step : Step : tu
            G = zeros( length( H{ 1 } ), 1 );
            tau = t - t0;
            Ht = H;
            for k = 0 : K
                Ht{ k + 1 } = Ht{ k + 1 } * tau ^ k;
            end
            
            for k = 0 : K
                G = G + Ht{ k + 1 };
            end
            [ HH, W ] = freqz( G );
            plot( W, 20 * log10( abs( HH ) ) );
        end
        hold off
        xlim( [ 0 pi ] );
        xlabel( 'Frequency (\omega/\pi)', 'Interpreter', 'tex', 'FontSize', 14 );
        ylabel( 'Magnitude (dB)', 'Interpreter', 'tex', 'FontSize', 14 );
    case 36
        % design of adjustible FIR filters
        % (hybrid pol. with pos. dom. scalar coefs.)
        %
        % example to show pos. dom. without 'nc';
        % full description for the pol. from the pos. dom.
        
        % degree of trigonometric variable
        n2 = 13;
        % degree of real variable
        K = 6;        
        % "hybrid" degree
        n = [ n2 K ];
        % passband bound
        gp = 0.01;
        % delta
        D = 0.25;
        
        tl = 0;
        tu = 0.56;
        t0 = ( tl + tu ) / 2;
        
        % adjustable filters
        G = adj_fir_v2( n, gp, D, tl, tu, t0 );
        
        K = 6;
        H = cell( K + 1, 1 );
        
        l = 1;
        u = n2 + 1;
        Shift = 0;
        for k = 0 : K
            H{ k + 1 } = [ G( u + Shift : -1 : l + 1 + Shift );  G( l + Shift : u + Shift ) ];
            Shift = Shift + n2 + 1;
        end        
        
        % draw filters;
        % we draw filters t = tl:Step:tu
        Step = 0.04;
        t = tl;
        G = zeros( length( H{ 1 } ), 1 );        
        tau = t - t0;
        Ht = H;
        for k = 0 : K
            Ht{ k + 1 } = Ht{ k + 1 } * tau ^ k;
        end        
        for k = 0 : K
            G = G + Ht{ k + 1 };
        end
        % first filter
        freqz( G );
        hold on
        % the rest of the filters
        for t = tl + Step : Step : tu
            G = zeros( length( H{ 1 } ), 1 );
            tau = t - t0;
            Ht = H;
            for k = 0 : K
                Ht{ k + 1 } = Ht{ k + 1 } * tau ^ k;
            end
            
            for k = 0 : K
                G = G + Ht{ k + 1 };
            end
            freqz( G );
        end
        hold off
    case 37
        % design of linear phase 2-D filters
        %
        % example to show pos. dom. without 'nc';
        % full description for the pol. from the pos. dom.
        
        n = [ 7 7 ];
        [ h ] = lin_phase_fir2d_v2( n, 0.1 );
        freqz2d( half2all2d( h, n ) );
    case 38
        % design of 2-D FIR MIMO filters       
        % (BRL multi. trig. pol. with matrix coef.)
        %
        % cvx style
        %
        % and
        %
        % example to show pos. dom. without 'nc';
        % full description for the pol. from the pos. dom.
        
        % degree of trig. pol.
        n = [ 4 4 ];
        % passband error bound
        gp = 0.1;
        % K1 K2
        K = [ 2 2 ];
        % D(z)
        degD = [ 2 2 ];
        coefD{ 1 } = eye( 2 ); 
        % passband edge
        wp = 0.4 * pi;
        % stopband edge
        ws = 0.9 * pi;        
        
        % filter design
        H = mimo_fir2d_cvx_v2( n, gp, K, degD, coefD, wp, ws );
        
        % plot filter 1-1 (input-output)
        freqz2d( mat( H( 1 : 4 : prod( n + 1 ) * prod( K ) ) ) );   
    case 39
        % design of linear phase 2-D filters
        %
        % 'nunion' example; sparse pos. dom.
        
        n = [ 7 7 ];
        [ h ] = lin_phase_fir2d_v3( n, 0.1 );
        freqz2d( half2all2d( h, n ) );
    case 40
        % design of linear phase 2-D filters
        %
        % 'nunion' example; full pos. dom.
        
        n = [ 7 7 ];
        [ h ] = lin_phase_fir2d_v4( n, 0.1 );
        freqz2d( half2all2d( h, n ) );    
    case 41
        % design of linear phase 2-D filters
        %
        % cvx style
        
        n = [ 7 7 ];
        [ h ] = lin_phase_fir2d_5_2_2_cvx( n, 0.1 );
        freqz2d( half2all2d( h, n ) );
    case 42
        % design of linear phase 2-D filters
        %
        % cvx style and reunion with sparse pos. dom.
        
        n = [ 7 7 ];
        [ h ] = lin_phase_fir2d_5_2_2_cvx_v2( n, 0.1 );
        freqz2d( half2all2d( h, n ) );
    case 43
        % design of linear phase 2-D filters
        %
        % cvx style and reunion with full pos. dom.
        
        n = [ 7 7 ];
        [ h ] = lin_phase_fir2d_5_2_2_cvx_v3( n, 0.1 );
        freqz2d( half2all2d( h, n ) );
    case 44
        % design of approximately linear phase 2-D filters
        % (BRL multi. trig. pol. with pos. dom.)
        %
        % cvx style, 'nunion'
        
        n = [ 10 10 ];
        gp = 0.1;
        tau = [ 4 4 ];
        % the filter will be diamond low pass
        [ h ] = approx_lin_phase_fir2d_5_2_3_cvx( n, gp, tau );
        freqz2d( mat( h ) );
    case 45
        % design of 2-D FIR MIMO filters       
        % (BRL multi. trig. pol. with matrix coef.)
        %
        % cvx style, 'nunion'
        
        % degree of trig. pol.
        n = [ 4 4 ];
        % passband error bound
        gp = 0.1;
        % K1 K2
        K = [ 2 2 ];
        % D(z)
        degD = [ 2 2 ];
        coefD{ 1 } = eye( 2 ); 
        % passband edge
        wp = 0.4 * pi;
        % stopband edge
        ws = 0.9 * pi;        
        
        % filter design
        H = mimo_fir2d_cvx_v3( n, gp, K, degD, coefD, wp, ws );
        
        % plot filter 1-1 (input-output)
        freqz2d( mat( H( 1 : 4 : prod( n + 1 ) * prod( K ) ) ) );   
    case 46
        % minimu value of a particular trig. pol.
        %
        % dual style
        
        [ mu ] = min_poly_value_dual;
        fprintf( '\nMinimum value:\n' );
        disp( mu );   
    case 47
        % mininum eigenvalue of matrix trig. pol.
        %
        % dual style
        
        n = 1;
        K = 2;
        
        % r = [ 1 2 -1 3 1 -2 2 ]';
        
        r = zeropol( n, K );
        r( coefpos( n, 0, [], [], K ) ) = vecs( [ 1 2; 2 -1 ] );
        r( coefpos( n, 1, [], [], K ) ) =  vec( [ 3 -2; 1 2 ] );      
        
        mu = min_eig_trig( r, n, K );
        
        fprintf( '\nMinimum eigenvalue:\n' );
        disp( mu );   
   case 48
        % design of linear phase 2-D filters
        %
        % 'nunion' example; sparse pos. dom.
        %
        % dual style
        
        n = [ 7 7 ];
        [ h ] = lp_fir2d( n, 0.1 );
        freqz2d( half2all2d( h, n ) );
    case 49
        % design of approximately linear phase 2-D filters
        % (BRL multi. trig. pol. with pos. dom.)
        %
        % dual style
        
        n = [ 10 10 ];
        gp = 0.1;
        tau = [ 4 4 ];
        % the filter will be diamond low pass
        [ h ] = app_lp_fir2d( n, gp, tau );
        freqz2d( mat( h ) );
    case 50
        % minimum eigenvalue of matrix pol.
        %
        % cvx style
        %
        % dual style
        
        n = 2;
        K = 2;
        
        % r = zeropol( n, K );
        % r( coefpos( n, 0, [], [], K ) ) = vecs( [ 1 2; 2 -1 ] );
        % r( coefpos( n, 2, [], [], K ) ) =  vec( [ 3 -2; 1 2 ] );
        
        cR{1} = [ 1 2; 2 -1 ];
        cR{2} = [ 3 -2; 1 2 ];
        r = sppol2fvec( 2, [ 0; 2 ], cR );
        
        mu = min_eig_trig_cvx( r, n, K );
        fprintf( '\nMinimum eigenvalue:\n' );
        disp( mu );
    case 51
        % 2-D H_inf deconvolution
        %
        % cvx style
        
        cA{ 1 } = [ 7 2; 2 3];
        cA{ 2 } = [ 10 0; 0 -6];
        cA{ 3 } = [ 2 3; -1 -2];
        cA{ 4 } = [ 5 8; 2 4];
        dA = [ 0 0; 1 0; 0 1; 1 1 ];
        
        cB{ 1 } = eye( 2); dB = [ 2 2 ];
        
        K = [ 2 2 ];
        
        [ X{ 1 }, g( 1 ) ] = deconv2d( dA, cA, dB, cB, [ 1 1 ], K );        
        [ X{ 2 }, g( 2 ) ] = deconv2d( dA, cA, dB, cB, [ 2 2 ], K );        
        [ X{ 3 }, g( 3 ) ] = deconv2d( dA, cA, dB, cB, [ 3 3 ], K );
        
        fprintf( '\nGamma for n=[1 1]:%d\n', g( 1 ) );
        fprintf( '\nGamma for n=[2 2]:%d\n', g( 2 ) );
        fprintf( '\nGamma for n=[3 3]:%d\n', g( 3 ) );
    case 52
        % minimum value of a trig. pol.
        %
        %  dual style
        
        r = [ 6 -3 2 ]';
        [ mu ] = min_poly_value_general_trig_dual( r );
        fprintf( '\nMinimum value:\n' );
        disp( mu );
    case 53
        % minimum value of hybrid pol.
        %
        % dual style
        %
        % H(z) = 3t^2 + z + z^{-1}
        
        n = [ 1 2 ];
        h = zeropol( n, 1, 1, 1 );
        h( coefpos( n, [ 0 2 ], 1, 1 ) ) = 3;
        h( coefpos( n, [ 1 0 ], 1, 1 ) ) = 1;  
        
        [ mu ] = min_poly_value_multi_general_hybrid_dual( n, h, 1 );
        fprintf( '\nMinimum value:\n' );
        disp( mu );
    case 54
        % minimal value of the smallest eigenvalue
        % for a real pol. with matrix coefficients
        %
        % dual style
        
        P( :, :, 1 ) = [ 5 0; 0 5 ];
        P( :, :, 2 ) = [ 0 1; 1 0 ];
        P( :, :, 3 ) = [ 2 0; 0 2 ];
        [ mu ] = min_matrix_poly_eigen_general_real_dual( P );
        fprintf( '\nMinimum eigenvalue:\n' );
        disp( mu );
    case 55
        % minimal value of the smallest eigenvalue for a
        % multivariate trig. pol. with matrix coefficients
        % 
        % dual style
        
        n = [ 1 1 ];
        deg( 1, : ) = [  0 0 ];
        deg( 2, : ) = [  1 0 ];
        deg( 3, : ) = [ -1 1 ];
        deg( 4, : ) = [  0 1 ];
        deg( 5, : ) = [  1 1 ];
        
        coef{ 1 } = [ 4 0; ...
                      0 4 ];
        coef{ 2 } = [ 1 0; ...
                      0 0 ];
        coef{ 3 } = [ 0 0;...
                      0 0 ];
        coef{ 4 } = [ 0 0;...
                      0 1 ];
        coef{ 5 } = [  0 0;...
                      -1 0 ];

        K   = length( coef{ 1 } );
        csP = sppol2fvec( n, deg, coef, 2 ); 

        [ mu ] = min_matrix_poly_eigen_multi_general_trig_dual( n, csP, K );
        
        fprintf( '\nMinimum eigenvalue:\n' );
        disp( mu );
    case 56
        % design of linear phase 2-D filters
        %
        % dual style
        
        n = [ 7 7 ];
        [ h ] = lin_phase_fir2d_dual( n, 0.1 );
        freqz2d( half2all2d( h, n ) );
    case 57
        % design of linear phase 2-D filters
        %
        % example to show pos. dom. without 'nc';
        % full description for the pol. from the pos. dom.
        %
        % dual style
        
        n = [ 7 7 ];
        [ h ] = lin_phase_fir2d_v2_dual( n, 0.1 );
        freqz2d( half2all2d( h, n ) );
    case 58
        % design of linear phase 2-D filters
        %
        % 'nunion' example; sparse pos. dom.
        %
        % dual style
        
        n = [ 7 7 ];
        [ h ] = lin_phase_fir2d_v3_dual( n, 0.1 );
        freqz2d( half2all2d( h, n ) );
    case 59
        % distance to unstabilizability using real polynomials
        % (with pos. dom.)
        %
        % dual style
        
        % matrices
        A = [ 1   1  1; ...
              0.1 3  5; ...
              0  -1 -1  ];
        B = [ 1; 0.1; 0 ];
        
        % degree
        n = [ 4 4 ];
        
        % distance
        sigma = dist2unstab_r_dual( A, B, n );
        
        fprintf( '\nDistance to unstabilizability:\n' );
        disp( sigma );
    case 60
        % design of linear phase filters,
        % optimizing the magnitude
        %
        % dual style
        
        [ wth ] = lin_phase_fir_dual( 25, 0.2 * pi, 0.25 * pi, 0.1, 0.0158 );
        fvtool( [ flipud( wth( 2 : end ) ); wth ] );
end


function Min = check_datko_grid( A0, A1, B, ovTau )

% Test minimum eigenvalue of delay system on a grid.
% (Used in example 21.)

% Author: Bogdan C. Sicleru

% j
SR = 1i;

Nw   = 100;
Ntau = 100;

v = zeros(Nw,Ntau);
w = logspace( 0, 2, Nw );
tau = 0 : ovTau / ( Ntau - 1 ) : ovTau;
for iw = 1 : Nw
  for it = 1 : Ntau
    P = SR * w( iw ) * ( eye( size( A0 ) ) - B * exp( -SR * w( iw ) * tau( it ) ) ) - A0 - A1 * exp( -SR * w( iw ) * tau( it ) );
    v( iw, it ) = min( eig( P * P' ) );
  end
end

Min = min( min( v ) );