function v = all2half2d( H, n )

% ALL2HALF2D All to half 2-D
%
%   V = ALL2HALF2D( H, N ) returns a vector V with half the 
%   coefficients of a trigonometric polynomial, where  the
%   matrix H holds all the coefficients of a trigonometric
%   bivariate polynomial of degree N.
%
%   Exampple:
%
%   H = half2all2d( 1 : 13, [ 2 2 ]  );
%   v = all2half2d( H, [ 2 2 ] );
%
%   See also half2all2d.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 17.06.09-01.10.09

assert( length( n ) == 2, 'The length of the vector ''n'' must be 2.' );

% middle of the matrix
mid = n + 1;

% half the coefficients
v = zeros( nch( n ), 1 );

% add coef. for degree (0, 0)
v( 1 ) = H( mid( 1 ), mid( 2 ) );

% add the rest of the
% "symmetric" coefs.
i = 0;
j = 0;
for k = 2 : length( v )
    if j == n( 2 )
        % reset column
        j = -n( 2 );
        % advance with rows
        i = i + 1;
    else 
        % next column
        j = j + 1;
    end
    
    v( k ) = H( mid( 1 ) - i, mid( 2 ) + j );
end