function v = all2half3d( H, n )

% ALL2HALF3D All to half 3-D
%
%   V = ALL2HALF3D( H, N ) returns a vector V with half the 
%   coefficients of a trigonometric polynomial, where  the
%   matrix H holds all the coefficients of a trigonometric
%   3-D polynomial of degree N.
%
%   Exampple:
%
%   H = half2all3d( 1 : 14, [ 1 1 1 ] );
%   v = all2half3d( H, [ 1 1 1 ] );
%
%   See also half2all3d.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 07.08.09-01.10.09

assert( length( n ) == 3, 'The length of the vector ''n'' must be 3.' );

% middle of the matrix
mid = n + 1;

% size
Hsize = n + mid;

% half the coefficients
v = zeros( nch( n ), 1 );

% add coef. for degree (0, 0)
v( 1 ) = H( mid( 1 ), mid( 2 ), n( 3 ) + 1 );

% nr. elems. in halfspace [ n(1) n(2) ]
nch2 = nch( n( 1 : end - 1 ) );

% nr. elements for a degree 
% of the 3rd dimension
Hsize12 = Hsize( 1 ) * Hsize( 2 );

% add the rest of the
% "symmetric" coefs. for degree 0 
% of the 3rd dimension
i = 0;
j = 0;
k = n( 3 ) + 1;
for cnt = 2 : nch2
    if j == n( 2 )
        % reset column
        j = -n( 2 );
        % advance with rows
        i = i + 1;
    else
        % next column
        j = j + 1;
    end
    
     v( cnt )=H( mid( 1 ) - i, mid( 2 ) + j, k );
end

% add the rest of the "symmetric" coefs.
% for the rest of the degrees of the 3rd dim.
% of the 3rd dimension
for k = 1 : n( 3 )
    % reset columns and lines for another
    % degree of the 3rd dimension
    i = -n( 1 );
    j = -n( 2 );
    
    for idx = 1 : Hsize12
        
        % next element in 'v'
        cnt = cnt + 1;
        
        v( cnt )=H( mid( 1 ) - i, mid( 2 ) + j, mid( 3 ) + k );        
        
        if j == n( 2 )
            % reset column
            j = -n( 2 );
            % advance with rows
            i = i + 1;
        else
            % next column
            j = j + 1;
        end
    end
end