function deg = degmon( n, nt, nr )

% DEGMON Degrees for the monomials of a polynomial
%
%   DEG = DEGMON( N, NT, NR ) returns a matrix DEG which holds on its
%   lines the degrees of all the monomials of a polynomial of degree N.
%   NT and NR are the number of trigonometric and real variables of a
%   polynomial, respectively.
%
%   If NR is missing or empty, then NR=0 (trigonometric pol.).
%   If NT is missing or empty, then NT=1 (trigonometric univariate pol.).
%
%   See also lenpol.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 14.12.11

if nargin < 3
    % trig. pol.
    nr = 0;
    
    if nargin < 2
        % univariate trig. pol.
        nt = 1;
    else
        if isempty( nt )
            % univariate trig. pol.
            nt = 1;
        end
    end
else
    if isempty( nr )
        % trig. pol.
        nr = 0;
    end
    
    if isempty( nt )
        % univariate trig. pol.
        nt = 1;
    end
end

if nt && nr
    % hybrid polynomial
    
    deg = ntupleh( n, nchp( n, nt), nt );
elseif nt
    % trigonometric polynomial
    
    deg = ntuple( n, nch( n ), 1 );
else
    % real polynomial
    
    deg = ntuple( n, prod( n + 1 ), 0 );
end