function [ Hw, f1, f2, f3, L1, L2, L3 ] = freqz3d( B, A, Eps1, Eps2, Eps3 )

% FREQZ3D 3-D Frequency response
%
%   [ HW, F1, F2, F3, L1, L2, L3 ] = FREQZ3D( B, A, EPS1, EPS2, EPS3 ) 
%   returns the frequency response HW of the function B/A with EPS1, 
%   EPS2, EPS3 being the accuracy on the three axes. F1, F2, F3 are the 
%   frequencies in 3-D and L1, L2, L3 are the number of points considered 
%   for the frequencies. EPS1, EPS2, EPS3 are optional.
%
%   [ ... ] = FREQZ3D( B ) sets A the identity polynomial.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date:   29.05.09-24.09.09

% checks
if nargin < 3
  Eps1 = 1e-2;
  Eps2 = 1e-2;
  Eps3 = 1e-2;
  if nargin == 1
      A = 1;
  end
end

% j
SR = 1i;

% frequencies
f1 = -1 : Eps1 : 1;
f2 = -1 : Eps2 : 1;
f3 = -1 : Eps3 : 1;

% number of frequencies
L1 = length( f1 );
L2 = length( f2 );
L3 = length( f3 );

Bw = polyval3d( B, exp( -SR * pi * f1 ), exp( -SR * pi * f2 ), exp( -SR * pi * f3 ) );
Aw = polyval3d( A, exp( -SR * pi * f1 ), exp( -SR * pi * f2 ), exp( -SR * pi * f3 ) );

% frequency response
Hw = Bw ./ Aw;