function H = half2all2d( v, n )

% HALF2ALL2D Half to all 2-D
%
%   H = HALF2ALL2D( V, N ) returns a matrix H with all the 
%   coefficients of a trigonometric polynomial, where  V
%   are the coefficients belonging to a halfspace and N is
%   a vector with the degree of the 2-D polynomial.
%
%   Exampple:
%
%   H = half2all2d( 1 : 13, [ 2 2 ]  );
%
%   See also all2half2d.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 17.06.09-01.10.09

assert( length( n ) == 2, 'The length of the vector ''n'' must be 2.' );

% middle of the matrix
mid = n + 1;

% size
Hsize = n + mid;

% all the coefficients
H = zeros( Hsize( 1 ), Hsize( 2 ) );

% add coef. for degree (0, 0)
H( mid( 1 ), mid( 2 ) ) = v( 1 );

% add the rest of the
% "symmetric" coefs.
i = 0;
j = 0;
for k = 2 : length( v )
    if j == n( 2 )
        % reset column
        j = -n( 2 );
        % advance with rows
        i = i + 1;
    else 
        % next column
        j = j + 1;
    end
    
    H( mid( 1 ) - i, mid( 2 ) + j ) = v( k ) ;
    H( mid( 1 ) + i, mid( 2 ) - j ) = v( k )';
end