function i = isinhalf( k )

% ISINHALF Is in halfspace
%
%   I = ISINHALF( K ) returns 1 if the vector K belongs to the
%   considered halfspace and 0 if the vector K doesn't belong
%   to the halfspace.
%
%   A vector K = ( k1, ... , kd ) belongs to a halfspace Hd if
%   ( kd > 0 ) || ( kd == 0 && ( (k1, ..., k_{d-1}) in H_{d-1} ). 

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 17.05.09

if k( end ) > 0
    i = 1;
elseif k( end ) == 0
    if length( k ) > 1
        i = isinhalf( k( 1 : end - 1 ) );
    else
        i = 1;
    end
else
    i = 0;
end