function NR = lenpol( n, K, nt, nr )

% LENPOL Length of a polynomial
%
%   NR = LENPOL( N, K, NT, NR ) returns the length of a polynomial of
%   degree N with coefficient size K. The type of the polynomial
%   is given by the number of trigonometric and real variables
%   NT and NR, respectively. 
%
%   If NR is missing, then NR=0. If NT is missing also, then the 
%   polynomial is trigonometric and the number of variables is taken 
%   from the degree. If K is also missing, then K=1.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 27.09.11-03.10.11

% number of variables
d = length( n );

if nargin < 4
    % trigonometric variables are default  
    nr = 0;
    if nargin < 3
        % degree has the number of variables
        nt = d;
        if nargin < 2
            % scalar polynomials are default
            K = 1;
        end
    end
end

% check number of variables
if d ~= nt + nr
    error( 'Degree does not match the number of variables.' );
end

% insert trig. var. if they exist
if nt
    ptype.trigonometric = nt;
end

% insert real var. if they exist
if nr
    ptype.real = nr;
end

% total number of coefficients
NR = ncoefpol( ptype, n, n, K, nt, nr, d );