function X = mat( v, m, n )

% MAT Matrix
%
%   This function recreates an MxN matrix X from a
%   vector V.
%
%   X = MAT( V ) returns a square matrix from V.
%
%   X = MAT( V, M ) returns an MxM square matrix
%   from V. The same as X = MAT( V ), with the only
%   advantage that the matrix dimensions are known.
%
%   X = MAT( V, M, N ) returns an MxN matrix from V.
%
%   See also vec.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 29.10.08-28.03.09

% nr. of input
% parameters
ni = nargin;

% one input param.
if ni == 1
   m = sqrt( length( v ) );
   
   % check if 'm' is square
   if m ~= fix( m )
       error( 'The length of ''v'' must be square.' );
   end
   n = m;
end

% two input param.
if ni == 2
   n = m;        
end

% create matrix X
X = reshape( v, m, n );