function nTuple = ntuple( n, nProd, isT )

% NTUPLE N-tuple for a trigonometric/real polynomial
%
%   This function returns all the n-tuples describing the coefficients
%   of a trigonometric or real polynomial.
%
%   NTUPLE = NTUPLE( N, NPROD, IST ) returns in each line of NTUPLE 
%   the n-tuples of a trigonometric (IST=1) or real (IST=0) polynomial
%   of degree N. NPROD is the number of n-tuples.
%
%   If IST = 0, then NPROD = prod( N + 1 ).
%   If IST = 1, then NPROD = nch( N ).
%
%   See also nch.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 18.02.09-18.02.10

% number of variables
nLen = length( n );

% adding a number,
% not to "get out" 
% of its size
n = [ n 2 ];

% number of variables + 1
nLen1 = nLen + 1;

% vector to be added at
% in the map
line  = zeros( 1,nLen1 );

% matrix that holds on its lines the n-tuples;
% nProd is the number of n-tuples;
% nProd = prod(n(1:end-1)+1), real pol.
% nProd = (1+prod(1+2*n(1:end-1))/2, trig. pol.
nTuple = zeros( nProd, nLen );

% current line
nLine = 1;

if isT
    % trigonometric polynomial
    
    % search for n-tuples
    while 1
        
        % add n-tuple
        nTuple( nLine, 1 : nLen ) = line( 1 : end - 1 );
        
        % reset to the
        % first variable
        j = 1;
        
        % while we have achieved
        % the maximum degree
        while line( j ) == n( j )
            % reset degree
            line( j ) = -n( j );
            % advance to the
            % next variable
            j = j + 1;
        end
        
        % check if we are at
        % the last variable
        if j == nLen1
            % done
            break;
        else
            % increase degree for
            % the j-th variable
            line( j ) = line( j ) + 1;
        end
        
        % next line
        nLine = nLine + 1;
    end
else
    % real polynomial
    
    % search for n-tuples
    while 1
        
        % add n-tuple
        nTuple( nLine, 1 : nLen ) = line( 1 : end - 1 );
        
        % reset to the
        % first variable
        j = 1;
        
        % while we have achieved
        % the maximum degree
        while line( j ) == n( j )
            % reset degree
            line( j ) = 0;
            % advance to the
            % next variable
            j = j + 1;
        end
        
        % check if we are at
        % the last variable
        if j == nLen1
            % done
            break;
        else
            % increase degree for
            % the j-th variable
            line( j ) = line( j ) + 1;
        end
        
        % next line
        nLine = nLine + 1;
    end
end

% sparse
nTuple = sparse( nTuple );