function nTuple = ntupleh( n, nProd, nt )

% NTUPLEH N-tuples for a hybrid polynomial
%
%   This function returns all the n-tuples describing the coefficients
%   of a hybrid polynomial.
%
%   NTUPLE = NTUPLEH( N, NPROD, NT ) returns in each line of NTUPLE 
%   the n-tuples of a hybrid polynomial of degree N. NT is the number of
%   trigonometric variables. 
%
%   NPROD is the number of n-tuples: NCHP( N, NT).
%
%   Example:
%
%   nTuple = ntupleh( [ 2 3 2 2 ], nchp( [ 2 3 2 2 ], 2 ), 2 );
%
%   See also nchp, ntuple, nch.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 23.04.09-18.02.10

% number of variables
nLen = length( n );

% number of coefficients for
% the trigonometric part
NTt = nch( n( 1 : nt ) );
% number of coefficients for
% the real part
NTr = prod( n( nt + 1 : end ) + 1 );

% n-tuples for the trig. part
nTuplet = ntuple( n( 1 : nt ), NTt, 1 );
% n-tuples for the real part
nTupler = ntuple( n( nt + 1 : end ), NTr, 0 );

% matrix that holds on its lines the n-tuples;
% nProd is the number of n-tuples for the 
% hybrid polynomial
nTuple = zeros( nProd, nLen );

% the n-tuples for the
% hybrid polynomial
for i = 1 : NTr
    nTuple( ( i - 1 ) * NTt + 1 : i * NTt, : ) = [ nTuplet repmat( nTupler( i, : ), NTt, 1 ) ]; 
end

% sparse
nTuple = sparse( nTuple );