function plotf3d2d( H, f, f1, f2, Eps, dB, Save, Text, Sub )

% PLOTF3D2D Plot 2-D filter from 3-D filter
%
%   H    - 3-D Filter
%   f    - vector with frequencies for the 3rd dim.
%   f1   - frequencies for the 1st dim.
%   f2   - frequencies for the 2nd dim.
%   Eps  - used for f1 & f2
%   dB   - representation in decibels (1-yes/0-no)
%   Save - save figure (1-yes/0-no)
%   Text - some text for the filename
%   Sub  - subplot (1-yes/0-no)
%
%   Example:
%
%   plotf3d2d( H,[ 0 : 0.1 : 1 ], f1, f2, 1e-2 );

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date:   04.06.09-13.09.09

nf = size( H, 1 );

% half of the frequencies
mid = floor( nf / 2 );

if ~Sub
    % check for representation
    % in decibels
    if dB
        for i = 1 : length( f )
            % figures
            figure
            x = f( i ) / Eps + mid;
            mesh( f1, f2, 20 * log10( abs( H( :, 1 + x * nf : ( x + 1 ) * nf ) ) ) );
            set( get( gca, 'XLabel' ), 'String', 'Frequency (\omega_1/\pi)' );
            set( get( gca, 'YLabel' ), 'String', 'Frequency (\omega_2/\pi)' );
            set( get( gca, 'ZLabel' ), 'String', 'Magnitude (dB)'       );
            if Save
                FileName = [ Text '_' num2str( f( i ) ) '.jpg' ];
                print( '-djpeg', FileName );
            end
        end
    else
        for i = 1 : length( f )
            % figures
            figure
            x = f( i ) / Eps + mid;
            mesh( f1, f2, abs( H( :, 1 + x * nf : ( x + 1 ) * nf ) ) );
            set( get( gca, 'XLabel' ), 'String', 'Frequency (\omega_1/\pi)' );
            set( get( gca, 'YLabel' ), 'String', 'Frequency (\omega_2/\pi)' );
            set( get( gca, 'ZLabel' ), 'String', 'Magnitude (dB)'       );
            if Save
                FileName = [ Text '_' num2str( f( i ) ) '.jpg' ];
                print( '-djpeg', FileName );
            end
        end
    end
else
    % subplot
    L   = length( f );
    sL1 = floor( sqrt( L ) ) + 1;
    % check for representation
    % in decibels
    if dB
        for i = 1 : length( f )
            
            subplot( sL1, sL1, i );
            
            x = f( i ) / Eps + mid;
            mesh( f1, f2, 20 * log10( abs( H( :, 1 + x * nf : ( x + 1 ) * nf ) ) ) );
            set( get( gca, 'XLabel' ), 'String', 'Frequency (\omega_1/\pi)' );
            set( get( gca, 'YLabel' ), 'String', 'Frequency (\omega_2/\pi)' );
            set( get( gca, 'ZLabel' ), 'String', 'Magnitude (dB)'       );
        end
    else
        for i = 1 : length( f )
            
            subplot( sL1, sL1, i );
            
            x = f( i ) / Eps + mid;
            mesh( f1, f2, abs( H( :, 1 + x * nf : ( x + 1 ) * nf ) ) );
            set( get( gca, 'XLabel' ), 'String', 'Frequency (\omega_1/\pi)' );
            set( get( gca, 'YLabel' ), 'String', 'Frequency (\omega_2/\pi)' );
            set( get( gca, 'ZLabel' ), 'String', 'Magnitude (dB)'       );
        end
    end
    if Save
        FileName = [ Text '_' 'sub' num2str( f( 1 ) ) num2str( f( L ) ) '.jpg' ];
        print( '-djpeg', FileName );
    end
end