function [ degr, coefr ] = pol2exp( deg, coef, type, r, ke, nt )

% POL2EXP Polynomial to exponent
%
%   [ DEGR, COEFR ] = POL2EXP( DEG, COEF, TYPE, R, KE, NT ) raises
%   the polynomial defined by DEG, COEF and TYPE to the power of R,
%   R >= 2. The optional parameter KE keps the zero coefficients in 
%   the resulting polynomial. The parameter NT is used for hybrid
%   polynomials to denote the number of trigonometric variables.
%   The resulting polynomial is returned in DEGR and COEFR.
%
%   See also prod_poly, sum_poly.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 27.09.09

% check that r >= 2
assert( r >= 2, 'The variable ''r'' must be greater than or equal to 2.' );

% the resulting polynomial
degr  = deg;
coefr = coef;

if nargin < 5
    for i = 1 : r - 1
        [ degr, coefr ] = prod_poly( degr, deg, coefr, coef, type );
    end
else
    if nargin == 5
        for i = 1 : r - 1
            [ degr, coefr ] = prod_poly( degr, deg, coefr, coef, type, ke );
        end
    elseif nargin == 6
        for i = 1 : r - 1
            [ degr, coefr ] = prod_poly( degr, deg, coefr, coef, type, ke, nt );
        end
    else
        error( 'Wrong number of input arguments.' );
    end
end