function H = polyval2d( A, x1, x2 )

% POLYVAL2D Value of 2-D polynomial
%
%   H = POLYVAL2D( A, X1, X2 ) returns in the matrix H the value
%   of a 2-D polinomial retained in the matrix A, in the pairs defined
%   in vectors/matrices X1 and X2.
%
%   If X1 and X2 are vectors then H is a matrix with the values of A
%   in each pair of the "cartesian product" of X1 and X2.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 19.02.02-24.09.09

% dimensionas for A
[ n1, n2 ] = size( A );

% column vectors
x1 = x1( : );
x2 = x2( : );

X1 = repmat(x1.',n1,1);
X1( 1, : ) = ones( 1, length( x1 ) );
X1 = cumprod( X1 );

X2 = repmat( x2.', n2, 1 );
X2( 1, : ) = ones( 1, length( x2 ) );
X2 = cumprod( X2 );

% values
H = X1.' * A * X2;