function s = rind2sub( x, n )

% RIND2SUB Index to subscripts
%
%   S = RIND2SUB( X, N ) transforms the index X in the subscripts S
%   of a real polynomial with maximum degree N.
%
%   Example:
%
%   s = rind2sub( 18, [ 4 5 ] );
%
%   See also gsub2ind, gsub2indh, ind2sub, sub2ind.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 14.01.09-08.06.10

% number of dimensions
d = length( n );  
% subscripts 
s = zeros(1,d);
% cum. prod. 
Cp = cumprod( n + 1 );
% idices start from 1
x = x - 1;              
for i = d - 1 : -1 : 1
  s( i + 1 ) = floor( x / Cp( i ) );
  x = x - s( i + 1 ) * Cp( i );
end
s( 1 ) = x;