function [ i, j ] = sind2sub( n, x )

% SIND2SUB Subscript from linear index for a symmetric matrix
%
%   This function returns the subscript equivalent for an
%   index in a symmetric matrix. We consider only half of the
%   symmetric matrix.
%
%   [ I, J ] = SIND2SUB( N, X ) returns in I and J the equivalent
%   row and column, respectively, for the linear index X, in a 
%   square symmetric NxN matrix.
%
%   Example:
%   
%   N = 7, X = 24
%   
%   1
%   2  8
%   3  9 14
%   4 10 15 19
%   5 11 16 20 23
%   6 12 17 21 24 26
%   7 13 18 22 25 27 28
%
%   >> [ i, j ] = sind2sub( 7, 24 );
%
%   >> i, j
%   i =
%        6
%   j =
%        5
%
%   See also ind2sub.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 10.11.08

% j^2-(2*n+1)+2*x=0;
b = 2 * n + 1;

% the column
j = ceil( ( b - sqrt( b ^ 2 - 8 * x ) ) / 2 );

% area of the 
% trapezoid
A = ( b - j ) * j / 2;

% the row
i = n - A + x;