function x = ssub2ind( n, i, j )

% SSUB2IND Linear index from subscript for a symmetric matrix
%
%   This function returns the linear index equivalent for an
%   subscript in a symmetric matrix. We consider only half of the
%   symmetric matrix.
%
%   X = SSUB2IND( N, I, J ) returns in the linear index X  for the 
%   equivalent row I and column J in a square symmetric NxN matrix.
%
%   Example:
%   
%   N = 7, I = 6, J = 5
%   
%   1
%   2  8
%   3  9 14
%   4 10 15 19
%   5 11 16 20 23
%   6 12 17 21 24 26
%   7 13 18 22 25 27 28
%
%   >> x = sind2sub( 7, 6, 5 );
%
%   >> x
%   x =
%        24
%
%   See also sub2ind.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 24.10.11

% the linear index
x = 0;

OK = 0;

for jj = 1 : n
    for ii = jj : n
        x = x + 1;
        if ii == i && jj == j
            OK = 1;
            break;
        end
    end
    if OK
        break;
    end
end