function v = unitcpol( n, K )

% UNITCPOL Constant unit causal polynomial
%
%   V = UNITCPOL( N, K ) creates a unitary causak polynomial of degree N 
%   with coefficient size K, where K is a two element vector. If K is 
%   missing, then K=1. If K is scalar, then K = [ K K ].

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 27.09.11-13.11.11

% number of variables
d = length( n );

if nargin < 2
    % scalar polynomials are default
    K = [ 1 1 ];
end

% if K is scalar, then K = [ K K ];
if isscalar( K )
    K = [ K K ];
end

% insert real var. for causal polynomial
ptype.real = d;
    
% compute nr. of coef. for scalar pol.
nCoefPol = ncoefpol( ptype, n, n, 1, 0, d, d );
if K( 1 ) > 1 || K( 2 ) > 1 
    % matrix polynomial case
    KK = prod( K );
    nCoefPol = nCoefPol * KK;
end

% the vector polynomial
v = sparse( nCoefPol, 1 );

% insert 1 or I
if K( 1 ) == 1 && K( 2 ) == 1
    v( 1 ) = 1;
else
    v( 1 : prod( K ) ) = vec( speye( K ) );
end