function [ v ] = vecrasm( M )

% VECRASM Vectorize real antisymmetric matrix
%
%   [ V ] = VECRASM( M ) returns a column vector V from
%   the strict lower part of the antisymmetric matrix M.
%
%   For complex matrices we consider the imaginary part.
%
%   See also vecras.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.10.08

% check if matrix M is real or complex
if isreal( M )
    % matrix is real
    
    % check if matrix is antisymmetric  
    if ( nnz( M ~= -M' ) ~= 0 )
        % matrix is not symmetric
        warning( 'MATLAB:SeDuMiPol:MatrixNotAntiSymm', ...
                 'Matrix M is not antisymmetric.' );
    end
    
    % return the strict lower part as a column
    v = vltri( M, -1 );
else
    % matrix is complex
    
    % imaginary part of the matrix
    ImagM = imag( M );
    
    % check if imaginary part of matrix 
    % is antisymmetric  
    if ( nnz( ImagM ~= -ImagM' ) ~= 0 )
        % imaginary part is not symmetric
        warning( 'MATLAB:SeDuMiPol:MatrixNotAntiSymm', ...
                 'Imaginary part of matrix M is not antisymmetric.' );
    end
    
    % return the strict lower part as a column
    v = vltri( ImagM, -1 );   
end