function [ v ] = vecrsm( M )

% VECRSM Vectorize real symmetric matrix
%
%   [ V ] = VECRSM( M ) returns a column vector V 
%   from the lower part of the real symmetric matrix M.
%
%   For complex matrices we consider the real part.
%
%   See also vecrs.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.10.08

% check if matrix M is real or complex
if isreal( M )
    % matrix is real
    
    % check if matrix is symmetric  
    if ( nnz( M ~= M' ) ~= 0 )
        % matrix is not symmetric
        warning( 'MATLAB:SeDuMiPol:MatrixNotSymm', ...
                 'Matrix M is not symmetric.' );
    end
    
    % return the lower part as a column
    v = vltri( M );
else
    % matrix is complex
    
    % real part of the matrix
    RealM = real( M );
    
    % check if real part of matrix is symmetric  
    if ( nnz( RealM ~= RealM' ) ~= 0 )
        % real part is not symmetric
        warning( 'MATLAB:SeDuMiPol:MatrixNotSymm', ...
                 'Real part of matrix M is not symmetric.' );
    end
    
    % return the lower part as a column
    v = vltri( RealM );
end