function [ v ] = vecs2( M )

% VECS2 Vectorize symmetric (Hermitian), 2nd function
%
%   This function returns a column vector V from
%   the lower part of a real symmetric or complex
%   Hermitian matrix M, taking its columns.
%
%   V = VECS2( M ) returns a column vector V from a matrix M.
%
%   Example:
%
%   >> M
%   M =
%        1     2     8-2i  5           
%        2     5     0     1            
%        8+2i  0     5     9             
%        5     1     9     7 
%
%   >> v = vecs2( M );
%
%   >> v'
%
%        1 2 8+2i 5 5 0 1 5 9 7 
%
%   See also vecs, vec, vecg, sumn.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 17.04.10

% size vector for M
d = size( M );

% the first two elements of 'd'
% representing the matrix dimensions
d1 = d( 1 );
d2 = d( 2 );

% check if the first two elements of 'd'
% are equal (square matrix)
assert( d1 == d2, 'The first two elements of ''d'' must be equal (square matrix).' );

% check if M is a matrix
assert( length( d ) == 2, 'M must be a matrix.' );

Step = 0;

% number of elems. to add
% at each step
addE = d1;

for i = 1 : d1
    v( 1 + Step : addE + Step ) = M( i : d1, i );
    
    Step = Step + addE;
    addE = addE - 1;
end