function [ v ] = vecsm( M )

% VECSM Vectorize symmetric (Hermitian) matrix
%
%   This function returns a column vector V from
%   the lower part of the symmetric (Hermitian)
%   matrix M.
%
%   See also vecs, vec, vecg.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 14.11.08

% check if matrix is Hermitian    
if ( nnz( M ~= M' ) ~= 0 )
    % matrix is not Hermitian
    warning( 'MATLAB:SeDuMiPol:MatrixNotHerm', ...
             'Matrix M is not Hermitian.' );
end

% return the lower part 
% as a column
v = vltri( M );